###################### R script: Haberman Survival #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 4 of the paper.

################################################################################ 

# Set seed
set.seed(2)

# Set directories
dir.InferNET.support <- "..\\..\\InferNETSupport\\"
dir.Figures <- "..\\..\\..\\Figures\\"

# Load support files
source(paste(dir.InferNET.support, "InferNETSupport.R", sep = ""), chdir = TRUE)

# Set flags:
doMCMC <- TRUE
doInferNET <- TRUE
doPlots <- TRUE
REMOVE.INPUTS <- TRUE
REMOVE.OUTPUTS <- TRUE

# Number of trials to run example (for timing)
TRIALS <- 1

# Read in data into data frame named 'onions':
data <- read.csv("haberman.data", header = FALSE)
colnames(data) <- c("age", "operationYear", "nodesDetected", "died")
data[, 2] <- data[, 2] + 1900
data[, 4] <- data[, 4] - 1

x1Orig <- data[, 1]
x2Orig <- data[, 2]
x3Orig <- data[, 3]

yOrig <- data[, 4]

# Use transformed versions of data for inputting into Infer.NET:
x1.mean <- mean(x1Orig)
x1.std <- sd(x1Orig)
x1 <- (x1Orig - x1.mean)/x1.std

x2.mean <- mean(x2Orig)
x2.std <- sd(x2Orig)
x2 <- (x2Orig - x2.mean)/x2.std

x3.mean <- mean(x3Orig)
x3.std <- sd(x3Orig)
x3 <- (x3Orig - x3.mean)/x3.std

y <- yOrig

# Set up X, Z2 and C matrices:
n <- length(y)
X <- cbind(rep(1, n), x1, x2, x3)

numIntKnots <- 15
intKnots.x1 <- quantile(unique(x1), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x1 <- c(1.05 * min(x1) - 0.05 * max(x1), 1.05 * max(x1) - 0.05 * min(x1))
Z1 <- ZOSull(x1, range.x1, intKnots.x1)

numIntKnots <- 3
intKnots.x2 <- quantile(unique(x2), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x2 <- c(1.05 * min(x2) - 0.05 * max(x2), 1.05 * max(x2) - 0.05 * min(x2))
Z2 <- ZOSull(x2, range.x2, intKnots.x2)

numIntKnots <- 15
intKnots.x3 <- quantile(unique(x3), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x3 <- c(1.05 * min(x3) - 0.05 * max(x3), 1.05 * max(x3) - 0.05 * min(x3))
Z3 <- ZOSull(x3, range.x3, intKnots.x3)

Cmat <- cbind(X, Z1, Z2, Z3)
numSpl1 <- ncol(Z1)
numSpl2 <- ncol(Z2)
numSpl3 <- ncol(Z3)

################################################################################ 

if (doMCMC) {
  nBurnin <- 5000  # Length of burn-in.
  nIter <- 5000  # Size of the kept sample.
  nThin <- 1  # Thinning factor. 
  
  ###################### Specify logit model:
  
  habermanLogitModel <- function() {
    for (i in 1:numObs) {
      mu[i] <- beta0 + beta1 * x1[i] + beta2 * x2[i]
      +beta3 * x3[i] + inprod(uSpl1[], Zspl1[i, ]) + inprod(uSpl2[], Zspl2[i, 
        ]) + inprod(uSpl3[], Zspl3[i, ])
      p[i] <- 1/(1 + exp(-mu[i]))
      y[i] ~ dbern(p[i])
    }
    
    for (iSpl in 1:numSpl1) {
      uSpl1[iSpl] ~ dnorm(0, tauSpl1)
    }
    for (iSpl in 1:numSpl2) {
      uSpl2[iSpl] ~ dnorm(0, tauSpl2)
    }
    for (iSpl in 1:numSpl3) {
      uSpl3[iSpl] ~ dnorm(0, tauSpl3)
    }
    
    beta0 ~ dnorm(0, 1e-10)
    beta1 ~ dnorm(0, 1e-10)
    beta2 ~ dnorm(0, 1e-10)
    beta3 ~ dnorm(0, 1e-10)
    
    cSpl1inv ~ dgamma(0.5, 1e-10)
    tauSpl1 ~ dgamma(0.5, cSpl1inv)
    sigSpl1 <- 1/sqrt(tauSpl1)
    
    cSpl2inv ~ dgamma(0.5, 1e-10)
    tauSpl2 ~ dgamma(0.5, cSpl2inv)
    sigSpl2 <- 1/sqrt(tauSpl2)
    
    cSpl3inv ~ dgamma(0.5, 1e-10)
    tauSpl3 ~ dgamma(0.5, cSpl3inv)
    sigSpl3 <- 1/sqrt(tauSpl3)
  }
  
  fileName <- file.path(getwd(), "habermanLogitModel.txt")
  writeModel(habermanLogitModel, fileName)
  
  
  # Fit model using MCMC via BRugs:
  
  numObs <- nrow(Cmat)
  Zspl1 <- Z1
  Zspl2 <- Z2
  Zspl3 <- Z3
  numSpl1 <- numSpl1
  numSpl2 <- numSpl2
  numSpl3 <- numSpl3
  
  allData <- list(numObs = numObs, numSpl1 = numSpl1, numSpl2 = numSpl2, numSpl3 = numSpl3, 
    x1 = x1, x2 = x2, x3 = x3, y = y, Zspl1 = Zspl1, Zspl2 = Zspl2, Zspl3 = Zspl3)
  
  parInits <- list(list(beta0 = 0, beta1 = 0, beta2 = 0, beta3 = 0, uSpl1 = rep(0, 
    numSpl1), uSpl2 = rep(0, numSpl2), uSpl3 = rep(0, numSpl3), cSpl1inv = 1, 
    cSpl2inv = 1, cSpl3inv = 1, tauSpl1 = 1, tauSpl2 = 1, tauSpl3 = 1))
  
  
  times.brugs.logit <- c()
  for (trial in 1:TRIALS) {
    # Do MCMC using BRugs:
    bval1 <- proc.time()
    BRugsFit(data = allData, inits = parInits, parametersToSave = c("beta0", 
      "beta1", "beta2", "beta3", "uSpl1", "uSpl2", "uSpl3"), nBurnin = nBurnin, 
      nIter = nIter, nThin = nThin, modelFile = "habermanLogitModel.txt", numChains = 1)
    eval1 <- proc.time()
    times.brugs.logit[trial] <- eval1[3] - bval1[3]
  }
  
  sbeta0MCMC <- samplesSample("beta0")
  sbeta1MCMC <- samplesSample("beta1")
  sbeta2MCMC <- samplesSample("beta2")
  sbeta3MCMC <- samplesSample("beta3")
  
  suSplMCMC <- samplesSample("uSpl1[1]")
  for (k in 2:numSpl1) suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl1[", 
    k, "]", sep = "")))
  
  suSplMCMC <- rbind(suSplMCMC, samplesSample("uSpl2[1]"))
  for (k in 2:numSpl2) suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl2[", 
    k, "]", sep = "")))
  
  suSplMCMC <- rbind(suSplMCMC, samplesSample("uSpl3[1]"))
  for (k in 2:numSpl3) suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl3[", 
    k, "]", sep = "")))
  
  mu.q.betauMCMCLogit <- rbind(sbeta0MCMC, sbeta1MCMC, sbeta2MCMC, sbeta3MCMC, 
    suSplMCMC)
  
  
  ####################### Specify probit model:
  
  habermanProbitModel <- function() {
    for (i in 1:numObs) {
      p[i] <- phi(beta0 + beta1 * x1[i] + beta2 * x2[i] + beta3 * x3[i] + inprod(uSpl1[], 
        Zspl1[i, ]) + inprod(uSpl2[], Zspl2[i, ]) + inprod(uSpl3[], Zspl3[i, 
        ]))
      y[i] ~ dbern(p[i])
    }
    
    for (iSpl in 1:numSpl1) {
      uSpl1[iSpl] ~ dnorm(0, tauSpl1)
    }
    for (iSpl in 1:numSpl2) {
      uSpl2[iSpl] ~ dnorm(0, tauSpl2)
    }
    for (iSpl in 1:numSpl3) {
      uSpl3[iSpl] ~ dnorm(0, tauSpl3)
    }
    
    beta0 ~ dnorm(0, 1e-10)
    beta1 ~ dnorm(0, 1e-10)
    beta2 ~ dnorm(0, 1e-10)
    beta3 ~ dnorm(0, 1e-10)
    
    tauSpl1 ~ dgamma(2, 2)
    tauSpl2 ~ dgamma(2, 2)
    tauSpl3 ~ dgamma(2, 2)
  }
  
  fileName <- file.path(getwd(), "habermanProbitModel.txt")
  writeModel(habermanProbitModel, fileName)
  
  
  # Fit model using MCMC via BRugs:
  
  numObs <- nrow(Cmat)
  Zspl1 <- Z1
  Zspl2 <- Z2
  Zspl3 <- Z3
  numSpl1 <- numSpl1
  numSpl2 <- numSpl2
  numSpl3 <- numSpl3
  
  allData <- list(numObs = numObs, numSpl1 = numSpl1, numSpl2 = numSpl2, numSpl3 = numSpl3, 
    x1 = x1, x2 = x2, x3 = x3, y = y, Zspl1 = Zspl1, Zspl2 = Zspl2, Zspl3 = Zspl3)
  
  parInits <- list(list(beta0 = 0, beta1 = 0, beta2 = 0, beta3 = 0, uSpl1 = rep(0, 
    numSpl1), uSpl2 = rep(0, numSpl2), uSpl3 = rep(0, numSpl3), tauSpl1 = 1, 
    tauSpl2 = 1, tauSpl3 = 1))
  
  times.brugs.probit <- c()
  for (trial in 1:TRIALS) {
    # Do MCMC using BRugs:
    bval1 <- proc.time()
    BRugsFit(data = allData, inits = parInits, parametersToSave = c("beta0", 
      "beta1", "beta2", "beta3", "uSpl1", "uSpl2", "uSpl3"), nBurnin = nBurnin, 
      nIter = nIter, nThin = nThin, modelFile = "habermanProbitModel.txt", 
      numChains = 1)
    eval1 <- proc.time()
    times.brugs.probit[trial] <- eval1[3] - bval1[3]
  }
  
  sbeta0MCMC <- samplesSample("beta0")
  sbeta1MCMC <- samplesSample("beta1")
  sbeta2MCMC <- samplesSample("beta2")
  sbeta3MCMC <- samplesSample("beta3")
  
  suSplMCMC <- samplesSample("uSpl1[1]")
  for (k in 2:numSpl1) suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl1[", 
    k, "]", sep = "")))
  
  suSplMCMC <- rbind(suSplMCMC, samplesSample("uSpl2[1]"))
  for (k in 2:numSpl2) suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl2[", 
    k, "]", sep = "")))
  
  suSplMCMC <- rbind(suSplMCMC, samplesSample("uSpl3[1]"))
  for (k in 2:numSpl3) suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl3[", 
    k, "]", sep = "")))
  
  mu.q.betauMCMCProbit <- rbind(sbeta0MCMC, sbeta1MCMC, sbeta2MCMC, sbeta3MCMC, 
    suSplMCMC)
}

################################################################################ 

if (doInferNET) {
  nIterVB <- 100
  
  # Write input information to files:
  
  write(ncol(X), "ncX.txt")
  write(ncol(Z1), "numKnots1.txt")
  write(ncol(Z2), "numKnots2.txt")
  write(ncol(Z3), "numKnots3.txt")
  write.table(Cmat, "Cmat.txt", sep = ",", row.names = FALSE, col.names = FALSE)
  write.table(y, "y.txt", sep = ",", row.names = FALSE, col.names = FALSE)
  write(nIterVB, "nIterVB.txt")
  
  # Set hyperparameters and write to files:
  
  sigsqBeta <- 1e+08
  Au <- 1e+05
  write(sigsqBeta^(-1), "tauBeta.txt")
  write(Au, "Au.txt")
  
  
  # Run Infer.NET:
  times.Infer.NET.logit <- c()
  for (trial in 1:TRIALS) {
    # Run Infer.NET:
    timeTaken <- system.time(RunInferNET(dir.InferNET.support, "habermanLogit.cs"))
    times.Infer.NET.logit[trial] <- timeTaken[3]
  }
  
  # Read in posterior density parameters:
  
  mu.q.betauLogit <- scan("mu.q.betau.txt")
  Sigma.q.betauLogit <- as.matrix(read.table("Sigma.q.betau.txt"))
  
  ### Probit
  
  times.Infer.NET.probit <- c()
  for (trial in 1:TRIALS) {
    # Run Infer.NET:
    timeTaken <- system.time(RunInferNET(dir.InferNET.support, "habermanProbit.cs"))
    times.Infer.NET.probit[trial] <- timeTaken[3]
  }
  
  # Read in posterior density parameters:
  
  mu.q.betauProbit <- scan("mu.q.betau.txt")
  Sigma.q.betauProbit <- as.matrix(read.table("Sigma.q.betau.txt"))
}

################################################################################ 

if (doPlots) {
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 2.2
  cex.mainVal <- 2
  cex.legendVal <- 1.5
  legendStr1 <- "Infer.NET"
  legendStr2 <- "Infer.NET 95% credible sets"
  legendStr3 <- "MCMC"
  legendStr4 <- "MCMC 95% credible sets"
  
  xg1 <- (seq(min(x1Orig), max(x1Orig), length = ng) - x1.mean)/x1.std
  Z1g <- ZOSull(xg1, range.x1, intKnots.x1)
  Z2g <- ZOSull(rep(0, length = ng), range.x2, intKnots.x2)
  Z3g <- ZOSull(rep(0, length = ng), range.x3, intKnots.x3)
  CmatgX1 <- cbind(rep(1, ng), xg1, rep(0, ng), rep(0, ng), Z1g, Z2g, Z3g)
  
  xg2 <- (seq(min(x2Orig), max(x2Orig)) - x2.mean)/x2.std
  Z1g <- ZOSull(rep(0, length = length(xg2)), range.x1, intKnots.x1)
  Z2g <- ZOSull(xg2, range.x2, intKnots.x2)
  Z3g <- ZOSull(rep(0, length = length(xg2)), range.x3, intKnots.x3)
  CmatgX2 <- cbind(rep(1, length(xg2)), rep(0, length(xg2)), xg2, rep(0, length(xg2)), 
    Z1g, Z2g, Z3g)
  
  xg3 <- (seq(min(x3Orig), max(x3Orig), length = ng) - x3.mean)/x3.std
  Z1g <- ZOSull(rep(0, length = ng), range.x1, intKnots.x1)
  Z2g <- ZOSull(rep(0, length = ng), range.x2, intKnots.x2)
  Z3g <- ZOSull(xg3, range.x3, intKnots.x3)
  CmatgX3 <- cbind(rep(1, ng), rep(0, ng), rep(0, ng), xg3, Z1g, Z2g, Z3g)
  
  ### Logit
  
  # Compute logit results:
  
  fhatgX1MCMCLogit <- CmatgX1 %*% mu.q.betauMCMCLogit
  fhatgX2MCMCLogit <- CmatgX2 %*% mu.q.betauMCMCLogit
  fhatgX3MCMCLogit <- CmatgX3 %*% mu.q.betauMCMCLogit
  
  sdgX1 <- sqrt(diag(CmatgX1 %*% Sigma.q.betauLogit %*% t(CmatgX1)))
  fhatgX1VBLogit <- CmatgX1 %*% mu.q.betauLogit
  fhatgX1VBLogitLow <- fhatgX1VBLogit - 1.96 * sdgX1
  fhatgX1VBLogitUpp <- fhatgX1VBLogit + 1.96 * sdgX1
  
  sdgX2 <- sqrt(diag(CmatgX2 %*% Sigma.q.betauLogit %*% t(CmatgX2)))
  fhatgX2VBLogit <- CmatgX2 %*% mu.q.betauLogit
  fhatgX2VBLogitLow <- fhatgX2VBLogit - 1.96 * sdgX2
  fhatgX2VBLogitUpp <- fhatgX2VBLogit + 1.96 * sdgX2
  
  sdgX3 <- sqrt(diag(CmatgX3 %*% Sigma.q.betauLogit %*% t(CmatgX3)))
  fhatgX3VBLogit <- CmatgX3 %*% mu.q.betauLogit
  fhatgX3VBLogitLow <- fhatgX3VBLogit - 1.96 * sdgX3
  fhatgX3VBLogitUpp <- fhatgX3VBLogit + 1.96 * sdgX3
  
  # Plot logit results
  
  par(mai = c(0.8, 0.9, 0, 0.2))
  
  low1 <- apply(fhatgX1MCMCLogit, 1, quantile, 0.025)
  upp1 <- apply(fhatgX1MCMCLogit, 1, quantile, 0.975)
  xlimVals <- c(min(x1Orig), max(x1Orig))
  ylimVals <- range(low1, upp1, fhatgX1VBLogitLow, fhatgX1VBLogitUpp)
  
  png(paste(dir.Figures, "haberman1Logit.png", sep = ""), res = 144, width = 930, 
    height = 1000)
  
  plot(seq(min(x1Orig), max(x1Orig), length = ng), apply(fhatgX1MCMCLogit, 1, mean), 
    type = "l", col = "red", xlab = "", ylab = "", xlim = xlimVals, ylim = ylimVals, 
    lwd = lwdVal, cex.lab = cex.labVal)
  mtext("logit(prob. patient dies within 5 years)", side = 2, line = 2.2, cex = cex.labVal - 
    0.3)
  mtext("age at time of operation", side = 1, line = 3, cex = cex.labVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), low1, col = "red", lty = 2, 
    lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), upp1, col = "red", lty = 2, 
    lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), fhatgX1VBLogit, col = "blue", 
    lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), fhatgX1VBLogitLow, col = "blue", 
    lty = 2, lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), fhatgX1VBLogitUpp, col = "blue", 
    lty = 2, lwd = lwdVal)
  jit1 <- jitter(x1Orig, amount = 1)
  rug(jit1, ticksize = 0.01, col = "blue")
  legend(0.99 * xlimVals[1] + 0.01 * xlimVals[2], 0.9999 * ylimVals[2] + 1e-04 * 
    ylimVals[1], c(legendStr1, legendStr2, legendStr3, legendStr4), lty = cbind(1, 
    2, 1, 2), col = cbind("blue", "blue", "red", "red"), cex = 1, lwd = lwdVal)
  
  dev.off()
  
  low3 <- apply(fhatgX3MCMCLogit, 1, quantile, 0.025)
  upp3 <- apply(fhatgX3MCMCLogit, 1, quantile, 0.975)
  xlimVals <- c(min(x3Orig), max(x3Orig))
  ylimVals <- range(low3, upp3, fhatgX3VBLogitLow, fhatgX3VBLogitUpp)
  
  png(paste(dir.Figures, "haberman3Logit.png", sep = ""), res = 144, width = 930, 
    height = 1000)
  
  plot(seq(min(x3Orig), max(x3Orig), length = ng), apply(fhatgX3MCMCLogit, 1, mean), 
    type = "l", col = "red", xlab = "", ylab = "", xlim = xlimVals, ylim = ylimVals, 
    lwd = lwdVal, cex.lab = cex.labVal)
  mtext("logit(prob. patient dies within 5 years)", side = 2, line = 2.2, cex = cex.labVal - 
    0.3)
  mtext("number of positive axillary nodes", side = 1, line = 3, cex = cex.labVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), low3, col = "red", lty = 2, 
    lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), upp3, col = "red", lty = 2, 
    lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), fhatgX3VBLogit, col = "blue", 
    lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), fhatgX3VBLogitLow, col = "blue", 
    lty = 2, lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), fhatgX3VBLogitUpp, col = "blue", 
    lty = 2, lwd = lwdVal)
  jit3 <- abs(jitter(x3Orig, amount = 1))
  rug(jit3, ticksize = 0.01, col = "blue")
  
  dev.off()
  
  
  low2 <- apply(fhatgX2MCMCLogit, 1, quantile, 0.025)
  upp2 <- apply(fhatgX2MCMCLogit, 1, quantile, 0.975)
  xlimVals <- c(min(x2Orig), max(x2Orig))
  ylimVals <- range(low2, upp2, fhatgX2VBLogitLow, fhatgX2VBLogitUpp)
  
  png(paste(dir.Figures, "haberman2Logit.png", sep = ""), res = 144, width = 930, 
    height = 1000)
  
  plotCI(seq(min(x2Orig), max(x2Orig)) - 0.1, apply(fhatgX2MCMCLogit, 1, mean), 
    ui = upp2, li = low2, col = "red", xlab = "", ylab = "", xlim = xlimVals, 
    ylim = ylimVals, lwd = lwdVal, cex.lab = cex.labVal)
  plotCI(seq(min(x2Orig), max(x2Orig)) + 0.1, fhatgX2VBLogit, ui = fhatgX2VBLogitUpp, 
    li = fhatgX2VBLogitLow, col = "blue", lwd = lwdVal, cex.lab = cex.labVal, 
    add = TRUE)
  mtext("logit(prob. patient dies within 5 years)", side = 2, line = 2.2, cex = cex.labVal - 
    0.3)
  mtext("year of operation", side = 1, line = 3, cex = cex.labVal)
  jit2 <- jitter(x2Orig, amount = 1)
  rug(jit2, ticksize = 0.01, col = "blue")
  
  dev.off()
  
  ### Probit
  
  # Compute MCMC probit results:
  
  fhatgX1MCMCProbit <- CmatgX1 %*% mu.q.betauMCMCProbit
  fhatgX2MCMCProbit <- CmatgX2 %*% mu.q.betauMCMCProbit
  fhatgX3MCMCProbit <- CmatgX3 %*% mu.q.betauMCMCProbit
  
  sdgX1 <- sqrt(diag(CmatgX1 %*% Sigma.q.betauProbit %*% t(CmatgX1)))
  fhatgX1VBProbit <- CmatgX1 %*% mu.q.betauProbit
  fhatgX1VBProbitLow <- fhatgX1VBProbit - 1.96 * sdgX1
  fhatgX1VBProbitUpp <- fhatgX1VBProbit + 1.96 * sdgX1
  
  sdgX2 <- sqrt(diag(CmatgX2 %*% Sigma.q.betauProbit %*% t(CmatgX2)))
  fhatgX2VBProbit <- CmatgX2 %*% mu.q.betauProbit
  fhatgX2VBProbitLow <- fhatgX2VBProbit - 1.96 * sdgX2
  fhatgX2VBProbitUpp <- fhatgX2VBProbit + 1.96 * sdgX2
  
  sdgX3 <- sqrt(diag(CmatgX3 %*% Sigma.q.betauProbit %*% t(CmatgX3)))
  fhatgX3VBProbit <- CmatgX3 %*% mu.q.betauProbit
  fhatgX3VBProbitLow <- fhatgX3VBProbit - 1.96 * sdgX3
  fhatgX3VBProbitUpp <- fhatgX3VBProbit + 1.96 * sdgX3
  
  
  # Plot MCMC Probit results
  
  par(mai = c(0.8, 0.9, 0, 0.2))
  
  low1 <- apply(fhatgX1MCMCProbit, 1, quantile, 0.025)
  upp1 <- apply(fhatgX1MCMCProbit, 1, quantile, 0.975)
  xlimVals <- c(min(x1Orig), max(x1Orig))
  ylimVals <- range(low1, upp1, fhatgX1VBProbitLow, fhatgX1VBProbitUpp)
  
  png(paste(dir.Figures, "haberman1Probit2.png", sep = ""), res = 144, width = 930, 
    height = 1000)
  
  plot(seq(min(x1Orig), max(x1Orig), length = ng), apply(fhatgX1MCMCProbit, 1, 
    mean), type = "l", col = "red", xlab = "", ylab = "", xlim = xlimVals, ylim = ylimVals, 
    lwd = lwdVal, cex.lab = cex.labVal - 0.3)
  mtext(expression(paste(Phi^-1, "(prob. patient dies within 5 years)")), side = 2, 
    line = 2, cex = cex.labVal - 0.3)
  mtext("age at time of operation", side = 1, line = 3, cex = cex.labVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), low1, col = "red", lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), upp1, col = "red", lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), fhatgX1VBProbit, col = "blue", 
    lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), fhatgX1VBProbitLow, col = "blue", 
    lty = 2, lwd = lwdVal)
  lines(seq(min(x1Orig), max(x1Orig), length = ng), fhatgX1VBProbitUpp, col = "blue", 
    lty = 2, lwd = lwdVal)
  rug(jit1, ticksize = 0.01, col = "blue")
  legend(0.99 * xlimVals[1] + 0.01 * xlimVals[2], 0.9999 * ylimVals[2] + 1e-04 * 
    ylimVals[1], c(legendStr1, legendStr2, legendStr3, legendStr4), lty = cbind(1, 
    2, 1, 2), col = cbind("blue", "blue", "red", "red"), cex = 1, lwd = lwdVal)
  
  dev.off()
  
  low2 <- apply(fhatgX2MCMCProbit, 1, quantile, 0.025)
  upp2 <- apply(fhatgX2MCMCProbit, 1, quantile, 0.975)
  xlimVals <- c(min(x2Orig), max(x2Orig))
  ylimVals <- range(low2, upp2, fhatgX2VBProbitLow, fhatgX2VBProbitUpp)
  
  png(paste(dir.Figures, "haberman2Probit2.png", sep = ""), res = 144, width = 930, 
    height = 1000)
  
  plotCI(seq(min(x2Orig), max(x2Orig)) - 0.1, apply(fhatgX2MCMCProbit, 1, mean), 
    ui = upp2, li = low2, col = "red", xlab = "", ylab = "", xlim = xlimVals, 
    ylim = ylimVals, lwd = lwdVal, cex.lab = cex.labVal - 0.3)
  plotCI(seq(min(x2Orig), max(x2Orig)) + 0.1, fhatgX2VBProbit, ui = fhatgX2VBProbitUpp, 
    li = fhatgX2VBProbitLow, col = "blue", lwd = lwdVal, cex.lab = cex.labVal, 
    add = TRUE)
  mtext(expression(paste(Phi^-1, "(prob. patient dies within 5 years)")), side = 2, 
    cex = cex.labVal - 0.3, line = 2)
  mtext("year of operation", side = 1, cex = cex.labVal, line = 3)
  rug(jit2, ticksize = 0.01, col = "blue")
  
  dev.off()
  
  low3 <- apply(fhatgX3MCMCProbit, 1, quantile, 0.025)
  upp3 <- apply(fhatgX3MCMCProbit, 1, quantile, 0.975)
  xlimVals <- c(min(x3Orig), max(x3Orig))
  ylimVals <- range(low3, upp3, fhatgX3VBProbitLow, fhatgX3VBProbitUpp)
  
  png(paste(dir.Figures, "haberman3Probit2.png", sep = ""), res = 144, width = 930, 
    height = 1000)
  
  plot(seq(min(x3Orig), max(x3Orig), length = ng), apply(fhatgX3MCMCProbit, 1, 
    mean), type = "l", col = "red", xlab = "", ylab = "", xlim = xlimVals, ylim = ylimVals, 
    lwd = lwdVal, cex.lab = cex.labVal)
  mtext(expression(paste(Phi^-1, "(prob. patient dies within 5 years)")), side = 2, 
    line = 2, cex = cex.labVal - 0.3)
  mtext("number of positive axillary nodes", side = 1, line = 3, cex = cex.labVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), low3, col = "red", lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), upp3, col = "red", lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), fhatgX3VBProbit, col = "blue", 
    lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), fhatgX3VBProbitLow, col = "blue", 
    lty = 2, lwd = lwdVal)
  lines(seq(min(x3Orig), max(x3Orig), length = ng), fhatgX3VBProbitUpp, col = "blue", 
    lty = 2, lwd = lwdVal)
  rug(jit3, ticksize = 0.01, col = "blue")
  
  dev.off()
}


################################################################################ 

if (REMOVE.INPUTS) {
  file.remove("ncX.txt")
  file.remove("numKnots1.txt")
  file.remove("numKnots2.txt")
  file.remove("numKnots3.txt")
  file.remove("Cmat.txt")
  file.remove("y.txt")
  file.remove("nIterVB.txt")
  file.remove("tauBeta.txt")
  file.remove("Au.txt")
  file.remove("habermanLogitModel.txt")
  file.remove("habermanProbitModel.txt")
}

if (REMOVE.OUTPUTS) {
  file.remove("mu.q.betau.txt")
  file.remove("Sigma.q.betau.txt")
  file.remove("parms.q.tauu1.txt")
  file.remove("parms.q.tauu2.txt")
  file.remove("parms.q.tauu3.txt")
}

###################### End of Haberman Survival #############################
 
