###################### R script: hauser #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 5 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set directories
dir.InferNET.support <- "..\\..\\InferNETSupport\\"
dir.Figures <- "..\\..\\..\\Figures\\"

# Load support files
source(paste(dir.InferNET.support, "InferNETSupport.R", sep = ""), chdir = TRUE)

# Set tasks:
doRobustPenSplinesTdistMCMC <- TRUE
doRobustPenSplinesTdistInferNET <- TRUE
createPNG <- TRUE
REMOVE.INPUTS <- TRUE
REMOVE.OUTPUTS <- TRUE

# Number of trials to run example (for timing)
TRIALS <- 1

# Read data:
data <- read.csv("HauserLeft.csv", header = TRUE)

xOrig <- data[, 1]
yOrig <- data[, 2]
x <- (xOrig - min(xOrig))/(max(xOrig) - min(xOrig))
y <- (yOrig - min(yOrig))/(max(yOrig) - min(yOrig))
n <- length(x)

# Set up Z matrix of spline basis functions:
numIntKnots <- 25
intKnots <- quantile(unique(x), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x <- c(1.01 * min(x) - 0.01 * max(x), 1.01 * max(x) - 0.01 * min(x))
Z <- ZOSull(x, range.x, intKnots)
numKnots <- ncol(Z)
X <- cbind(rep(1, n), x)
Nbins <- 30
nu.gridmin <- 0.05
nu.gridmax <- 10
nu <- seq(nu.gridmin, nu.gridmax, length = Nbins)

################################################################################ 

if (doRobustPenSplinesTdistMCMC) {
  # Set MCMC parameters:
  
  set.seed(45129)
  
  nBurnin <- 5000  # Length of burn-in. 5000
  nIter <- 100000  # Size of the kept sample. 5000
  nThin <- 1  # Thinning factor. 
  
  # Specify model in BUGS:
  
  tNpRegModel <- function() {
    for (i in 1:n) {
      mu[i] <- beta0 + beta1 * x[i] + inprod(u[], Z[i, ])
      precis[i] <- g[i] * tauEps
      y[i] ~ dnorm(mu[i], precis[i])
      g[i] ~ dgamma(halfNu, halfNu)
    }
    for (k in 1:numKnots) {
      u[k] ~ dnorm(0, tauU)
    }
    for (l in 1:Nbins) {
      prior[l] <- 1/Nbins
    }
    beta0 ~ dnorm(0, 1e-08)
    beta1 ~ dnorm(0, 1e-08)
    
    cuinv ~ dgamma(0.5, 1e-10)
    tauU ~ dgamma(0.5, cuinv)
    cepsinv ~ dgamma(0.5, 1e-10)
    tauEps ~ dgamma(0.5, cepsinv)
    
    nuCat ~ dcat(prior[])
    halfNu <- 0.5 * nu[nuCat]
  }
  
  modelFileName <- file.path(getwd(), "tNpRegModel.txt")
  writeModel(tNpRegModel, modelFileName)
  
  # Set up input data:
  allData <- list(n = n, numKnots = numKnots, x = x, y = y, Z = Z, Nbins = Nbins, 
    nu = nu)
  
  # Initialise parameters:
  parInits <- list(list(beta0 = 0, beta1 = 0, u = rep(0, numKnots), g = rep(1, 
    n), cuinv = 1, cepsinv = 1, tauU = 1, tauEps = 1, nuCat = 1))
  
  # Obtain MCMC samples using BRugs:
  times.brugs <- c()
  for (trial in 1:TRIALS) {
    # Do MCMC using BRugs:
    bval1 <- proc.time()
    BRugsFit(data = allData, inits = parInits, parametersToSave = c("beta0", 
      "beta1", "u", "tauU", "tauEps", "nuCat"), nBurnin = nBurnin, nIter = nIter, 
      nThin = nThin, modelFile = "tNpRegModel.txt", numChains = 1)
    eval1 <- proc.time()
    times.brugs[trial] <- eval1[3] - bval1[3]
  }
  
  # Save BRugs output:
  betaMCMC <- rbind(samplesSample("beta0"), samplesSample("beta1"))
  uMCMC <- samplesSample("u[1]")
  for (k in 2:numKnots) uMCMC <- rbind(uMCMC, samplesSample(paste("u[", k, "]", 
    sep = "")))
  tauEpsMCMC <- samplesSample("tauEps")
  tauUMCMC <- samplesSample("tauU")
  nuMCMC <- samplesSample("nuCat")
  sigEpsMCMC <- 1/sqrt(tauEpsMCMC)
  sigUMCMC <- 1/sqrt(tauUMCMC)
  
  # Display fitted curve:
  ng <- 101
  xg <- seq(range.x[1], range.x[2], length = ng)
  Xg <- cbind(rep(1, ng), xg)
  Zg <- ZOSull(xg, range.x, intKnots)
  fMCMC <- Xg %*% betaMCMC + Zg %*% uMCMC
  credLowerMCMC <- apply(fMCMC, 1, quantile, 0.025)
  credUpperMCMC <- apply(fMCMC, 1, quantile, 0.975)
  fhatgMCMC <- apply(fMCMC, 1, mean)
}

################################################################################ 

if (doRobustPenSplinesTdistInferNET) {
  Cmat <- cbind(X, Z)
  
  # Write input information to files:
  write(ncol(X), "ncX.txt")
  write.table(Cmat, "Cmat.txt", sep = ",", row.names = FALSE, col.names = FALSE)
  write.table(y, "y.txt", sep = ",", row.names = FALSE, col.names = FALSE)
  
  # Set hyperparameters and write to files:
  sigsqBeta <- 1e+08
  Au <- 1e+05
  Aeps <- 1e+05
  
  write(sigsqBeta^(-1), "tauBeta.txt")
  write(Au, "Au.txt")
  write(Aeps, "Aeps.txt")
  
  # Set variational Bayes interations:
  nIterVB <- 100
  write(nIterVB, "nIterVB.txt")
  
  times.Infer.NET <- c()
  for (trial in 1:TRIALS) {
    bval1 <- proc.time()
    
    # Iterate over grid of nus
    nu.gridmin <- 0.05
    nu.gridmax <- 10
    nu.gridsize <- length(seq(nu.gridmin, nu.gridmax, length = Nbins))
    mu.q.beta0.all <- matrix(0, nu.gridsize, 1)
    mu.q.beta1.all <- matrix(0, nu.gridsize, 1)
    mu.q.U.all <- matrix(0, nu.gridsize, numIntKnots + 2)
    sigsq.q.beta0.all <- matrix(0, nu.gridsize, 1)
    sigsq.q.beta1.all <- matrix(0, nu.gridsize, 1)
    sigsq.q.U.all <- matrix(0, nu.gridsize, numIntKnots + 2)
    mll.all <- matrix(0, nu.gridsize, 1)
    count <- 1
    
    Sigma.q.betau.all <- array(0, c(nu.gridsize, numIntKnots + 4, numIntKnots + 
      4))
    
    for (nu in seq(nu.gridmin, nu.gridmax, length = Nbins)) {
      write(nu, "nu.txt")
      
      # Run Infer.NET:
      timeTaken <- system.time(RunInferNET(dir.InferNET.support, "hauser.cs"))
      
      # Read in posterior density parameters:
      mu.q.betau <- scan("mu.q.betau.txt", quiet = TRUE)
      mu.q.beta0.all[count] <- mu.q.betau[1]
      mu.q.beta1.all[count] <- mu.q.betau[2]
      mu.q.U.all[count, ] <- mu.q.betau[3:length(mu.q.betau)]
      Sigma.q.betau <- matrix(scan("Sigma.q.betau.txt"), ncol(Cmat), ncol(Cmat))
      Sigma.q.betau.all[count, , ] <- Sigma.q.betau
      sigsq.q.beta0.all[count] <- Sigma.q.betau[1, 1]
      sigsq.q.beta1.all[count] <- Sigma.q.betau[2, 2]
      sigsq.q.U.all[count, ] <- diag(Sigma.q.betau[3:(numIntKnots + 4), 3:(numIntKnots + 
        4)])
      mll.all[count] <- scan("marginalLogLikelihood.txt", quiet = TRUE) + log(1/nu.gridsize)
      count <- count + 1
    }
    eval1 <- proc.time()
    times.Infer.NET[trial] <- eval1[3] - bval1[3]
  }
  
  q.nu.all <- exp(mll.all - logOfSum(mll.all))
  mu.q.beta0 <- qnorMix(0.5, q.nu.all, mu.q.beta0.all, sqrt(sigsq.q.beta0.all))
  mu.q.beta1 <- qnorMix(0.5, q.nu.all, mu.q.beta1.all, sqrt(sigsq.q.beta1.all))
  mu.q.U <- matrix(0, numIntKnots + 2, 1)
  for (j in 1:(numIntKnots + 2)) {
    mu.q.U[j] <- qnorMix(0.5, q.nu.all, mu.q.U.all[, j], sqrt(sigsq.q.U.all[, 
      j]))
  }
  
  # Plot regression fit:
  
  ng <- 101
  xg <- seq(range.x[1], range.x[2], length = ng)
  Cmatg <- cbind(rep(1, ng), xg, ZOSull(xg, range.x, intKnots))
  mu.q.betau <- rbind(mu.q.beta0, mu.q.beta1, mu.q.U)
  fhatgIN <- Cmatg %*% mu.q.betau
  
  fhatgQIN <- matrix(NA, 3, ng)
  mus <- t(cbind(mu.q.beta0.all, mu.q.beta1.all, mu.q.U.all))
  for (i in 1:ng) {
    alphamus <- t(Cmatg[i, ] %*% mus)
    
    SigmasSq <- matrix(NA, nu.gridsize, 1)
    for (j in 1:nu.gridsize) {
      SigmasSq[j] <- as.numeric(Cmatg[i, ] %*% Sigma.q.betau.all[j, , ] %*% 
        Cmatg[i, ])
    }
    
    fhatgQIN[1, i] <- qnorMix(0.025, q.nu.all, alphamus, sqrt(SigmasSq))
    fhatgQIN[2, i] <- qnorMix(0.5, q.nu.all, alphamus, sqrt(SigmasSq))
    fhatgQIN[3, i] <- qnorMix(0.975, q.nu.all, alphamus, sqrt(SigmasSq))
  }
  
  par(mfrow = c(1, 1), mai = c(0.9, 0.9, 0.2, 0.1))
  lwdVal <- 2
  cex.labVal <- 1.5
  legendStr1 <- "Infer.NET"
  legendStr2 <- "Infer.NET 95% credible sets"
  legendStr3 <- "MCMC"
  legendStr4 <- "MCMC 95% credible sets"
  ylimVal <- range(yOrig)
  xlimVal <- range(xOrig)
  
  if (createPNG) 
    png(paste(dir.Figures, "hauser1.png", sep = ""), res = 144, width = 850, 
      height = 800)
  
  plot(xOrig, yOrig, type = "p", xlab = "time in seconds", ylab = "log(adjusted time of exhalation)", 
    bty = "l", col = "darkgreen", cex.lab = cex.labVal, cex = 0.7)
  lines(xg * (max(xOrig) - min(xOrig)) + min(xOrig), fhatgMCMC * (max(yOrig) - 
    min(yOrig)) + min(yOrig), col = "red", lwd = lwdVal)
  lines(xg * (max(xOrig) - min(xOrig)) + min(xOrig), credLowerMCMC * (max(yOrig) - 
    min(yOrig)) + min(yOrig), col = "red", lwd = lwdVal, lty = 2)
  lines(xg * (max(xOrig) - min(xOrig)) + min(xOrig), credUpperMCMC * (max(yOrig) - 
    min(yOrig)) + min(yOrig), col = "red", lwd = lwdVal, lty = 2)
  lines(xg * (max(xOrig) - min(xOrig)) + min(xOrig), fhatgIN * (max(yOrig) - min(yOrig)) + 
    min(yOrig), col = "blue", lwd = lwdVal)
  lines(xg * (max(xOrig) - min(xOrig)) + min(xOrig), fhatgQIN[1, ] * (max(yOrig) - 
    min(yOrig)) + min(yOrig), col = "blue", lwd = lwdVal, lty = 2)
  lines(xg * (max(xOrig) - min(xOrig)) + min(xOrig), fhatgQIN[3, ] * (max(yOrig) - 
    min(yOrig)) + min(yOrig), col = "blue", lwd = lwdVal, lty = 2)
  legend(0.6 * xlimVal[1] + 0.4 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2, legendStr3, legendStr4), lty = cbind(1, 2, 1, 2), 
    col = cbind("blue", "blue", "red", "red"), cex = 1, lwd = lwdVal)
  dev.off()
  
  if (createPNG) 
    png(paste(dir.Figures, "hauser2.png", sep = ""), res = 144, width = 730, 
      height = 800)
  
  nu <- seq(nu.gridmin, nu.gridmax, length = Nbins)
  nuValuesMCMC <- c(0, 0, table(nu[nuMCMC]))
  
  resultMCMC <- nuValuesMCMC/sum(nuValuesMCMC)
  
  par(mfrow = c(1, 1), mai = c(0.9, 0.9, 0.5, 0.1))
  ylimVal <- c(0, 0.25)
  xlimVal <- c(0.8, 7.5)
  cex.mainVal <- 2
  plot(seq(nu.gridmin, nu.gridmax, length = Nbins), q.nu.all, col = "blue", lwd = lwdVal, 
    lty = 1, type = "n", xlim = xlimVal, ylim = ylimVal, bty = "l", cex.lab = cex.labVal, 
    xlab = expression(nu), ylab = "approx. posterior mass", main = " ", cex.main = cex.mainVal)
  for (i in 1:Nbins) {
    lines(c(nu[i], nu[i]), c(0, resultMCMC[i]), lty = 1, lwd = lwdVal, col = "red")
    lines(c(nu[i], nu[i]), c(0, q.nu.all[i]), col = "blue", lwd = lwdVal, lty = 1)
    points(nu[i], resultMCMC[i], col = "red", pch = 19, cex = 1.5)
    points(nu[i], q.nu.all[i], col = "blue", pch = 19, cex = 1.5)
  }
  legend(0.4 * xlimVal[1] + 0.6 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr3), lty = cbind(1, 1), col = cbind("blue", "red"), 
    cex = 1, lwd = lwdVal)
  abline(0, 0)
  dev.off()
}

################################################################################ 

if (REMOVE.INPUTS) {
  file.remove("ncX.txt")
  file.remove("Cmat.txt")
  file.remove("y.txt")
  file.remove("Au.txt")
  file.remove("Aeps.txt")
  file.remove("tauBeta.txt")
  file.remove("nu.txt")
  file.remove("nIterVB.txt")
  file.remove("tNpRegModel.txt")
}

if (REMOVE.OUTPUTS) {
  file.remove("mu.q.betau.txt")
  file.remove("Sigma.q.betau.txt")
  file.remove("parms.q.tauEps.txt")
  file.remove("parms.q.tauu.txt")
  file.remove("marginalLogLikelihood.txt")
}

###################### End of hauser #############################
 
