//////////////////////////  C# program: hauser /////////////////////////

// C# file containing support functions for the R/InferNET code for the paper:
//
// Luts J, Shen SSJ, Ormerod JT and Wand MP (2014)
//    Semiparametric Regression Analysis via Infer.NET

// Example for Section 5 of the paper.

// Last changed: 22 JULY 2014 by J.T Ormerod

// Load required C# and Infer.NET libraries: 
using System; using System.Collections.Generic;
using System.Text; using System.IO;
using System.Reflection; using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
   class penSplineRobusttIN
   {
      public static void Main()
      {
         // Set full path-name of current working directory:
         string dir = ".\\";

         // Read in the number of columns of the X matrix:
         int ncX = new DataTable(dir+"ncX.txt").IntNumeric;
         
         // Read input files into data tables: 
         DataTable CTable = new DataTable(dir+"Cmat.txt");
         DataTable yTable = new DataTable(dir+"y.txt");

         // Extract total number of observations and number of columns
         // in the C matrix, and declare as integers:
         int nObs = CTable.numRow;
         int ncC = CTable.numCol;

         // Read in degrees of freedom:
         double nu = new DataTable(dir + "nu.txt").DoubleNumeric;         

         // Read in hyperparameter values:
         double tauBeta = new DataTable(dir+"tauBeta.txt").DoubleNumeric;
         double Au = new DataTable(dir+"Au.txt").DoubleNumeric;
         double Aeps = new DataTable(dir + "Aeps.txt").DoubleNumeric;

         // Read in number of iterations for variational Bayes:
         int nIterVB = new DataTable(dir+"nIterVB.txt").IntNumeric;
          
         // Make a block for storage of the marginal log-likelihood:
         Variable<bool> auxML = Variable.Bernoulli(0.5).Named("auxML");
         IfBlock model = Variable.If(auxML);

         // Specify prior for the precision of spline coefficients:
         Variable<double> tauu = Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Au, -2))).Named("tauu");

         // Specify prior for g to impose a t-distribution:
         Range index = new Range(nObs).Named("index");
         VariableArray<double> g = Variable.Array<double>(index);
         g[index] = Variable.GammaFromShapeAndScale(nu/2,nu/2).ForEach(index); 

         // Create a vector named "zeroVec" of length ncC:
         // (Note that this is automatically set to 
         // a vector of zeroes by convention.)
         Vector zeroVec = Vector.Zero(ncC);

         // Create positive definite matrix of dimension ncC by ncC:
         // (Note that this is automatically set to a matrix of zeroes
         // by convention.)
         PositiveDefiniteMatrix betauPrecDummy = new PositiveDefiniteMatrix(ncC, ncC);

         // Fill in diagonal entries of betauPrecDummy to make it diffuse:
         for (int i = 0; i < ncC; i++)
             betauPrecDummy[i, i] = 1e-10;

         // Create a vector named "betauWork".
         Variable<Vector> betauWork = Variable.VectorGaussianFromMeanAndPrecision(
                        zeroVec, betauPrecDummy).Named("betauWork");

         // Creating an array of variables of type "double" of length ncC: 
         Variable<double>[] betauDummy = new Variable<double>[ncC];

         // Populate "betauDummy" with the entries of "betauWork": 
         for (int j = 0; j < ncC; j++)
             betauDummy[j] = Variable.GetItem(betauWork, j);

         // Set up an auxiliary variable a with mean betau and
         // covariance of betau, and then make observed value
         // zero (this is a trick that has mathematical justification
         // in Wang, S.S.J and Wand, M.P. Using Infer.NET for 
         // Statistical Analyses. The American Statistician, 2011, 
         // 65, 115-126.):

         Variable<double>[] a = new Variable<double>[ncC];
         for (int j = 0; j < ncX; j++) {
             a[j] = Variable.GaussianFromMeanAndPrecision(betauDummy[j],tauBeta);
             a[j].ObservedValue = 0.0;
         }

         for (int j = ncX; j < ncC; j++) {
             a[j] = Variable.GaussianFromMeanAndPrecision(betauDummy[j],tauu);
             a[j].ObservedValue = 0.0;
         }

         // Specify prior for the error precision:
         Variable<double> tauEps = (Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Aeps, -2)))).Named("tauEps");

         // Specify the likelihood: 
         VariableArray<double> y = Variable.Array<double>(index).Named("y");
         VariableArray<Vector> cvec = Variable.Array<Vector>(index).Named("cvec");
         y[index] = Variable.GaussianFromMeanAndPrecision(Variable.InnerProduct(betauWork, cvec[index]), g[index] * tauEps);
          
         // Close block for storage of the marginal log-likelihood:
         model.CloseBlock();
          
         // Create "cvecTable", and then assign the regression input data, 
         // stored in "cvecTable" and "yTable", to the "cvec" and "y" variables:
         Vector[] cvecTable = new Vector[nObs];
         for (int i = 0; i < nObs; i++) 
         {
            cvecTable[i] = Vector.Zero(ncC);
            for (int j = 0; j < ncC; j++)
               cvecTable[i][j] = CTable.DataMatrix[i,j];
         }
         cvec.ObservedValue = cvecTable;
         y.ObservedValue = yTable.arrayForIN;

         // Specify inference engine and algorithm:
         InferenceEngine engine = new InferenceEngine();
       
         engine.Algorithm = new VariationalMessagePassing();
         engine.NumberOfIterations = nIterVB;
         
         // Do not print out warnings
         engine.ShowWarnings = false;
         
         // Output parameters of the approximate posterior: 
         double marginalLogLikelihood = engine.Infer<Bernoulli>(auxML).LogOdds;
         SaveData.SaveTable(marginalLogLikelihood,
                            dir + "marginalLogLikelihood.txt");
         SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetMean(),
                   dir+"mu.q.betau.txt");
         SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetVariance(),
                   dir+"Sigma.q.betau.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauEps), dir + "parms.q.tauEps.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauu), dir + "parms.q.tauu.txt");
      }
   }
}

/////////////////////////////  End of hauser /////////////////////////////
