###################### R script: lasso #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 8 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set directories
dir.InferNET.support <- "..\\..\\InferNETSupport\\"
dir.Figures <- "..\\..\\..\\Figures\\"

# Load support files
source(paste(dir.InferNET.support, "InferNETSupport.R", sep = ""), chdir = TRUE)

# Set tasks:
doMCMC <- TRUE
doInferNET <- TRUE
REMOVE.INPUTS <- TRUE
REMOVE.OUTPUTS <- TRUE

# Number of trials to run example (for timing)
TRIALS <- 1

# Set MCMC parameters:
nBurnin <- 5000  # Length of burn-in.
nIter <- 5000  # Size of the kept sample.
nThin <- 5  # Thinning factor.

# Set flags:
center <- FALSE

# Load data
data("diabetes", package = "lars")
X <- diabetes$x
y <- diabetes$y

n <- length(y)
p <- ncol(X)

if (center) {
  y <- as.numeric(scale(y))
  y <- (y - min(y))/(max(y) - min(y))
  Xmat <- NULL
  for (i in 1:p) {
    Xmat <- cbind(Xmat, (X[, i] - min(X[, i]))/(max(X[, i]) - min(X[, i])))
  }
  X <- Xmat
}

# Set hyper-parameters
sigsq.beta0 <- 10^8
A <- 100
C <- 100

################################################################################ 

if (doMCMC) {
  # set up model
  lassoModel <- function() {
    for (i in 1:n) {
      mu[i] <- beta0 + inprod(beta[], X[i, ])
      y[i] ~ dnorm(mu[i], tauEps)
    }
    for (k in 1:p) {
      b[k] ~ dgamma(1, rateb)
      taub[k] <- 1/b[k]
      beta[k] ~ dnorm(0, taub[k])
    }
    rateb <- 0.5 * taubeta
    beta0 ~ dnorm(0, taubeta0)
    numerEps ~ dnorm(0, 1)
    denomEps ~ dnorm(0, InvAsq)
    tauEps <- pow(numerEps/denomEps, 2)
    numerbeta ~ dnorm(0, 1)
    denombeta ~ dnorm(0, InvAsq)
    taubeta <- pow(numerbeta/denombeta, 2)
  }
  fileName <- file.path(getwd(), "lassoModel.txt")
  writeModel(lassoModel, fileName)
  
  # Set up input data:
  allData <- list(n = n, p = p, X = X, y = y, taubeta0 = 1/sigsq.beta0, InvAsq = 1/A^2)
  
  # Initialise parameters:
  parInits <- list(list(beta0 = 0, beta = rep(0, p), b = rep(1, p), numerEps = 1, 
    denomEps = 1, numerbeta = 1, denombeta = 1))
  
  # Do MCMC using BRugs:
  times.brugs <- c()
  for (trial in 1:TRIALS) {
    bval1 <- proc.time()
    res.BRugsMCMC <- BRugsFit(data = allData, inits = parInits, parametersToSave = c("beta0", 
      "beta", "tauEps", "b", "taubeta"), nBurnin = nBurnin, nIter = nIter, 
      nThin = nThin, modelFile = "lassoModel.txt", numChains = 1)
    eval1 <- proc.time()
    times.brugs[trial] <- eval1[3] - bval1[3]
  }
  
  beta0MCMC1 <- samplesSample("beta0")
  tauEpsMCMC1 <- samplesSample("tauEps")
  sigsqEpsMCMC1 <- 1/tauEpsMCMC1
  taubetaMCMC1 <- samplesSample("taubeta")
  sigsqbetaMCMC1 <- 1/taubetaMCMC1
  betaMCMC1 <- samplesSample("beta[1]")
  for (k in 2:p) { 
      betaMCMC1 <- rbind(betaMCMC1, samplesSample(paste("beta[", k, "]", sep = "")))
  }
  
  # Display summary for the parameters:
  parms <- list(cbind(sigsqEpsMCMC1, beta0MCMC1, sigsqbetaMCMC1))
  par.names.Val <- list(c(expression(sigma[epsilon]^2)), c(expression(beta[0])), 
    c(expression(sigma[beta]^2)))
  summMCMC(xList = parms, parNames = par.names.Val, KDEvertLine = FALSE, addTruthToKDE = c(-999, 
    -999, -999))
  
  parms <- list(t(betaMCMC1))
  par.names.Val <- list(c(expression(beta[1])), c(expression(beta[2])), c(expression(beta[3])), 
    c(expression(beta[4])), c(expression(beta[5])), c(expression(beta[6])), c(expression(beta[7])), 
    c(expression(beta[8])), c(expression(beta[9])), c(expression(beta[10])))
  
  summMCMC(xList = parms, parNames = par.names.Val, KDEvertLine = FALSE, addTruthToKDE = rep(-999, 
    p))
}

################################################################################ 

if (doInferNET) {
  Cmat <- cbind(rep(1, n), X)
  
  # Write input information to files:
  write(ncol(X), "ncX.txt")
  write.table(Cmat, "Cmat.txt", sep = ",", row.names = FALSE, col.names = FALSE)
  write.table(y, "y.txt", sep = ",", row.names = FALSE, col.names = FALSE)
  
  # write hyperparameters to files:
  write(sigsq.beta0, "sigsqBeta0.txt")
  write(A, "A.txt")
  write(C, "C.txt")
  
  # Set Gibbs and variational Bayes iteration sizes.  If `nIterGibbs' is zero then
  # variational Bayes used instead of Gibbs sampling:
  nIterGibbs <- 0
  nBurnin <- 1
  nThin <- 1
  nIterVB <- 100
  write(nIterVB, "nIterVB.txt")
  write(nIterGibbs, "nIterGibbs.txt")
  write(nBurnin, "nBurnin.txt")
  write(nThin, "nThin.txt")
  
  # Run Infer.NET:
  times.Infer.NET <- c()
  for (trial in 1:TRIALS) {
    timeTaken <- system.time(RunInferNET(dir.InferNET.support, "lasso.cs"))
    times.Infer.NET[trial] <- timeTaken[3]
  }
}

################################################################################ 

# Plot approximate posterior for beta1 to betap

mu.q.betau <- scan("mu.q.betau.txt", quiet = TRUE)
Sigma.q.betau <- matrix(scan("Sigma.q.betau.txt"), (p + 1), (p + 1))
parms.q.sigsqEps <- scan("parms.q.sigsqEps.txt", quiet = TRUE)
parms.q.sigsqBeta <- scan("parms.q.sigsqBeta.txt", quiet = TRUE)
A.q.sigsqEps <- parms.q.sigsqEps[1]
B.q.sigsqEps <- 1/parms.q.sigsqEps[2]
A.q.sigsqBeta <- parms.q.sigsqBeta[1]
B.q.sigsqBeta <- 1/parms.q.sigsqBeta[2]

png(paste(dir.Figures, "simpLogistMix.png", sep = ""), res = 144, width = 2400, height = 1200)

mai.dft <- c(1.02, 0.82, 0.82, 0.42)
op <- par(mai = mai.dft)
pcnt1 <- 0.6
pcnt2 <- 0.8
pcnt3 <- 0.4
pcnt4 <- 0.2
mai.new <- c(pcnt1, pcnt2, pcnt3, pcnt4) * mai.dft
par(mfrow = c(3, 5), mai = mai.new)

accVarApp(c(mu.q.betau[1], Sigma.q.betau[1, 1]), "intercept", beta0MCMC1, "Normal", 
  xlabVal = expression(beta[0]))
accVarApp(c(mu.q.betau[2], Sigma.q.betau[2, 2]), "age", betaMCMC1[1, ], "Normal", 
  xlabVal = expression(beta[1]))
accVarApp(c(mu.q.betau[3], Sigma.q.betau[3, 3]), "sex", betaMCMC1[2, ], "Normal", 
  xlabVal = expression(beta[2]))
accVarApp(c(mu.q.betau[4], Sigma.q.betau[4, 4]), "body mass index", betaMCMC1[3, 
  ], "Normal", xlabVal = expression(beta[3]))
accVarApp(c(mu.q.betau[5], Sigma.q.betau[5, 5]), "average blood pressure", betaMCMC1[4, 
  ], "Normal", xlabVal = expression(beta[4]))
accVarApp(c(mu.q.betau[6], Sigma.q.betau[6, 6]), "tc", betaMCMC1[5, ], "Normal", 
  xlabVal = expression(beta[5]))
accVarApp(c(mu.q.betau[7], Sigma.q.betau[7, 7]), "ldl", betaMCMC1[6, ], "Normal", 
  xlabVal = expression(beta[6]))
accVarApp(c(mu.q.betau[8], Sigma.q.betau[8, 8]), "hdl", betaMCMC1[7, ], "Normal", 
  xlabVal = expression(beta[7]))
accVarApp(c(mu.q.betau[9], Sigma.q.betau[9, 9]), "tch", betaMCMC1[8, ], "Normal", 
  xlabVal = expression(beta[8]))
accVarApp(c(mu.q.betau[10], Sigma.q.betau[10, 10]), "ltg", betaMCMC1[9, ], "Normal", 
  xlabVal = expression(beta[9]))
accVarApp(c(mu.q.betau[11], Sigma.q.betau[11, 11]), "glu", betaMCMC1[10, ], "Normal", 
  xlabVal = expression(beta[10]))
accVarApp(c(A.q.sigsqBeta, B.q.sigsqBeta), "variance of coefficients", sigsqbetaMCMC1, 
  "InvGamma", xlabVal = expression(tau[beta]^-1))
accVarApp(c(A.q.sigsqEps, B.q.sigsqEps), "variance of error", sigsqEpsMCMC1, "InvGamma", 
  xlabVal = expression(tau[epsilon]^-1))

dev.off()

################################################################################ 

if (REMOVE.INPUTS) {
  file.remove("ncX.txt")
  file.remove("Cmat.txt")
  file.remove("y.txt")
  file.remove("A.txt")
  file.remove("C.txt")
  file.remove("sigsqBeta0.txt")
  file.remove("nIterVB.txt")
  file.remove("nIterGibbs.txt")
  file.remove("nBurnin.txt")
  file.remove("nThin.txt")
  file.remove("lassoModel.txt")
}

if (REMOVE.OUTPUTS) {
  file.remove("mu.q.betau.txt")
  file.remove("Sigma.q.betau.txt")
  file.remove("parms.q.sigsqEps.txt")
  file.remove("parms.q.sigsqBeta.txt")
  file.remove("lassoModel.txt")
}

###################### End of lasso #############################
 
