//////////////////////////  C# programme: lasso.cs /////////////////////////

// C# file containing support functions for the R/InferNET code for the paper:
//
// Luts J, Shen SSJ, Ormerod JT and Wand MP (2014)
//    Semiparametric Regression Analysis via Infer.NET

// Example for Section 8 of the paper.

// Last changed: 22 JULY 2014 by J.T Ormerod
 
// Load required C# and Infer.NET libraries: 
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
    class lasso
    {
        public static void Main()
        {
            // Set full path-name of current working directory:
            string dir = ".\\";

            // Read in the number of columns of the X matrix:
            int ncX = new DataTable(dir + "ncX.txt").IntNumeric;

            // Read input files into data tables: 
            DataTable CTable = new DataTable(dir + "Cmat.txt");
            DataTable yTable = new DataTable(dir + "y.txt");

            // Extract total number of observations and and number of columns
            // in the C matrix, and declare as integers:
            int nObs = CTable.numRow;
            int ncC = CTable.numCol;

            // Read in hyperparameter values:
            double sigsqBeta0 = new DataTable(dir + "sigsqBeta0.txt").DoubleNumeric;
            double C = new DataTable(dir + "C.txt").DoubleNumeric;
            double A = new DataTable(dir + "A.txt").DoubleNumeric;
            
            // Read in number of Gibbs samples. If this number
            // is zero then varitional approximation is used.
            int nIterGibbs = new DataTable(dir + "nIterGibbs.txt").IntNumeric;
            int nBurnin = new DataTable(dir + "nBurnin.txt").IntNumeric;
            int nThin = new DataTable(dir + "nThin.txt").IntNumeric;

            // Read in number of Iterations for variational Bayes:
            int nIterVB = new DataTable(dir + "nIterVB.txt").IntNumeric;

            // Specify prior for the error precision:
            Variable<double> tauEps = Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(A, -2))).Named("tauEps");

            // Create a vector named "zeroVec" of length ncC:
            // (Note that this is automatically set to 
            // a vector of zeroes by convention.)
            Vector zeroVec = Vector.Zero(ncC);

            // Create positive definite matrix of dimension ncC by ncC:
            // (Note that this is automatically set to a matrix of zeroes
            // by convention.)
            PositiveDefiniteMatrix betauPrecDummy = new PositiveDefiniteMatrix(ncC, ncC);
            Variable<double> tauBeta = Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(A, -2))).Named("lambda2");

            Variable<double>[] c = new Variable<double>[ncX];
            Variable<double>[] d = new Variable<double>[ncX];
            for (int i = 0; i < ncX; i++) {
                d[i] = Variable.GammaFromShapeAndRate(1, 0.5 * tauBeta);
                c[i] = Variable.GammaFromShapeAndRate(C, C * d[i]);
            }

            // Fill in diagonal entries of betauPrecDummy to make it diffuse:
            for (int i = 0; i < ncC; i++)
                betauPrecDummy[i, i] = 1e-10;

            // Create a vector named "betauWork".
            Variable<Vector>
               betauWork = Variable.VectorGaussianFromMeanAndPrecision(
                           zeroVec, betauPrecDummy).Named("betauWork");

            // Creating an array of variables of type "double" of length ncC: 
            Variable<double>[] betauDummy = new Variable<double>[ncC];

            // Populate "betauDummy" with the entries of "betauWork": 
            for (int j = 0; j < ncC; j++)
                betauDummy[j] = Variable.GetItem(betauWork, j);

            // Set up an auxiliary variable with mean betau and
            // covariance of betau, and then make observed value
            // zero (this is a trick that has mathematical justification
            // in the Shen & Wang paper):
            Variable<double>[] auxVar = new Variable<double>[ncC];
            auxVar[0] = Variable.GaussianFromMeanAndVariance(betauDummy[0], sigsqBeta0);
            auxVar[0].ObservedValue = 0.0;
            for (int j = 1; j < ncC; j++) {
                auxVar[j] = Variable.GaussianFromMeanAndPrecision(betauDummy[j], c[j - 1]);
                auxVar[j].ObservedValue = 0.0;
            }

            // Specify the likelihood: 
            Range index = new Range(nObs).Named("index");
            VariableArray<double> y = Variable.Array<double>(index).Named("y");
            VariableArray<Vector> cvec = Variable.Array<Vector>(index).Named("cvec");
            y[index] = Variable.GaussianFromMeanAndPrecision(
                      Variable.InnerProduct(betauWork, cvec[index]), tauEps);

            // Create "cvecTable", and then assign the regression input data, 
            // stored in "cvecTable" and "yTable", to the "cvec" and "y" variables:
            Vector[] cvecTable = new Vector[nObs];
            for (int i = 0; i < nObs; i++) {
                cvecTable[i] = Vector.Zero(ncC);
                for (int j = 0; j < ncC; j++)
                    cvecTable[i][j] = CTable.DataMatrix[i, j];
            }
            cvec.ObservedValue = cvecTable;
            y.ObservedValue = yTable.arrayForIN;

            // Specify inference engine and algorithm:
            InferenceEngine engine = new InferenceEngine();

            if (nIterGibbs > 0) {
                GibbsSampling gs = new GibbsSampling();
                gs.BurnIn = nBurnin; gs.Thin = nThin;
                engine.Algorithm = gs;
                engine.NumberOfIterations = nBurnin + nIterGibbs;
            } else {
                engine.Algorithm = new VariationalMessagePassing();
                engine.NumberOfIterations = nIterVB;
            }

            // Output parameters of the approximate posterior: 
            SaveData.SaveTable(engine.Infer<Gamma>(tauBeta), dir + "parms.q.sigsqBeta.txt");
            SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetMean(),
                      dir + "mu.q.betau.txt");
            SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetVariance(),
                      dir + "Sigma.q.betau.txt");
            SaveData.SaveTable(engine.Infer<Gamma>(tauEps), dir + "parms.q.sigsqEps.txt");
        }
    }
}

/////////////////////////////  End of lasso /////////////////////////////