###################### R script: meaErrSLR #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 9 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set directories
dir.InferNET.support <- "..\\..\\InferNETSupport\\"
dir.Figures <- "..\\..\\..\\Figures\\"

# Load support files
source(paste(dir.InferNET.support, "InferNETSupport.R", sep = ""), chdir = TRUE)

# Set tasks:
doMCMC <- TRUE
doInferNET <- TRUE
REMOVE.INPUTS <- TRUE
REMOVE.OUTPUTS <- TRUE

# Set MCMC parameters:
nBurnin <- 5000  # Length of burn-in.
nIter <- 50000  # Size of the kept sample.
nThin <- 1  # Thinning factor. 

# Number of trials to run example (for timing)
TRIALS <- 1

################################################################################ 

# Simulate some data
n <- 50
beta0True <- 0.3
beta1True <- 0.7
sigZ <- 0.1
tauZ <- 1/sigZ^2
sigEpsTrue <- 0.25
tauEpsTrue <- 1/sigEpsTrue^2
muXTrue <- 0.5
tauXTrue <- 36
sigXTrue <- 1/sqrt(tauXTrue)
x <- rnorm(n, muXTrue, sigXTrue)
w <- x + sigZ * rnorm(n)
y <- beta0True + beta1True * x + sigEpsTrue * rnorm(n)

# Specify values of hyperparameters:
sigsq.beta <- 1e+08
sigsq.mu.x <- 1e+08
A.eps <- 1e+05
A.x <- 1e+05

################################################################################ 

if (doMCMC) {
  # Load required libraries and functions:
  library("BRugs")
  
  # Specify model in BUGS:
  meaErrSLRModel <- function() {
    for (i in 1:n) {
      x[i] ~ dnorm(muX, tauX)
      w[i] ~ dnorm(x[i], tauZ)
      mu[i] <- beta0 + beta1 * x[i]
      y[i] ~ dnorm(mu[i], tauEps)
    }
    beta0 ~ dnorm(0, 1e-08)
    beta1 ~ dnorm(0, 1e-08)
    muX ~ dnorm(0, 1e-08)
    bX ~ dgamma(0.5, 1e-10)
    tauX ~ dgamma(0.5, bX)
    bEps ~ dgamma(0.5, 1e-10)
    tauEps ~ dgamma(0.5, bEps)
  }
  
  modelFileName <- file.path(getwd(), "meaErrSLRModel.txt")
  writeModel(meaErrSLRModel, modelFileName)
  
  # Set up input data:
  allData <- list(n = n, tauZ = tauZ, w = w, y = y)
  
  # Initialise parameters:
  parInits <- list(list(beta0 = 0, beta1 = 0, muX = 0, tauX = 1, tauEps = 1, x = w, 
    bX = 1, bEps = 1))
  
  # Parameters to save
  parToSave <- c("beta0", "beta1", "tauEps", "muX", "tauX", "x")
  
  times.brugs <- c()
  for (trial in 1:TRIALS) {
    # Do MCMC using BRugs:
    bval1 <- proc.time()
    BRugsFit(data = allData, inits = parInits, parametersToSave = parToSave, 
      nBurnin = nBurnin, nIter = nIter, nThin = nThin, modelFile = "meaErrSLRModel.txt", 
      numChains = 1)
    eval1 <- proc.time()
    times.brugs[trial] <- eval1[3] - bval1[3]
  }
  
  # Save BRugs output:
  beta0MCMC <- samplesSample("beta0")
  beta1MCMC <- samplesSample("beta1")
  tauEpsMCMC <- samplesSample("tauEps")
  muXMCMC <- samplesSample("muX")
  tauXMCMC <- samplesSample("tauX")
  sigEpsMCMC <- 1/sqrt(tauEpsMCMC)
  sigXMCMC <- 1/sqrt(tauXMCMC)
  x7MCMC <- samplesSample("x[7]")
  
  # Do some summary plots:
  parms <- list(cbind(beta0MCMC, beta1MCMC, muXMCMC, sigXMCMC, sigEpsMCMC))
  parNamesVal <- list(c(expression(beta[0])), c(expression(beta[1])), c(expression(mu[x])), 
    c(expression(sigma[x])), c(expression(sigma[epsilon])))
  
  summMCMC(parms, parNames = parNamesVal, KDEvertLine = FALSE, addTruthToKDE = c(beta0True, 
    beta1True, muXTrue, sigXTrue, sigEpsTrue, -99))
}

################################################################################ 

# Save the original data
w.til <- w

# Transform data: Subtract the mean from w so that coefficient parameter
# posteriors are approximately independent
mean.w <- mean(w)
w <- w - mean.w

# Set Gibbs and variational Bayes iteration sizes.  If `nIterGibbs' is zero then
# variational Bayes used instead of Gibbs sampling:
nIterGibbs <- 0
nBurnin <- 1000
nThin <- 5
nIterVB <- 100
write(nIterVB, "nIterVB.txt")
write(nIterGibbs, "nIterGibbs.txt")
write(nBurnin, "nBurnin.txt")
write(nThin, "nThin.txt")

# Set hyperparameters:
write(sigsq.beta, "sigsqBeta.txt")
write(sigsq.mu.x, "sigsqMuX.txt")
write(A.x, "Ax.txt")
write(A.eps, "Aeps.txt")
write(tauZ, "tauZ.txt")

# Write input information to files:
write.table(y, "y.txt", row.names = FALSE, col.names = FALSE)
write.table(w, "w.txt", row.names = FALSE, col.names = FALSE)

if (doInferNET) {
  times.Infer.NET <- c()
  for (trial in 1:TRIALS) {
    # Run Infer.NET:
    timeTaken <- system.time(RunInferNET(dir.InferNET.support, "meaErrSLR.cs"))
    times.Infer.NET[trial] <- timeTaken[3]
  }
  
  mu.q.beta0.Tformed <- scan("mu.q.beta0.txt", quiet = TRUE)
  sigsq.q.beta0.Tformed <- scan("sigsq.q.beta0.txt", quiet = TRUE)
  mu.q.beta1.Tformed <- scan("mu.q.beta1.txt", quiet = TRUE)
  sigsq.q.beta1.Tformed <- scan("sigsq.q.beta1.txt", quiet = TRUE)
  mu.q.muX.Tformed <- scan("mu.q.muX.txt", quiet = TRUE)
  sigsq.q.muX.Tformed <- scan("sigsq.q.muX.txt", quiet = TRUE)
  
  parms.q.sigsqX.Tformed <- scan("parms.q.tauX.txt", quiet = TRUE)
  A.q.sigsqX.Tformed <- parms.q.sigsqX.Tformed[1]
  B.q.sigsqX.Tformed <- 1/parms.q.sigsqX.Tformed[2]
  
  parms.q.sigsqEps.Tformed <- scan("parms.q.tauEps.txt", quiet = TRUE)
  A.q.sigsqEps.Tformed <- parms.q.sigsqEps.Tformed[1]
  B.q.sigsqEps.Tformed <- 1/parms.q.sigsqEps.Tformed[2]
  
  xDistVals <- read.table("parms.q.x.txt", sep = ",")
}

# Back transform results
R <- matrix(c(1, 0, -mean.w, 1), 2, 2)
Sigma.q.beta <- R %*% diag(c(sigsq.q.beta0.Tformed, sigsq.q.beta1.Tformed), 2) %*% 
  t(R)
mu.q.beta <- R %*% matrix(c(mu.q.beta0.Tformed, mu.q.beta1.Tformed), 2, 1)
mu.q.muX <- mu.q.muX.Tformed + mean.w
xDistVals[, 2] <- xDistVals[, 2] + mean.w

################################################################################ 

# Plot the results

png(paste(dir.Figures, "measErrSLR.png", sep = ""), res = 144, width = 1400, height = 1200)

mai.dft <- c(1.02, 0.82, 0.82, 0.42)
op <- par(mai = mai.dft)
pcnt1 <- 0.6
pcnt2 <- 0.8
pcnt3 <- 1
pcnt4 <- 0.6
mai.new <- c(pcnt1, pcnt2, pcnt3, pcnt4) * mai.dft
par(mfrow = c(2, 3), mai = mai.new)

a1 <- accVarApp(c(mu.q.beta[1], Sigma.q.beta[1, 1]), "intercept", beta0MCMC, type = "Normal", 
  parTrue = beta0True, xlabVal = expression(beta[0]))
a2 <- accVarApp(c(mu.q.beta[2], Sigma.q.beta[2, 2]), "slope", beta1MCMC, type = "Normal", 
  parTrue = beta1True, xlabVal = expression(beta[1]))
a3 <- accVarApp(c(mu.q.muX, sigsq.q.muX.Tformed), "mean of x", muXMCMC, type = "Normal", 
  parTrue = muXTrue, xlabVal = expression(mu[x]))
a4 <- accVarApp(c(A.q.sigsqX.Tformed, B.q.sigsqX.Tformed), "variance of x", 1/tauXMCMC, 
  type = "InvGamma", parTrue = 1/tauXTrue, xlabVal = expression(tau[x]^-1))
a5 <- accVarApp(c(A.q.sigsqEps.Tformed, B.q.sigsqEps.Tformed), "error variance", 
  1/tauEpsMCMC, type = "InvGamma", parTrue = 1/tauEpsTrue, xlabVal = expression(tau[epsilon]^-1))
a6 <- accVarApp(c(xDistVals[7, 2], xDistVals[7, 3]), expression(x[7]), x7MCMC, type = "Normal", 
  parTrue = x[7], xlabVal = expression(x[7]))

dev.off()

################################################################################ 

if (REMOVE.INPUTS) {
  file.remove("nIterVB.txt")
  file.remove("nIterGibbs.txt")
  file.remove("nBurnin.txt")
  file.remove("nThin.txt")
  file.remove("sigsqBeta.txt")
  file.remove("sigsqMuX.txt")
  file.remove("Ax.txt")
  file.remove("Aeps.txt")
  file.remove("tauZ.txt")
  file.remove("y.txt")
  file.remove("w.txt")
  file.remove("meaErrSLRModel.txt")
}

if (REMOVE.OUTPUTS) {
  file.remove("mu.q.beta0.txt")
  file.remove("sigsqMuX.txt")
  file.remove("sigsq.q.beta0.txt")
  file.remove("mu.q.beta1.txt")
  file.remove("sigsq.q.beta1.txt")
  file.remove("mu.q.muX.txt")
  file.remove("sigsq.q.muX.txt")
  file.remove("parms.q.tauX.txt")
  file.remove("parms.q.tauEps.txt")
  file.remove("parms.q.x.txt")
}

########## End of meaErrSLR ##########
 
