//////////////////////////  C# programme: meaErrSLR /////////////////////////

// C# file containing support functions for the R/InferNET code for the paper:
//
// Luts J, Shen SSJ, Ormerod JT and Wand MP (2014)
//    Semiparametric Regression Analysis via Infer.NET

// Example for Section 9 of the paper.

// Last changed: 22 JULY 2014 by J.T Ormerod

// Load required C# and Infer.NET libraries: 
using System; using System.Collections.Generic;
using System.Text; using System.IO;
using System.Reflection; using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
   class simpLinReg1
   {
      public static void Main()
      {
         // Set full path-name of current working directory:
         string dir = ".\\";

         // Read input files into data tables: 
         DataTable wTable = new DataTable(dir+"w.txt");
         DataTable yTable = new DataTable(dir+"y.txt");

         // Extract sample size and declare as an integer:
         int n = wTable.numRow;

         // Read in hyperparameter values:
         double sigsqBeta = new DataTable(dir+"sigsqBeta.txt").DoubleNumeric;
         double sigsqMuX = new DataTable(dir+"sigsqMuX.txt").DoubleNumeric;
         double Ax = new DataTable(dir+"Ax.txt").DoubleNumeric;
         double Aeps = new DataTable(dir+"Aeps.txt").DoubleNumeric;
         double tauZ = new DataTable(dir+"tauZ.txt").DoubleNumeric;
         
         // Read in number of Gibbs samples, burnin and thinning
         // factor values. If `nIterGibbs' is zero then variational
         // Bayes is used.
         int nIterGibbs = new DataTable(dir+"nIterGibbs.txt").IntNumeric;
         int nBurnin = new DataTable(dir+"nBurnin.txt").IntNumeric;
         int nThin = new DataTable(dir+"nThin.txt").IntNumeric;

         // Read in number of iterations for variational Bayes:
         int nIterVB = new DataTable(dir+"nIterVB.txt").IntNumeric;

         // Specify priors  
         Variable<double> beta0 = Variable.GaussianFromMeanAndVariance(0.0,sigsqBeta).Named("beta0");
         Variable<double> beta1 = Variable.GaussianFromMeanAndVariance(0.0,sigsqBeta).Named("beta1");
         Variable<double> muX = Variable.GaussianFromMeanAndVariance(0.0,sigsqMuX).Named("muX");
         Variable<double> tauX = Variable.GammaFromShapeAndRate(0.5,
             Variable.GammaFromShapeAndRate(0.5,Math.Pow(Ax,-2))).Named("tauX");
         Variable<double> tauEps = Variable.GammaFromShapeAndRate(0.5,
             Variable.GammaFromShapeAndRate(0.5,Math.Pow(Aeps,-2))).Named("tauEps");         

         // Specify the likelihood:
         Range index = new Range(n).Named("index");
         VariableArray<double> x =  Variable.Array<double>(index).Named("x");
         VariableArray<double> w =  Variable.Array<double>(index).Named("w");
         VariableArray<double> y =  Variable.Array<double>(index).Named("y");
         VariableArray<double> muY = Variable.Array<double>(index).Named("muY");
         x[index] = Variable.GaussianFromMeanAndPrecision(muX,tauX).ForEach(index);
         muY[index] = beta0 + beta1*x[index]; 
         y[index] = Variable.GaussianFromMeanAndPrecision(muY[index],tauEps);
         w[index] = Variable.GaussianFromMeanAndPrecision(x[index],tauZ);

         // Assign the regression input data, stored in "wTable" and
         // and "yTable", to the "w" and "y" variables:
         w.ObservedValue = wTable.arrayForIN;
         y.ObservedValue = yTable.arrayForIN;

         // Specify inference engine and algorithm:
         InferenceEngine engine = new InferenceEngine();

         if (nIterGibbs>0) {
            GibbsSampling gs = new GibbsSampling();
            gs.BurnIn = nBurnin ; gs.Thin = nThin;
            engine.Algorithm = gs;
            engine.NumberOfIterations =  nBurnin + nIterGibbs;
         } else {
            engine.Algorithm = new VariationalMessagePassing();
            engine.NumberOfIterations = nIterVB;
         }

         // Write the parameters of the posterior densities to files: 
         SaveData.SaveTable(engine.Infer<Gaussian>(beta0).GetMean(),dir+"mu.q.beta0.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(beta0).GetVariance(),dir+"sigsq.q.beta0.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(beta1).GetMean(),dir+"mu.q.beta1.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(beta1).GetVariance(),dir+"sigsq.q.beta1.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(muX).GetMean(),dir+"mu.q.muX.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(muX).GetVariance(),dir+"sigsq.q.muX.txt");         
         SaveData.SaveTable(engine.Infer<Gamma>(tauX),dir+"parms.q.tauX.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauEps),dir+"parms.q.tauEps.txt");
         SaveData.SaveTable(engine.Infer<DistributionStructArray<Gaussian,double>>(x),dir+"parms.q.x.txt");
     
      }
   }
}

/////////////////////////////  End of simpLinReg1IN ////////////////////////////

