//////////////////////////  C# programme: mcarSLR /////////////////////////

// C# file containing support functions for the R/InferNET code for the paper:
//
// Luts J, Shen SSJ, Ormerod JT and Wand MP (2014)
//    Semiparametric Regression Analysis via Infer.NET

// Example for Section 10 of the paper.

// Last changed: 22 JULY 2014 by J.T Ormerod

// Load required C# and Infer.NET libraries: 
using System; using System.Collections.Generic;
using System.Text; using System.IO;
using System.Reflection; using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
   class mcarSLR
   {
      public static void Main()
      {
         // Set full path-name of current working directory:
         string dir = ".\\";

         // Read input files into data tables: 
         DataTable xobsTable = new DataTable(dir+"xobs.txt");
         DataTable yobsTable = new DataTable(dir+"yobs.txt");
         DataTable ymisTable = new DataTable(dir+"ymis.txt");
         
         // Extract sample size and declare as an integer:
         int nmis = ymisTable.numRow;
         int nobs = yobsTable.numRow;
         
         // Read in hyperparameter values:
         double sigsqBeta = new DataTable(dir+"sigsqBeta.txt").DoubleNumeric;
         double sigsqMuX = new DataTable(dir+"sigsqMuX.txt").DoubleNumeric;
         double Ax = new DataTable(dir+"Ax.txt").DoubleNumeric;
         double Aeps = new DataTable(dir+"Aeps.txt").DoubleNumeric;

         // Read in number of Gibbs samples, burnin and thinning
         // factor values. If `nIterGibbs' is zero then variational
         // Bayes is used.
         int nIterGibbs = new DataTable(dir+"nIterGibbs.txt").IntNumeric;
         int nBurnin = new DataTable(dir+"nBurnin.txt").IntNumeric;
         int nThin = new DataTable(dir+"nThin.txt").IntNumeric;

         // Read in number of iterations for variational Bayes:
         int nIterVB = new DataTable(dir+"nIterVB.txt").IntNumeric;

         // Specify priors  
         Variable<double> beta0  = Variable.GaussianFromMeanAndVariance(0.0,sigsqBeta).Named("beta0");
         Variable<double> beta1  = Variable.GaussianFromMeanAndVariance(0.0,sigsqBeta).Named("beta1");
         Variable<double> muX    = Variable.GaussianFromMeanAndVariance(0.0,sigsqMuX).Named("muX");
         Variable<double> tauX = Variable.GammaFromShapeAndRate(0.5,
             Variable.GammaFromShapeAndRate(0.5,Math.Pow(Ax,-2))).Named("tauX");
         Variable<double> tauEps = Variable.GammaFromShapeAndRate(0.5,
             Variable.GammaFromShapeAndRate(0.5,Math.Pow(Aeps,-2))).Named("tauEps");   
             
         // Specify the likelihood:
         Range indexMis = new Range(nmis).Named("indexMis");
         Range indexObs = new Range(nobs).Named("indexObs");
         VariableArray<double> xobs =  Variable.Array<double>(indexObs).Named("xobs");
         VariableArray<double> xmis =  Variable.Array<double>(indexMis).Named("xmis");
         VariableArray<double> yobs =  Variable.Array<double>(indexObs).Named("yobs");
         VariableArray<double> ymis =  Variable.Array<double>(indexMis).Named("ymis");
         VariableArray<double> muYobs = Variable.Array<double>(indexObs).Named("muYobs");
         VariableArray<double> muYmis = Variable.Array<double>(indexMis).Named("muYmis");
         xobs[indexObs] = Variable.GaussianFromMeanAndPrecision(muX,tauX).ForEach(indexObs);
         xmis[indexMis] = Variable.GaussianFromMeanAndPrecision(muX,tauX).ForEach(indexMis);
         muYobs[indexObs] = beta0 + beta1*xobs[indexObs]; 
         muYmis[indexMis] = beta0 + beta1*xmis[indexMis]; 
         yobs[indexObs] = Variable.GaussianFromMeanAndPrecision(muYobs[indexObs],tauEps);
         ymis[indexMis] = Variable.GaussianFromMeanAndPrecision(muYmis[indexMis],tauEps);
         yobs.ObservedValue = yobsTable.arrayForIN;
         ymis.ObservedValue = ymisTable.arrayForIN;
         xobs.ObservedValue = xobsTable.arrayForIN;
        
         // Specify inference engine and algorithm:
         InferenceEngine engine = new InferenceEngine();

         if (nIterGibbs>0) {
            GibbsSampling gs = new GibbsSampling();
            gs.BurnIn = nBurnin ; gs.Thin = nThin;
            engine.Algorithm = gs;
            engine.NumberOfIterations =  nBurnin + nIterGibbs;
         } else {
            engine.Algorithm = new VariationalMessagePassing();
            engine.NumberOfIterations = nIterVB;
         }

         // Write the parameters of the posterior densities to files: 
         SaveData.SaveTable(engine.Infer<Gaussian>(beta0).GetMean(),dir+"mu.q.beta0.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(beta0).GetVariance(),dir+"sigsq.q.beta0.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(beta1).GetMean(),dir+"mu.q.beta1.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(beta1).GetVariance(),dir+"sigsq.q.beta1.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(muX).GetMean(),dir+"mu.q.muX.txt");
         SaveData.SaveTable(engine.Infer<Gaussian>(muX).GetVariance(),dir+"sigsq.q.muX.txt");         
         SaveData.SaveTable(engine.Infer<Gamma>(tauX),dir+"parms.q.tauX.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauEps),dir+"parms.q.tauEps.txt");
         SaveData.SaveTable(engine.Infer<DistributionStructArray<Gaussian,double>>(xmis),dir+"parms.q.xmis.txt");
     
      }
   }
}

/////////////////////////////  End of mcarSLR ////////////////////////////

