###################### R script: onionsIN #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 3 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set directories
dir.InferNET.support <- "../../InferNETSupport/"
dir.Figures <- "../../../Figures/"

# Load support files
source(paste(dir.InferNET.support, "InferNETSupport.R", sep = ""), chdir = TRUE)

# Set flags:
doData <- TRUE
doInferNET <- TRUE
doMCMC <- TRUE
plotFits <- TRUE
createPNG <- FALSE
removeTempFiles <- TRUE

# Number of trials to run example (for timing)
TRIALS <- 1

################################################################################ 

# Read in data into data frame named 'onions':
data("onions", package = "SemiPar")

# Use transformed versions of data for inputting into Infer.NET:
x1Orig <- onions[, 3]
x2Orig <- onions[, 1]
yOrig <- log(onions[, 2])
x2.mean <- mean(x2Orig)
y.mean <- mean(yOrig)
x2.std <- sd(x2Orig)
y.std <- sd(yOrig)
x1 <- x1Orig
x2 <- (x2Orig - x2.mean)/x2.std
y <- (yOrig - y.mean)/y.std

# Set up X, Z2 and C matrices:
n <- length(y)
X <- cbind(rep(1, n), x1, x2)
numIntKnots <- 1
intKnots.x2 <- quantile(unique(x2), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x2 <- c(1.05 * min(x2) - 0.05 * max(x2), 1.05 * max(x2) - 0.05 * min(x2))
Z2 <- ZOSull(x2, range.x2, intKnots.x2)



if (doData) {
  # Read in data into data frame named 'onions':
  data("onions", package = "SemiPar")
  
  # Use transformed versions of data for inputting into Infer.NET:
  x1Orig <- onions[, 3]
  x2Orig <- onions[, 1]
  yOrig <- log(onions[, 2])
  
  # Standardised data for MCMC:
  x2.mean <- mean(x2Orig)
  y.mean <- mean(yOrig)
  x2.std <- sd(x2Orig)
  y.std <- sd(yOrig)
  x1 <- x1Orig
  x2 <- (x2Orig - x2.mean)/x2.std
  y <- (yOrig - y.mean)/y.std
  n <- length(y)
  
  X <- cbind(rep(1, n), x1, x2)
  
  # Set up matrices for additive model:
  numIntKnots <- 1
  
  intKnots2 <- quantile(unique(x2), seq(0, 1, length = (numIntKnots + 2))[-c(1, 
    (numIntKnots + 2))])
  range.x2 <- c(min(x2), max(x2))
  Z2 <- ZOSull(x2, range.x2, intKnots2)
  numKnots2 <- ncol(Z2)
  
  Cmat <- cbind(X, Z2)
}

# Write input information to files:
write(ncol(X), "ncX.txt")
write(ncol(Z2), "numKnots.txt")
write(1, "numAddComp.txt")
write.table(Cmat, "Cmat.txt", sep = ",", row.names = FALSE, col.names = FALSE)
write.table(y, "y.txt", sep = ",", row.names = FALSE, col.names = FALSE)

# Set hyperparameters and write to files:
sigsqBeta <- 1e+08
Au <- 10^5
Aeps <- 10^5
write(sigsqBeta^(-1), "tauBeta.txt")
write(Au, "Au.txt")
write(Aeps, "Aeps.txt")

################################################################################ 

# Set Gibbs and variational Bayes iteration sizes.  If `nIterGibbs' is zero then
# variational Bayes used instead of Gibbs sampling:

nIterGibbs <- 0
nBurnin <- 1000
nThin <- 5
nIterVB <- 100
write(nIterVB, "nIterVB.txt")
write(nIterGibbs, "nIterGibbs.txt")
write(nBurnin, "nBurnin.txt")
write(nThin, "nThin.txt")

if (doInferNET) {
  # Run Infer.NET:
  vtimes <- c()
  for (trial in 1:TRIALS) {
    timeTaken <- system.time(RunInferNET(dir.InferNET.support, "onion.cs"))
    vtimes[trial] <- timeTaken[3]
  }
  
  # Read in posterior density parameters:
  mu.q.betau <- scan("mu.q.betau.txt")
  Sigma.q.betau <- as.matrix(read.table("Sigma.q.betau.txt"))
  parms.q.sigsqEps <- scan("parms.q.tauEps.txt", quiet = TRUE)
  parms.q.sigsqU <- scan("parms.q.tauu1.txt", quiet = TRUE)
  A.q.sigsqEps <- parms.q.sigsqEps[1]
  B.q.sigsqEps <- 1/parms.q.sigsqEps[2]
  A.q.sigsqU <- parms.q.sigsqU[1]
  B.q.sigsqU <- 1/parms.q.sigsqU[2]
  
  # Transform back to original units:
  mu.q.beta1TF <- (y.std) * mu.q.betau[2]
  sigsq.q.beta1TF <- (y.std^2) * Sigma.q.betau[2, 2]
  A.q.sigsqEpsTF <- A.q.sigsqEps
  B.q.sigsqEpsTF <- (y.std^2) * B.q.sigsqEps
}

################################################################################ 

if (doMCMC) {
  source("onionsMCMC.R")
}

################################################################################ 

if (plotFits) {
  load("onionsMCMCResults.RData")
  
  # Figure 1
  par(mfrow = c(1, 1), mai = c(1, 1, 0.1, 0.1))
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 1.5
  ylimVal <- c(3.3, 5.7)
  xlimVal <- c(17, 188)
  legendStr1 <- "Purnong Landing"
  legendStr2 <- "Virginia"
  legendStr3 <- "Infer.NET"
  legendStr4 <- "Infer.NET 95% credible sets"
  legendStr5 <- "MCMC"
  legendStr6 <- "MCMC 95% credible sets"
  
  # Fit line for x1:
  x2g <- seq(min(x2), max(x2), length = ng)
  Z2g <- ZOSull(x2g, range.x2, intKnots2)
  Cmatg0 <- cbind(rep(1, ng), rep(0, ng), x2g, Z2g)
  Cmatg1 <- cbind(rep(1, ng), rep(1, ng), x2g, Z2g)
  fhatg0 <- Cmatg0 %*% mu.q.betau
  fhatg1 <- Cmatg1 %*% mu.q.betau
  
  sdg0 <- sqrt(diag(Cmatg0 %*% Sigma.q.betau %*% t(Cmatg0)))
  sdg1 <- sqrt(diag(Cmatg1 %*% Sigma.q.betau %*% t(Cmatg1)))
  lowerg0 <- Cmatg0 %*% mu.q.betau + 2 * sdg0
  upperg0 <- Cmatg0 %*% mu.q.betau - 2 * sdg0
  lowerg1 <- Cmatg1 %*% mu.q.betau + 2 * sdg1
  upperg1 <- Cmatg1 %*% mu.q.betau - 2 * sdg1
  
  x2gOrig <- x2g * x2.std + x2.mean
  fhatg0Orig <- fhatg0 * y.std + y.mean
  fhatg1Orig <- fhatg1 * y.std + y.mean
  fhatg0OrigLower <- lowerg0 * y.std + y.mean
  fhatg1OrigLower <- lowerg1 * y.std + y.mean
  fhatg0OrigUpper <- upperg0 * y.std + y.mean
  fhatg1OrigUpper <- upperg1 * y.std + y.mean
  
  onionType <- rbind(matrix("Purnong Landing", 42, 1), matrix("Virginia", 42))
  onions$x <- x2Orig
  onions$y <- yOrig
  teller <<- 0
  ylimValue <- range(c(credUpper0, credLower0, fhatg0OrigLower, fhatg0OrigUpper, 
    credUpper1, credLower1, fhatg1OrigLower, fhatg1OrigUpper))
  
  pobj <- xyplot(y ~ x | onionType, data = onions, ylim = c(ylimValue[1] * 0.97, 
    ylimValue[2] * 1.02), strip = function(...) strip.default(..., style = 1), 
    subscripts = TRUE, panel = function(x, y, subscripts) {
      panel.grid()
      panel.xyplot(x, y, pch = 1, col = "darkgreen")
      teller <<- teller + 1
      if (teller == 1) {
        panel.superpose(x2gOrig, f2gOrig0, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 1, lwd = 1.5, col = "red")
        panel.superpose(x2gOrig, credLower0, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 2, lwd = 1.5, col = "red")
        panel.superpose(x2gOrig, credUpper0, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 2, lwd = 1.5, col = "red")
        panel.superpose(x2gOrig, fhatg0Orig, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 1, lwd = 1.5, col = "blue")
        panel.superpose(x2gOrig, fhatg0OrigLower, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 1.5, col = "blue")
        panel.superpose(x2gOrig, fhatg0OrigUpper, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 1.5, col = "blue")
        
      } else if (teller == 2) {
        panel.superpose(x2gOrig, f2gOrig1, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 1, lwd = 1.5, col = "red")
        panel.superpose(x2gOrig, credLower1, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 2, lwd = 1.5, col = "red")
        panel.superpose(x2gOrig, credUpper1, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 2, lwd = 1.5, col = "red")
        panel.superpose(x2gOrig, fhatg1Orig, seq(1, ng), matrix(1, ng), type = "l", 
          lty = 1, lwd = 1.5, col = "blue")
        panel.superpose(x2gOrig, fhatg1OrigLower, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 1.5, col = "blue")
        panel.superpose(x2gOrig, fhatg1OrigUpper, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 1.5, col = "blue")
        
        draw.key(list(border = TRUE, lines = list(lwd = c(1.5, 1.5, 1.5, 
          1.5, 1.5, 1.5), lty = c(1, 1, 2, 1, 2, 1), col = c(rgb(1, 0, 0, 
          alpha = 0), "blue", "blue", "red", "red", rgb(1, 0, 0, alpha = 0))), 
          text = list(lab = c(" ", "Infer.NET", "Infer.NET 95% credible sets", 
          "MCMC", "MCMC 95% credible sets", " "), cex = c(0.01, 0.7, 0.7, 
          0.7, 0.7, 0.01))), draw = TRUE, vp = viewport(x = unit(0.55, 
          "npc"), y = unit(0.87, "npc")))
        
        
      }
    }, xlab = list("density", cex = cex.labVal), ylab = list("log(yield)", cex = cex.labVal))
  
  png(paste(dir.Figures, "onions1.png", sep = ""), res = 144, width = 1000, height = 600)
  print(pobj)
  dev.off()
  
  # Figure 2
  
  par(mfrow = c(1, 3), mai = c(0.9, 0.1, 0.7, 0.1))
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 1.6
  cex.mainVal <- 2
  cex.legendVal <- 1
  legendStr1 <- "Infer.NET"
  legendStr2 <- "MCMC"
  
  png(paste(dir.Figures, "onions2a.png", sep = ""), res = 144, width = 700, height = 700)
  
  xg <- seq(mu.q.beta1TF - 3.5 * sqrt(sigsq.q.beta1TF), mu.q.beta1TF + 3.5 * sqrt(sigsq.q.beta1TF), 
    length = ng)
  beta0.postg <- dnorm(xg, mu.q.beta1TF, sqrt(sigsq.q.beta1TF))
  xlimVal <- range(xg)
  ylimVal <- range(c(beta0.postg, beta0.postg))
  plot(0, 0, type = "n", bty = "l", xlim = xlimVal, ylim = ylimVal, cex.lab = cex.labVal, 
    xlab = "", ylab = "approx. posterior density", main = "location", cex.main = cex.mainVal, 
    xpd = FALSE)
  text(-0.33, -3.9, expression(beta[1]), cex = cex.labVal, xpd = TRUE)
  lines(xg, beta0.postg, lty = 1, lwd = lwdVal, col = "blue", xpd = FALSE)
  dest <- bkde(beta1MCMCTF, bandwidth = dpik(beta1MCMCTF))
  lines(dest, lty = 1, lwd = lwdVal, col = "red", xpd = FALSE)
  legend(0.35 * xlimVal[1] + 0.65 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2), pch = cbind(NA, NA), lty = cbind(1, 1), col = cbind("blue", 
      "red"), cex = cex.legendVal, lwd = lwdVal, xpd = FALSE)
  abline(0, 0, xpd = FALSE)
  
  dev.off()
  
  png(paste(dir.Figures, "onions2b.png", sep = ""), res = 144, width = 700, height = 700)
  
  xLow <- 1/qgamma(0.99995, A.q.sigsqEpsTF, rate = B.q.sigsqEpsTF)
  xUpp <- 1/qgamma(5e-04, A.q.sigsqEpsTF, rate = B.q.sigsqEpsTF)
  xg <- seq(xLow, xUpp, length = ng)
  sigsqEps.postg <- exp(A.q.sigsqEpsTF * log(B.q.sigsqEpsTF) - lgamma(A.q.sigsqEpsTF) - 
    (A.q.sigsqEpsTF + 1) * log(xg) - B.q.sigsqEpsTF/(xg))
  xlimVal <- range(xg)
  ylimVal <- range(c(sigsqEps.postg, sigsqEps.postg))
  plot(0, 0, type = "n", bty = "l", xlim = range(xg), cex.lab = cex.labVal, xlab = "", 
    ylab = "approx. posterior density", ylim = range(sigsqEps.postg), main = "error variance", 
    cex.main = cex.mainVal, xpd = FALSE)
  text(0.0135, -52.5, expression(tau[epsilon]^-1), cex = cex.labVal, xpd = TRUE)
  lines(xg, sigsqEps.postg, lty = 1, lwd = lwdVal, col = "blue", xpd = FALSE)
  dest <- bkde(EpsSqMCMCTF, bandwidth = dpik(EpsSqMCMCTF))
  lines(dest, lty = 1, lwd = lwdVal, col = "red", xpd = FALSE)
  legend(0.35 * xlimVal[1] + 0.65 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2), pch = cbind(NA, NA), lty = cbind(1, 1), col = cbind("blue", 
      "red"), cex = cex.legendVal, lwd = lwdVal, xpd = FALSE)
  abline(0, 0, xpd = FALSE)
  
  dev.off()
  
  png(paste(dir.Figures, "onions2c.png", sep = ""), res = 144, width = 700, height = 700)
  
  N <- 1e+06
  CTC <- crossprod(Cmat)
  Dmat <- diag(c(0, 0, 0, rep(1, ncol(Z2))))
  sigmaSqEps <- 1/rgamma(N, A.q.sigsqEps, rate = B.q.sigsqEps)
  sigmaSqU <- 1/rgamma(N, A.q.sigsqU, rate = B.q.sigsqU)
  lamIN <- (sigmaSqEps/sigmaSqU)
  dfIN <- rep(NA, length(lamIN))
  for (i in 1:length(lamIN)) dfIN[i] <- matrix.trace(solve(CTC + lamIN[i] * Dmat) %*% 
    CTC)
  xLow <- 4.3
  xUpp <- 6.2
  xg <- seq(xLow, xUpp, length = ng)
  xlimVal <- range(xg)
  ylimVal <- cbind(0, 3.55)
  plot(0, 0, type = "n", bty = "l", xlim = range(xg), cex.lab = cex.labVal, xlab = "", 
    ylab = "approx. posterior density", main = "", ylim = ylimVal, cex.main = cex.mainVal, 
    xpd = FALSE)
  text(5.2, -0.825, expression("effective degrees of freedom"), cex = cex.labVal, 
    xpd = TRUE)
  text(5.04, 4.17, bquote(bold("effective degrees of freedom")), cex = cex.mainVal, 
    xpd = TRUE)
  dest <- bkde(dfIN, bandwidth = dpik(dfIN))
  lines(dest, lty = 1, lwd = lwdVal, col = "blue", xpd = FALSE)
  dest <- bkde(dfMCMC, bandwidth = dpik(dfMCMC))
  lines(dest, lty = 1, lwd = lwdVal, col = "red", xpd = FALSE)
  legend(0.95 * xlimVal[1] + 0.05 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2), pch = cbind(NA, NA), lty = cbind(1, 1), col = cbind("blue", 
      "red"), cex = cex.legendVal, lwd = lwdVal, xpd = FALSE)
  abline(0, 0, xpd = FALSE)
  
  dev.off()
}

################################################################################

# Code to generate table 2.

library("xtable")

# Column 1

col1 <- c("$\\beta_0$", "$\\beta_1$", "$\\beta_2$",
          "$\\tau_\\varepsilon$", "$\\tau_u$")

# Column 2		
		
tauEpsMCMC <- 1/sigEpsMCMC^2
tauUMCMC <- 1/sigUMCMC^2
		
col2 <- c(round(apply(betaMCMC, 1, mean), 3),
          round(mean(tauEpsMCMC), 1),
          round(mean(tauUMCMC), 3))

# Column 3

beta0MCMC.L <- round(quantile(betaMCMC[1, ], 0.025), 3)
beta1MCMC.L <- round(quantile(betaMCMC[2, ], 0.025), 3)
beta2MCMC.L <- round(quantile(betaMCMC[3, ], 0.025), 3)
tauEpsMCMC.L <- round(quantile(tauEpsMCMC, 0.025), 1)
tauUMCMC.L <- round(quantile(tauUMCMC, 0.025), 3)

beta0MCMC.U <- round(quantile(betaMCMC[1, ], 0.975), 3)
beta1MCMC.U <- round(quantile(betaMCMC[2, ], 0.975), 3)
beta2MCMC.U <- round(quantile(betaMCMC[3, ], 0.975), 3)
tauEpsMCMC.U <- round(quantile(tauEpsMCMC, 0.975), 1)
tauUMCMC.U <- round(quantile(tauUMCMC, 0.975), 3)

col3 <-  c(paste("(", beta0MCMC.L, ",", beta0MCMC.U, ")", sep = ""), 
           paste("(", beta1MCMC.L, ",", beta1MCMC.U, ")", sep = ""), 
           paste("(", beta2MCMC.L, ",", beta2MCMC.U, ")", sep = ""), 		
           paste("(", tauEpsMCMC.L, ",", tauEpsMCMC.U, ")", sep = ""), 
           paste("(", tauUMCMC.L, ",", tauUMCMC.U, ")", sep = ""))

# Column 4

beta0VB.M  <- round(mu.q.betau[1], 3)
beta1VB.M  <- round(mu.q.betau[2], 3)
beta2VB.M  <- round(mu.q.betau[3], 3)
tauEpsVB.M <- round(A.q.sigsqEps/B.q.sigsqEps, 1)
tauUVB.M   <- round(A.q.sigsqU/B.q.sigsqU, 3)

col4 <- c(beta0VB.M, beta1VB.M, beta2VB.M, tauEpsVB.M, tauUVB.M)

# Column 5

beta0VB.L  <- round(qnorm(0.025, mu.q.betau[1], sqrt(Sigma.q.betau[1, 1])), 3)
beta1VB.L  <- round(qnorm(0.025, mu.q.betau[2], sqrt(Sigma.q.betau[2, 2])), 3)
beta2VB.L  <- round(qnorm(0.025, mu.q.betau[3], sqrt(Sigma.q.betau[3, 3])), 3)
tauEpsVB.L <- round(qgamma(0.025, A.q.sigsqEps, B.q.sigsqEps), 1)
tauUVB.L   <- round(qgamma(0.025, A.q.sigsqU, B.q.sigsqU), 3)

beta0VB.U  <- round(qnorm(0.975, mu.q.betau[1], sqrt(Sigma.q.betau[1, 1])), 3)
beta1VB.U  <- round(qnorm(0.975, mu.q.betau[2], sqrt(Sigma.q.betau[2, 2])), 3)
beta2VB.U  <- round(qnorm(0.975, mu.q.betau[3], sqrt(Sigma.q.betau[3, 3])), 3)
tauEpsVB.U <- round(qgamma(0.975, A.q.sigsqEps, B.q.sigsqEps), 1)
tauUVB.U   <- round(qgamma(0.975, A.q.sigsqU, B.q.sigsqU), 3)

col5 <-  c(paste("(", beta0VB.L, ",", beta0VB.U, ")", sep = ""), 
           paste("(", beta1VB.L, ",", beta1VB.U, ")", sep = ""), 
           paste("(", beta2VB.L, ",", beta2VB.U, ")", sep = ""), 		
           paste("(", tauEpsVB.L, ",", tauEpsVB.U, ")", sep = ""), 
           paste("(", tauUVB.L, ",", tauUVB.U, ")", sep = ""))

# Put all columns together
 
tab <- cbind(col1, col2, col3, col4, col5)
colnames(tab) <- c("Parameter", "MCMC_mean", "MCMC_cred", "VB_mean", "VB_cred")

print(xtable(tab))

library("huxtable")

tab2 <- hux(Parameter = col1, MCMC_mean = col2, MCMC_cred = col3,
            VB_mean = col4, VB_cred = col5)

bold(tab2)[1, ]           <- FALSE
top_border(tab2)[1, ]     <- 1
bottom_border(tab2)[5, ]  <- 1

align(tab2)[, 1]          <- "left"
align(tab2)[, 2]          <- "right"
align(tab2)[, 3]          <- "center"
align(tab2)[, 4]          <- "right"
align(tab2)[, 5]          <- "center"

number_format(tab2)[, 2]  <- "%1.3f"
number_format(tab2)[4, 2] <- "%2.1f"
number_format(tab2)[, 4]  <- "%1.3f"
number_format(tab2)[4, 4] <- "%2.1f"

print(tab2) 

################################################################################ 

if (removeTempFiles) {
  # Remove all temporary files:
  file.remove("tauBeta.txt")
  file.remove("y.txt")
  file.remove("Sigma.q.betau.txt")
  file.remove("parms.q.tauEps.txt")
  file.remove("parms.q.tauu1.txt")
  file.remove("mu.q.betau.txt")
  file.remove("ncX.txt")
  file.remove("numAddComp.txt")
  file.remove("numKnots.txt")
  file.remove("Aeps.txt")
  file.remove("Au.txt")
  file.remove("Cmat.txt")
  file.remove("nBurnin.txt")
  file.remove("nIterGibbs.txt")
  file.remove("nIterVB.txt")
  file.remove("nThin.txt")
  file.remove("onionsModel.txt")
  file.remove("onionsMCMCResults.RData")
}

############### End of onionsIN ###############
 
