###################### R script: onionsMCMC #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 3 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set tasks
doData <- FALSE

# Set MCMC parameters:
nBurnin <- 50000  # Length of burn-in.
nIter <- 1e+06  # Size of the kept sample.
nThin <- 5  # Thinning factor. 

# Number of trials to run example (for timing)
TRIALS <- 1

################################################################################ 

if (doData) {
  # Read in data into data frame named 'onions':
  data("onions", package = "SemiPar")
  
  # Use transformed versions of data for inputting into Infer.NET:
  x1Orig <- onions[, 3]
  x2Orig <- onions[, 1]
  yOrig <- log(onions[, 2])
  
  # Standardised data for MCMC:
  x2.mean <- mean(x2Orig)
  y.mean <- mean(yOrig)
  x2.std <- sd(x2Orig)
  y.std <- sd(yOrig)
  x1 <- x1Orig
  x2 <- (x2Orig - x2.mean)/x2.std
  y <- (yOrig - y.mean)/y.std
  
  # Set up matrices for additive model:
  numIntKnots <- 25
  
  intKnots2 <- quantile(unique(x2), seq(0, 1, length = (numIntKnots + 2))[-c(1, 
    (numIntKnots + 2))])
  range.x2 <- c(min(x2), max(x2))
  Z2 <- ZOSull(x2, range.x2, intKnots2)
  numKnots2 <- ncol(Z2)
}

################################################################################ 

# Specify model in BUGS:
onionsModel <- function() {
  for (i in 1:n) {
    mu[i] <- (beta0 + beta1 * x1[i] + beta2 * x2[i] + inprod(u2[], Z2[i, ]))
    y[i] ~ dnorm(mu[i], tauEps)
  }
  for (k in 1:numKnots2) {
    u2[k] ~ dnorm(0, tauU2)
  }
  
  beta0 ~ dnorm(0, 1e-08)
  beta1 ~ dnorm(0, 1e-08)
  beta2 ~ dnorm(0, 1e-08)
  cuinv ~ dgamma(0.5, 1e-10)
  tauU2 ~ dgamma(0.5, cuinv)
  cepsinv ~ dgamma(0.5, 1e-10)
  tauEps ~ dgamma(0.5, cepsinv)
  
}
modelFileName <- file.path(getwd(), "onionsModel.txt")
writeModel(onionsModel, modelFileName)

# Store data in a list in format required by BRugs:
allData <- list(n = length(y), numKnots2 = numKnots2, x1 = x1, x2 = x2, y = y, Z2 = Z2)

# Obtain MCMC samples for each parameter using BRugs:
parInits <- list(list(beta0 = 0, beta1 = 0, beta2 = 0, u2 = rep(0, numKnots2), cuinv = 1, 
  cepsinv = 1, tauU2 = 1, tauEps = 1))

times.brugs <- c()
for (trial in 1:TRIALS) {
  # Do MCMC using BRugs:
  bval1 <- proc.time()
  BRugsFit(data = allData, inits = parInits, parametersToSave = c("beta0", "beta1", 
    "beta2", "u2", "tauU2", "tauEps"), nBurnin = nBurnin, nIter = nIter, nThin = nThin, 
    modelFile = "onionsModel.txt", numChains = 1)
  eval1 <- proc.time()
  times.brugs[trial] <- eval1[3] - bval1[3]
}

betaMCMC <- samplesSample("beta0")
betaMCMC <- rbind(betaMCMC, samplesSample("beta1"))
betaMCMC <- rbind(betaMCMC, samplesSample("beta2"))

u2MCMC <- samplesSample("u2[1]")
for (k in 2:numKnots2) u2MCMC <- rbind(u2MCMC, samplesSample(paste("u2[", k, "]", 
  sep = "")))

uMCMC <- u2MCMC

################################################################################ 

# Draw plots
par(mfrow = c(1, 1))
ng <- 401
ylimVal <- c(3.3, 5.7)
xlimVal <- c(17, 188)

# Plot second function fit (with other variables set at their averages):
x2g <- seq(min(x2), max(x2), length = ng)
Z2g <- ZOSull(x2g, range.x2, intKnots2)
Cmatg0 <- cbind(rep(1, ng), rep(0, ng), x2g, Z2g)
Cmatg1 <- cbind(rep(1, ng), rep(1, ng), x2g, Z2g)
betauMCMC <- rbind(betaMCMC, uMCMC)
fhatgMCMC0 <- Cmatg0 %*% betauMCMC
fhatgMCMC1 <- Cmatg1 %*% betauMCMC
f2MCMCOrig0 <- y.std * fhatgMCMC0 + y.mean
f2MCMCOrig1 <- y.std * fhatgMCMC1 + y.mean
credLower0 <- apply(f2MCMCOrig0, 1, quantile, 0.025)
credUpper0 <- apply(f2MCMCOrig0, 1, quantile, 0.975)
credLower1 <- apply(f2MCMCOrig1, 1, quantile, 0.025)
credUpper1 <- apply(f2MCMCOrig1, 1, quantile, 0.975)
f2gOrig0 <- apply(f2MCMCOrig0, 1, mean)
f2gOrig1 <- apply(f2MCMCOrig1, 1, mean)

x2gOrig <- x2.std * x2g + x2.mean
plot(x2gOrig, f2gOrig0, type = "l", bty = "l", xlab = "density", ylab = "log(yield)", 
  ylim = ylimVal)
lines(x2gOrig, f2gOrig1, type = "l", bty = "l")
lines(x2gOrig, credLower0, type = "l", bty = "l")
lines(x2gOrig, credLower1, type = "l", bty = "l")
lines(x2gOrig, credUpper0, type = "l", bty = "l")
lines(x2gOrig, credUpper1, type = "l", bty = "l")
points(x2Orig[1:42], yOrig[1:42], , cex = 0.7, col = "dodgerblue")
points(x2Orig[43:84], yOrig[43:84], cex = 0.7, col = "tomato")

# Backtransform to original units
beta1MCMCTF <- (y.std) * samplesSample("beta1")
EpsSqMCMCTF <- (y.std^2) * 1/samplesSample("tauEps")

# Compute degrees of freedom MCMC values:
sigEpsMCMC <- 1/sqrt(samplesSample("tauEps"))
sigUMCMC <- 1/sqrt(samplesSample("tauU2"))

CTC <- crossprod(cbind(rep(1, length(x1)), x1, x2, Z2))
Dmat <- diag(c(0, 0, 0, rep(1, numKnots2)))
lamMCMC <- (sigEpsMCMC/sigUMCMC)^2
dfMCMC <- rep(NA, length(lamMCMC))
for (i in 1:length(lamMCMC)) dfMCMC[i] <- matrix.trace(solve(CTC + lamMCMC[i] * Dmat) %*% 
  CTC)

save(beta1MCMCTF, credLower0, credLower1, credUpper0, credUpper1, dfMCMC, EpsSqMCMCTF, 
  f2gOrig0, f2gOrig1, file = "onionsMCMCResults.RData")

########## End of onionsMCMC ##########



