###################### R script: lasso #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 6 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set directories
dir.InferNET.support <- "..\\..\\InferNETSupport\\"
dir.Figures <- "..\\..\\..\\Figures\\"

# Load support files
source(file.path(dir.InferNET.support, "InferNETSupport.R"), chdir = TRUE)

# Set tasks:
doMCMC <- TRUE
doInferNET <- TRUE
plotFits <- TRUE
createPNG <- TRUE
REMOVE.INPUTS <- TRUE
REMOVE.OUTPUTS <- TRUE

# MCMC
nBurnin <- 5000  # Length of burn-in.
nIter <- 1e+05  # Size of the kept sample.
nThin <- 1 # Thinning factor. 

# Number of trials to run example (for timing)
TRIALS <- 1

# Read in data into data frame named 'spinalbone':
spinalbone <- read.table("sbmd.txt")
spinalbone <- spinalbone[which(spinalbone[, 4] == "fem"), ]  # the females
x1Orig <- matrix(NA, nrow(spinalbone), 1)  # black
x2Orig <- matrix(NA, nrow(spinalbone), 1)  # hispanic
x3Orig <- matrix(NA, nrow(spinalbone), 1)  # white

for (i in (1:nrow(spinalbone))) {
  if (spinalbone[i, 2] == "Asian") {
    x1Orig[i] <- 0
    x2Orig[i] <- 0
    x3Orig[i] <- 0
  } else if (spinalbone[i, 2] == "Black") {
    x1Orig[i] <- 1
    x2Orig[i] <- 0
    x3Orig[i] <- 0
  } else if (spinalbone[i, 2] == "Hispanic") {
    x1Orig[i] <- 0
    x2Orig[i] <- 1
    x3Orig[i] <- 0
  } else if (spinalbone[i, 2] == "White") {
    x1Orig[i] <- 0
    x2Orig[i] <- 0
    x3Orig[i] <- 1
  }
}

x4Orig <- as.numeric(levels(spinalbone[, 3])[spinalbone[, 3]])  # age
yOrig <- as.numeric(levels(spinalbone[, 5])[spinalbone[, 5]])

# Use transformed versions of data for inputting into Infer.NET:
x1 <- x1Orig
x2 <- x2Orig
x3 <- x3Orig
x4.mean <- mean(x4Orig)
x4.std <- sd(x4Orig)
x4 <- (x4Orig - x4.mean)/x4.std
y.mean <- mean(yOrig)
y.std <- sd(yOrig)
y <- (yOrig - y.mean)/y.std

# Set up X, Z2 and C matrices:
n <- length(y)
X <- cbind(x1, x2, x3, rep(1, n), x4)
numIntKnots <- 15
intKnots.x4 <- quantile(unique(x4), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x4 <- c(1.05 * min(x4) - 0.05 * max(x4), 1.05 * max(x4) - 0.05 * min(x4))
Z4 <- ZOSull(x4, range.x4, intKnots.x4)
ids <- unique(spinalbone[, 1])
Z123 <- matrix(0, n, length(ids))
for (i in (1:length(ids))) {
  id <- ids[i]
  Z123[which(spinalbone[, 1] == id), i] <- 1
}
Cmat <- cbind(X, Z123, Z4)
numSubjects <- length(ids)

# Write input information to files:
write(ncol(X), "ncX.txt")
write(ncol(Z4), "numKnots.txt")
write(1, "numAddComp.txt")
write.table(Cmat, "Cmat.txt", sep = ", ", row.names = FALSE, col.names = FALSE)
write.table(y, "y.txt", sep = ", ", row.names = FALSE, col.names = FALSE)
write(numSubjects, "numSubjects.txt")

# Set hyperparameters and write to files:
sigsqBeta <- 1e+08
Au <- 10^5
Aeps <- 10^5
AU <- 10^5
write(sigsqBeta^(-1), "tauBeta.txt")
write(Au, "Au.txt")
write(AU, "AU.txt")
write(Aeps, "Aeps.txt")

# Set Gibbs and variational Bayes iteration sizes.  If `nIterGibbs' is zero then
# variational Bayes used instead of Gibbs sampling:
nIterGibbs <- 0
nBurnin <- 1
nThin <- 1
nIterVB <- 100
write(nIterVB, "nIterVB.txt")
write(nIterGibbs, "nIterGibbs.txt")
write(nBurnin, "nBurnin.txt")
write(nThin, "nThin.txt")

if (doInferNET) {
  times.Infer.NET <- c()
  for (trial in 1:TRIALS) {
    # Run Infer.NET:
    timeTaken <- system.time(RunInferNET(dir.InferNET.support, "spinalBone.cs"))
    print(timeTaken)
    
    times.Infer.NET[trial] <- timeTaken[3]
  }
  
  # Read in posterior density parameters:
  mu.q.betau <- scan("mu.q.betaUu.txt")
  Sigma.q.betau <- as.matrix(read.table("Sigma.q.betaUu.txt"))
  parms.q.sigsqEps <- scan("parms.q.tauEps.txt", quiet = TRUE)
  parms.q.sigsqU <- scan("parms.q.tauU1.txt", quiet = TRUE)
  
  A.q.sigsqEps <- parms.q.sigsqEps[1]
  B.q.sigsqEps <- 1/parms.q.sigsqEps[2]
  A.q.sigsqU <- parms.q.sigsqU[1]
  B.q.sigsqU <- 1/parms.q.sigsqU[2]
  
  # Transform back to original units:
  mu.q.beta1TF <- (y.std) * mu.q.betau[1]
  sigsq.q.beta1TF <- (y.std^2) * Sigma.q.betau[1, 1]
  mu.q.beta2TF <- (y.std) * mu.q.betau[2]
  sigsq.q.beta2TF <- (y.std^2) * Sigma.q.betau[2, 2]
  mu.q.beta3TF <- (y.std) * mu.q.betau[3]
  sigsq.q.beta3TF <- (y.std^2) * Sigma.q.betau[3, 3]
}

if (plotFits) {
  # Figure 1
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 1
  xlimVal <- c(9, 26)
  ylimVal <- c(0.5, 1.425)
  legendStr1 <- "Infer.NET"
  legendStr2 <- "Infer.NET 95% credible sets"
  
  # Fit line for x4:
  x4g <- seq(min(x4), max(x4), length = ng)
  Z4g <- ZOSull(x4g, range.x4, intKnots.x4)
  CmatgAsian <- cbind(rep(0, ng), rep(0, ng), rep(0, ng), rep(1, ng), x4g, matrix(0, 
    ng, ncol(Z123)), Z4g)
  CmatgBlack <- cbind(rep(1, ng), rep(0, ng), rep(0, ng), rep(1, ng), x4g, matrix(0, 
    ng, ncol(Z123)), Z4g)
  CmatgHispanic <- cbind(rep(0, ng), rep(1, ng), rep(0, ng), rep(1, ng), x4g, matrix(0, 
    ng, ncol(Z123)), Z4g)
  CmatgWhite <- cbind(rep(0, ng), rep(0, ng), rep(1, ng), rep(1, ng), x4g, matrix(0, 
    ng, ncol(Z123)), Z4g)
  fhatgAsian <- CmatgAsian %*% mu.q.betau
  fhatgBlack <- CmatgBlack %*% mu.q.betau
  fhatgHispanic <- CmatgHispanic %*% mu.q.betau
  fhatgWhite <- CmatgWhite %*% mu.q.betau
  
  sdgAsian <- sqrt(diag(CmatgAsian %*% Sigma.q.betau %*% t(CmatgAsian)))
  sdgBlack <- sqrt(diag(CmatgBlack %*% Sigma.q.betau %*% t(CmatgBlack)))
  sdgHispanic <- sqrt(diag(CmatgHispanic %*% Sigma.q.betau %*% t(CmatgHispanic)))
  sdgWhite <- sqrt(diag(CmatgWhite %*% Sigma.q.betau %*% t(CmatgWhite)))
  lowergAsian <- CmatgAsian %*% mu.q.betau - 2 * sdgAsian
  uppergAsian <- CmatgAsian %*% mu.q.betau + 2 * sdgAsian
  lowergBlack <- CmatgBlack %*% mu.q.betau - 2 * sdgBlack
  uppergBlack <- CmatgBlack %*% mu.q.betau + 2 * sdgBlack
  lowergHispanic <- CmatgHispanic %*% mu.q.betau - 2 * sdgHispanic
  uppergHispanic <- CmatgHispanic %*% mu.q.betau + 2 * sdgHispanic
  lowergWhite <- CmatgWhite %*% mu.q.betau - 2 * sdgWhite
  uppergWhite <- CmatgWhite %*% mu.q.betau + 2 * sdgWhite
  
  x4gOrig <- x4g * x4.std + x4.mean
  fhatgAsianOrig <- fhatgAsian * y.std + y.mean
  fhatgBlackOrig <- fhatgBlack * y.std + y.mean
  fhatgHispanicOrig <- fhatgHispanic * y.std + y.mean
  fhatgWhiteOrig <- fhatgWhite * y.std + y.mean
  fhatgAsianOrigLower <- lowergAsian * y.std + y.mean
  fhatgAsianOrigUpper <- uppergAsian * y.std + y.mean
  fhatgBlackOrigLower <- lowergBlack * y.std + y.mean
  fhatgBlackOrigUpper <- uppergBlack * y.std + y.mean
  fhatgHispanicOrigLower <- lowergHispanic * y.std + y.mean
  fhatgHispanicOrigUpper <- uppergHispanic * y.std + y.mean
  fhatgWhiteOrigLower <- lowergWhite * y.std + y.mean
  fhatgWhiteOrigUpper <- uppergWhite * y.std + y.mean
  
  # Specify model:
  
  SBMDfemModel <- function() {
    for (i in 1:numObs) {
      mu[i] <- beta0plusUSbj[idnum[i]] + beta1 * black[i] + beta2 * hispanic[i] + 
        beta3 * white[i] + betaAge * age[i] + inprod(uSpl[], Zspl[i, ])
      spnbmd[i] ~ dnorm(mu[i], tauEps)
    }
    
    for (iSbj in 1:numSbj) {
      beta0plusUSbj[iSbj] ~ dnorm(beta0, tauSbj)
    }
    
    for (iSpl in 1:numSpl) {
      uSpl[iSpl] ~ dnorm(0, tauSpl)
    }
    
    beta0 ~ dnorm(0, 1e-10)
    beta1 ~ dnorm(0, 1e-10)
    beta2 ~ dnorm(0, 1e-10)
    beta3 ~ dnorm(0, 1e-10)
    betaAge ~ dnorm(0, 1e-10)
    
    csplinv ~ dgamma(0.5, 1e-10)
    tauSpl ~ dgamma(0.5, csplinv)
    
    csbjinv ~ dgamma(0.5, 1e-10)
    tauSbj ~ dgamma(0.5, csbjinv)
    
    cepsinv ~ dgamma(0.5, 1e-10)
    tauEps ~ dgamma(0.5, cepsinv)
    
    sigSbj <- 1/sqrt(tauSbj)
    sigSpl <- 1/sqrt(tauSpl)
    sigEps <- 1/sqrt(tauEps)
  }
  
  fileName <- file.path(getwd(), "SBMDfemModel.txt")
  writeModel(SBMDfemModel, fileName)
  
  
  # Fit model using MCMC via BRugs:
  numObs <- nrow(Cmat)
  numSpl <- ncol(Z4)
  numSbj <- length(ids)
  black <- Cmat[, 1]
  hispanic <- Cmat[, 2]
  white <- Cmat[, 3]
  age <- Cmat[, 5]
  spnbmd <- y
  Zspl <- Z4
  
  idnum <- as.vector(matrix(NA, length(y), 1))
  for (i in 1:ncol(Z123)) {
    idnum[which(Z123[, i] == 1)] <- i
  }
  
  allData <- list(numObs = numObs, numSpl = numSpl, numSbj = numSbj, idnum = idnum, 
    black = black, hispanic = hispanic, white = white, age = age, spnbmd = spnbmd, 
    Zspl = Zspl)
  
  parInits <- list(list(beta0 = 0, beta1 = 0, beta2 = 0, beta3 = 0, betaAge = 0, 
    beta0plusUSbj = rep(0, numSbj), uSpl = rep(0, numSpl), csplinv = 1, csbjinv = 1, 
    cepsinv = 1, tauSpl = 1, tauEps = 1, tauSbj = 1))
  
  times.brugs <- c()
  for (trial in 1:TRIALS) {
    # Do MCMC using BRugs:
    bval1 <- proc.time()
    BRugsFit(data = allData, inits = parInits, parametersToSave = c("beta0", 
      "beta1", "beta2", "beta3", "betaAge", "uSpl", "sigSbj", "sigSpl", "sigEps"), 
      nBurnin = nBurnin, nIter = nIter, nThin = nThin, modelFile = "SBMDfemModel.txt", 
      numChains = 1)
    eval1 <- proc.time()
    times.brugs[trial] <- eval1[3] - bval1[3]
  }
  
  sbeta0MCMC <- samplesSample("beta0")
  sbeta1MCMC <- samplesSample("beta1")
  sbeta2MCMC <- samplesSample("beta2")
  sbeta3MCMC <- samplesSample("beta3")
  
  sbetaAgeMCMC <- samplesSample("betaAge")
  
  suSplMCMC <- samplesSample("uSpl[1]")
  for (k in 2:numSpl) {
    suSplMCMC <- rbind(suSplMCMC, samplesSample(paste("uSpl[", k, "]", sep = "")))
  }
  
  # Convert to parameters original scale:
  beta1MCMC <- sbeta1MCMC * (y.std)
  beta2MCMC <- sbeta2MCMC * (y.std)
  beta3MCMC <- sbeta3MCMC * (y.std)
  
  CmatgAsianMCMC <- cbind(rep(0, ng), rep(0, ng), rep(0, ng), rep(1, ng), x4g, 
    Z4g)
  CmatgBlackMCMC <- cbind(rep(1, ng), rep(0, ng), rep(0, ng), rep(1, ng), x4g, 
    Z4g)
  CmatgHispanicMCMC <- cbind(rep(0, ng), rep(1, ng), rep(0, ng), rep(1, ng), x4g, 
    Z4g)
  CmatgWhiteMCMC <- cbind(rep(0, ng), rep(0, ng), rep(1, ng), rep(1, ng), x4g, 
    Z4g)
  
  mu.q.betauMCMC <- rbind(sbeta1MCMC, sbeta2MCMC, sbeta3MCMC, sbeta0MCMC, sbetaAgeMCMC, 
    suSplMCMC)
  
  fhatgAsianMCMC <- CmatgAsianMCMC %*% mu.q.betauMCMC
  fhatgBlackMCMC <- CmatgBlackMCMC %*% mu.q.betauMCMC
  fhatgHispanicMCMC <- CmatgHispanicMCMC %*% mu.q.betauMCMC
  fhatgWhiteMCMC <- CmatgWhiteMCMC %*% mu.q.betauMCMC
  
  fhatgAsianOrigMCMC <- fhatgAsianMCMC * y.std + y.mean
  fhatgBlackOrigMCMC <- fhatgBlackMCMC * y.std + y.mean
  fhatgHispanicOrigMCMC <- fhatgHispanicMCMC * y.std + y.mean
  fhatgWhiteOrigMCMC <- fhatgWhiteMCMC * y.std + y.mean
  
  # Extract data and plot results
  
  library("lattice")
  sbmd <- read.table("sbmd.txt", header = TRUE)
  
  fem.inds <- (1:nrow(sbmd))[as.character(sbmd$sex) == "fem"]
  sbmd.fem <- sbmd[fem.inds, ]
  ethnic <- as.character(sbmd.fem$ethnic)
  age <- sbmd.fem$age
  spnsbmd <- sbmd.fem$spnbmd
  idnum <- sbmd.fem$idnum
  teller <<- 0
  
  pobj <- xyplot(spnbmd ~ age | ethnic, data = sbmd.fem, groups = sbmd.fem$idnum, 
    ylim = c(0.5, 1.6), strip = function(...) strip.default(..., style = 1), 
    subscripts = TRUE, panel = function(x, y, subscripts, groups) {
      panel.grid()
      panel.xyplot(x, y, pch = 1, col = "darkgreen")
      panel.superpose(x, y, subscripts, groups, type = "l", lty = 1, lwd = 1, 
        col = "darkgreen")
      teller <<- teller + 1
      if (teller == 1) {
        # asian
        panel.superpose(x4gOrig, apply(fhatgAsianOrigMCMC, 1, mean), seq(1, 
          ng), matrix(1, ng), type = "l", lty = 2, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgAsianOrigMCMC, 1, quantile, 0.025), 
          seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgAsianOrigMCMC, 1, quantile, 0.975), 
          seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, col = "red")
        
        panel.superpose(x4gOrig, fhatgAsianOrig, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgAsianOrigLower, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgAsianOrigUpper, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
        
        draw.key(list(border = TRUE, lines = list(lwd = c(2, 2, 2, 2, 2, 
          2), lty = c(2, 2, 3, 2, 3, 2), col = c(rgb(1, 0, 0, alpha = 0), 
          "blue", "blue", "red", "red", rgb(1, 0, 0, alpha = 0))), text = list(lab = c(" ", 
          "Infer.NET", "Infer.NET 95% credible sets", "MCMC", "MCMC 95% credible sets", 
          " "), cex = c(0.01, 0.7, 0.7, 0.7, 0.7, 0.01))), draw = TRUE, vp = viewport(x = unit(0.55, 
          "npc"), y = unit(0.87, "npc")))
      } else if (teller == 2) {
        # black
        panel.superpose(x4gOrig, apply(fhatgBlackOrigMCMC, 1, mean), seq(1, 
          ng), matrix(1, ng), type = "l", lty = 2, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgBlackOrigMCMC, 1, quantile, 0.025), 
          seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgBlackOrigMCMC, 1, quantile, 0.975), 
          seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, col = "red")
        
        panel.superpose(x4gOrig, fhatgBlackOrig, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgBlackOrigLower, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgBlackOrigUpper, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
      } else if (teller == 3) {
        # hispanic
        panel.superpose(x4gOrig, apply(fhatgHispanicOrigMCMC, 1, mean), seq(1, 
          ng), matrix(1, ng), type = "l", lty = 2, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgHispanicOrigMCMC, 1, quantile, 
          0.025), seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, 
          col = "red")
        panel.superpose(x4gOrig, apply(fhatgHispanicOrigMCMC, 1, quantile, 
          0.975), seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, 
          col = "red")
        
        panel.superpose(x4gOrig, fhatgHispanicOrig, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 2, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgHispanicOrigLower, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgHispanicOrigUpper, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
      } else if (teller == 4) {
        # white
        panel.superpose(x4gOrig, apply(fhatgWhiteOrigMCMC, 1, mean), seq(1, 
          ng), matrix(1, ng), type = "l", lty = 2, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgWhiteOrigMCMC, 1, quantile, 0.025), 
          seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, col = "red")
        panel.superpose(x4gOrig, apply(fhatgWhiteOrigMCMC, 1, quantile, 0.975), 
          seq(1, ng), matrix(1, ng), type = "l", lty = 3, lwd = 3, col = "red")
        
        panel.superpose(x4gOrig, fhatgWhiteOrig, seq(1, ng), matrix(1, ng), 
          type = "l", lty = 2, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgWhiteOrigLower, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
        panel.superpose(x4gOrig, fhatgWhiteOrigUpper, seq(1, ng), matrix(1, 
          ng), type = "l", lty = 3, lwd = 3, col = "blue")
      }
    }, xlab = list("age in years", cex = cex.labVal), ylab = list("spinal bone mineral density", 
      cex = cex.labVal))
  
  
  
  if (createPNG) {
    png(paste(dir.Figures, "spinalbone1.png", sep = ""), res = 144, width = 1000, 
      height = 1100)
  }
  print(pobj)
  
  dev.off()
  
  # Figure 2
  par(mfrow = c(1, 3), mai = c(0.7, 0.6, 0.5, 0.1))
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 1.6
  cex.mainVal <- 2
  cex.legendVal <- c(1, 2)
  
  lwdVal <- c(1, 2)
  ltyVal <- c(1, 2)
  
  ylimVal <- c(0, 20)
  legendStr1 <- "Infer.NET"
  legendStr2 <- "MCMC"
  
  if (createPNG) {
    png(paste(dir.Figures, "spinalbone2a.png", sep = ""), res = 144, width = 700, 
      height = 700)
  }
  
  xg <- seq(mu.q.beta1TF - 3.5 * sqrt(sigsq.q.beta1TF), mu.q.beta1TF + 3.5 * sqrt(sigsq.q.beta1TF), 
    length = ng)
  beta1.postg <- dnorm(xg, mu.q.beta1TF, sqrt(sigsq.q.beta1TF))
  xlimVal <- range(xg)
  plot(0, 0, type = "n", bty = "l", xlim = xlimVal, ylim = ylimVal, cex.lab = cex.labVal, 
    xlab = "", ylab = "approx. posterior density", main = "Black versus Asian", 
    cex.main = cex.mainVal, xpd = FALSE)
  text(0.105, -5, expression(beta[1]), cex = cex.labVal, xpd = TRUE)
  dest <- bkde(beta1MCMC, bandwidth = dpik(beta1MCMC))
  lines(dest, lwd = lwdVal[2], lty = ltyVal[2], col = "red", xpd = FALSE)
  lines(xg, beta1.postg, lwd = lwdVal[1], lty = ltyVal[1], col = "blue", xpd = FALSE)
  abline(0, 0, xpd = FALSE)
  legend(0.35 * xlimVal[1] + 0.65 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2), lty = ltyVal, col = c("blue", "red"), cex = 1, 
    lwd = lwdVal, xpd = FALSE)
  
  
  dev.off()
  
  if (createPNG) 
    png(paste(dir.Figures, "spinalbone2b.png", sep = ""), res = 144, width = 700, 
      height = 700)
  
  xg <- seq(mu.q.beta2TF - 3.5 * sqrt(sigsq.q.beta2TF), mu.q.beta2TF + 3.5 * sqrt(sigsq.q.beta2TF), 
    length = ng)
  beta2.postg <- dnorm(xg, mu.q.beta2TF, sqrt(sigsq.q.beta2TF))
  xlimVal <- range(xg)
  plot(0, 0, type = "n", bty = "l", xlim = xlimVal, ylim = ylimVal, cex.lab = cex.labVal, 
    xlab = "", ylab = "approx. posterior density", main = "Hispanic versus Asian", 
    cex.main = cex.mainVal, xpd = FALSE)
  text(0.015, -5, expression(beta[2]), cex = cex.labVal, xpd = TRUE)
  dest <- bkde(beta2MCMC, bandwidth = dpik(beta2MCMC))
  lines(dest, lwd = lwdVal[2], lty = ltyVal[2], col = "red", xpd = FALSE)
  lines(xg, beta2.postg, lwd = lwdVal[1], lty = ltyVal[1], col = "blue", xpd = FALSE)
  abline(0, 0, xpd = FALSE)
  legend(0.35 * xlimVal[1] + 0.65 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2), lty = ltyVal, col = c("blue", "red"), cex = 1, 
    lwd = lwdVal, xpd = FALSE)
  
  
  dev.off()
  
  if (createPNG) 
    png(paste(dir.Figures, "spinalbone2c.png", sep = ""), res = 144, width = 710, 
      height = 700)
  
  xg <- seq(mu.q.beta3TF - 3.5 * sqrt(sigsq.q.beta3TF), mu.q.beta3TF + 3.5 * sqrt(sigsq.q.beta3TF), 
    length = ng)
  beta3.postg <- dnorm(xg, mu.q.beta3TF, sqrt(sigsq.q.beta3TF))
  xlimVal <- range(xg)
  plot(0, 0, type = "n", bty = "l", xlim = xlimVal, ylim = ylimVal, cex.lab = cex.labVal, 
    xlab = "", ylab = "approx. posterior density", main = "White versus Asian", 
    cex.main = cex.mainVal, xpd = FALSE)
  text(0.03, -5, expression(beta[3]), cex = cex.labVal, xpd = TRUE)
  dest <- bkde(beta3MCMC, bandwidth = dpik(beta3MCMC))
  lines(dest, lwd = lwdVal[2], lty = ltyVal[2], col = "red", xpd = FALSE)
  lines(xg, beta3.postg, lwd = lwdVal[1], lty = ltyVal[1], col = "blue", xpd = FALSE)
  abline(0, 0, xpd = FALSE)
  legend(0.35 * xlimVal[1] + 0.65 * xlimVal[2], 0.9999 * ylimVal[2] + 1e-04 * ylimVal[1], 
    c(legendStr1, legendStr2), lty = ltyVal, col = c("blue", "red"), cex = 1, 
    lwd = lwdVal, xpd = FALSE)
  
  dev.off()
}

################################################################################ 

if (REMOVE.INPUTS) {
  file.remove("ncX.txt")
  file.remove("numKnots.txt")
  file.remove("numAddComp.txt")
  file.remove("Cmat.txt")
  file.remove("y.txt")
  file.remove("numSubjects.txt")
  file.remove("tauBeta.txt")
  file.remove("Au.txt")
  file.remove("AU.txt")
  file.remove("Aeps.txt")
  file.remove("nIterVB.txt")
  file.remove("nBurnin.txt")
  file.remove("nIterGibbs.txt")
  file.remove("nThin.txt")
  file.remove("SBMDfemModel.txt")
}

if (REMOVE.OUTPUTS) {
  file.remove("mu.q.betaUu.txt")
  file.remove("Sigma.q.betaUu.txt")
  file.remove("parms.q.tauEps.txt")
  file.remove("parms.q.tauU.txt")
  file.remove("parms.q.tauu1.txt")
}

############### End of spinal bone ###############
