//////////////////////////  C# programme: spinalBone.cs ////////////////////////

// C# file containing support functions for the R/InferNET code for the paper:
//
// Luts J, Shen SSJ, Ormerod JT and Wand MP (2014)
//    Semiparametric Regression Analysis via Infer.NET

// Example for Section 6 of the paper.

// Last changed: 25 JULY 2014 by J.T Ormerod

// Load required C# and Infer.NET libraries: 
using System; using System.Collections.Generic;
using System.Text; using System.IO;
using System.Reflection; using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
   class addModel
   {
      public static void Main()
      {
         // Set full path-name of current working directory:
         string dir = ".\\";

         // Read in the number of columns of the X matrix:
         int ncX = new DataTable(dir+"ncX.txt").IntNumeric;

         // Read in the number of additive model components:
         int numAddComp = new DataTable(dir+"numAddComp.txt").IntNumeric;

         // Read in the number of knots for each additive model component:
         int numKnots = new DataTable(dir+"numKnots.txt").IntNumeric;

         // Read in the number of subjects:
         int numSubjects = new DataTable(dir + "numSubjects.txt").IntNumeric;
         
         // Read input files into data tables: 
         DataTable CTable = new DataTable(dir+"Cmat.txt");
         DataTable yTable = new DataTable(dir+"y.txt");

         // Extract total number of observations and number of columns
         // in the C matrix, and declare as integers:
         int n = CTable.numRow;
         int ncC = CTable.numCol;

         // Read in hyperparameter values:
         double tauBeta = new DataTable(dir+"tauBeta.txt").DoubleNumeric;
         double Au = new DataTable(dir+"Au.txt").DoubleNumeric;
         double AU = new DataTable(dir+"AU.txt").DoubleNumeric;
         double Aeps = new DataTable(dir+"Aeps.txt").DoubleNumeric;

         // Read in number of iterations for variational Bayes:
         int nIterVB = new DataTable(dir+"nIterVB.txt").IntNumeric;

         // Specify prior for the precision of spline coefficients for 
         // each additive model component:
         Variable<double>[] tauu = new Variable<double>[numAddComp];
         for (int i = 0; i < numAddComp; i++) {
             tauu[i] = Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Au, -2)));
         }

         // Specify prior for the precision of random intercepts for subjects
         Variable<double> tauU = (Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(AU, -2))));
            
         // Create a vector named "zeroVec" of length ncC:
         // (Note that this is automatically set to 
         // a vector of zeroes by convention.)
         Vector zeroVec = Vector.Zero(ncC);

         // Create positive definite matrix of dimension ncC by ncC:
         // (Note that this is automatically set to a matrix of zeroes
         // by convention.)
         PositiveDefiniteMatrix betauPrecDummy = new PositiveDefiniteMatrix(ncC,ncC);

         // Fill in diagonal entries of betauPrecDummy to make it diffuse:
         for (int i = 0; i < ncC; i++)
            betauPrecDummy[i,i] = 1e-10;

         // Creating a "VectorGaussian" (Infer.NET class) variable 
         // named "betauWork". Note that the <Vector> specifies that it
         // inherits methods of class "Vector".
         Variable<Vector> betauWork = Variable.VectorGaussianFromMeanAndPrecision(
                        zeroVec,betauPrecDummy).Named("betauWork");

         // Creating an array of variables of type "double" of length ncC: 
         Variable<double>[] betauDummy = new Variable<double>[ncC];

         // Populate betauDummy with the entries of "betauWork": 
         for (int j = 0; j < ncC; j++)
            betauDummy[j] = Variable.GetItem(betauWork,j);

         // Set up an auxiliary variable a with mean betau and
         // covariance of betau, and then make observed value
         // zero (this is a trick that has mathematical justification
         // in Wang, S.S.J and Wand, M.P. Using Infer.NET for 
         // Statistical Analyses. The American Statistician, 2011, 
         // 65, 115-126.):
         Variable<double>[] a = new Variable<double>[ncC];
         for (int j = 0; j < ncX; j++) {
            a[j] = Variable.GaussianFromMeanAndPrecision(betauDummy[j],tauBeta);
            a[j].ObservedValue = 0.0;
         }
         for (int j = 0; j < numSubjects; j++) {
             a[ncX+j] = Variable.GaussianFromMeanAndPrecision(betauDummy[ncX+j],tauU);
             a[ncX+j].ObservedValue = 0.0;
         }
         for (int i = 0; i < numAddComp; i++) {
            for (int j = 0; j < numKnots; j++) {
                a[ncX+numSubjects+i*numKnots+j] = Variable.GaussianFromMeanAndPrecision(betauDummy[ncX+numSubjects+i*numKnots+j], tauu[i]);
                a[ncX+numSubjects+i*numKnots+j].ObservedValue = 0.0;
            }
         }

         // Specify prior for the error precision:
         Variable<double> tauEps = (Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Aeps, -2)))).Named("tauEps");

         // Specify the likelihood: 
         Range index = new Range(n).Named("index");
         VariableArray<double> y = Variable.Array<double>(index).Named("y");
         VariableArray<Vector> cvec = Variable.Array<Vector>(index).Named("cvec");
         y[index] = Variable.GaussianFromMeanAndPrecision(Variable.InnerProduct(betauWork,cvec[index]),tauEps);

         // Create "cvecTable", and then assign the regression input data, 
         // stored in "cvecTable" and "yTable", to the "cvec" and "y" variables:
         Vector[] cvecTable = new Vector[n];
         for (int i = 0; i < n; i++) {
            cvecTable[i] = Vector.Zero(ncC);
            for (int j = 0; j < ncC; j++)
               cvecTable[i][j] = CTable.DataMatrix[i,j];
         }
         cvec.ObservedValue = cvecTable;
         y.ObservedValue = yTable.arrayForIN;

         // Specify inference engine and algorithm:
         InferenceEngine engine = new InferenceEngine();
         engine.Algorithm = new VariationalMessagePassing();
         engine.NumberOfIterations = nIterVB;

         // Output parameters of the approximate posterior: 
         SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetMean(),
                   dir+"mu.q.betaUu.txt");
         SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetVariance(),
                   dir+"Sigma.q.betaUu.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauEps),dir+"parms.q.tauEps.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauU),dir+"parms.q.tauU.txt");
         for (int i = 0; i < numAddComp; i++) {
            SaveData.SaveTable(engine.Infer<Gamma>(tauu[i]), 
                      dir+"parms.q.tauu"+(i + 1).ToString()+".txt");
         }
      }
   }
}

/////////////////////////////  End of spinalBone /////////////////////////////
