###################### R script: sydneyHousingData #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

# Example for Section 7 of the paper.

################################################################################ 

# Set seed
set.seed(1)

# Set directories
dir.InferNET.support <- "..\\..\\InferNETSupport\\"
dir.Figures <- "..\\..\\..\\Figures\\"

# Load support files
source(file.path(dir.InferNET.support, "InferNETSupport.R"), chdir = TRUE)

# Set flags
readData <- TRUE
useOnlySelectionOfData <- TRUE
doPlot <- TRUE
chooseKnots <- FALSE
displayKnots <- TRUE
chooseBdry <- FALSE
displayBdry <- TRUE
computeZgeog <- TRUE
fitModel <- TRUE
plotFits <- TRUE
createPNG <- TRUE
REMOVE.INPUTS <- TRUE
REMOVE.OUTPUTS <- TRUE

wait <- function() {
  cat("Hit Enter to continue\n")
  ans <- readline()
  invisible()
}

# Read data
if (readData) {
  chernih <- read.csv("chernih.csv")
}

# Use transformed versions of data for inputting into Infer.NET:
x1.mean <- mean(chernih$Income)
x1.std <- sd(chernih$Income)
x1 <- (chernih$Income - x1.mean)/x1.std
x2.mean <- mean(chernih$Coastline)
x2.std <- sd(chernih$Coastline)
x2 <- (chernih$Coastline - x2.mean)/x2.std
x3.mean <- mean(chernih$PM10)
x3.std <- sd(chernih$PM10)
x3 <- (chernih$PM10 - x3.mean)/x3.std
x4.mean <- mean(chernih$NO2)
x4.std <- sd(chernih$NO2)
x4 <- (chernih$NO2 - x4.mean)/x4.std

longit.mean <- mean(chernih$XCOORD)
longit.std <- sd(chernih$XCOORD)
longit <- (chernih$XCOORD - longit.mean)/longit.std
latit.mean <- mean(chernih$YCOORD)
latit.std <- sd(chernih$YCOORD)
latit <- (chernih$YCOORD - latit.mean)/latit.std

y.mean <- mean(chernih$LogSalePrice)
y.std <- sd(chernih$LogSalePrice)
y <- (chernih$LogSalePrice - y.mean)/y.std

# How many data do you want to use: all or 500?
if (useOnlySelectionOfData) {
  selectedN <- 37676  # 25000
  x1 <- x1[1:selectedN]
  x2 <- x2[1:selectedN]
  x3 <- x3[1:selectedN]
  x4 <- x4[1:selectedN]
  longit <- longit[1:selectedN]
  latit <- latit[1:selectedN]
  y <- y[1:selectedN]
}

# Extract Z for geographical variables
if (doPlot) {
  plot(longit, latit, pch = ".", col = "blue", bty = "l")
}

if (chooseKnots) {
  # Choose knots:
  chernihKnots <- save.default.knots.2D(longit, latit, num.knots = 100, "chernihKnots.txt")
}

if (displayKnots) {
  chernihKnots <- read.table("chernihKnots.txt")
  knotsLongit <- chernihKnots[, 1]
  knotsLatit <- chernihKnots[, 2]
  plot(longit, latit, pch = ".", col = "blue", bty = "l")
  points(knotsLongit, knotsLatit, col = "red", cex = 1.2, lwd = 2)
}

if (chooseBdry) {
  chernihBdry <- default.bdry(longit, latit)
  write.table(chernihBdry, "chernihBdry.txt", row.names = FALSE, col.names = FALSE)
}

if (displayBdry) {
  chernihBdry <- read.table("chernihBdry.txt")
  plot(longit, latit, pch = ".", col = "pink", bty = "l")
  lines(chernihBdry[, 1], chernihBdry[, 2], col = "green3", lwd = 5)
}

if (computeZgeog) {
  Zgeog <- Ztps(cbind(longit, latit), chernihKnots)
}

# Set up X, Z and C matrices:
n <- length(y)
X <- cbind(rep(1, n), x1, x2, x3, x4, longit, latit)

numIntKnots <- 15
intKnots.x1 <- quantile(unique(x1), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x1 <- c(1.05 * min(x1) - 0.05 * max(x1), 1.05 * max(x1) - 0.05 * min(x1))
Z1 <- ZOSull(x1, range.x1, intKnots.x1)

numIntKnots <- 15
intKnots.x2 <- quantile(unique(x2), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x2 <- c(1.05 * min(x2) - 0.05 * max(x2), 1.05 * max(x2) - 0.05 * min(x2))
Z2 <- ZOSull(x2, range.x2, intKnots.x2)

numIntKnots <- 15
intKnots.x3 <- quantile(unique(x3), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x3 <- c(1.05 * min(x3) - 0.05 * max(x3), 1.05 * max(x3) - 0.05 * min(x3))
Z3 <- ZOSull(x3, range.x3, intKnots.x3)

numIntKnots <- 15
intKnots.x4 <- quantile(unique(x4), seq(0, 1, length = (numIntKnots + 2))[-c(1, (numIntKnots + 
  2))])
range.x4 <- c(1.05 * min(x4) - 0.05 * max(x4), 1.05 * max(x4) - 0.05 * min(x4))
Z4 <- ZOSull(x4, range.x4, intKnots.x4)

Cmat <- cbind(X, Z1, Z2, Z3, Z4, Zgeog)

# Write input information to files:
write(ncol(X), "ncX.txt")
write(ncol(Z1), "numKnots.txt")
write(ncol(Zgeog), "numGeoKnots.txt")
write(4, "numAddComp.txt")
write.table(Cmat, "Cmat.txt", sep = ", ", row.names = FALSE, col.names = FALSE)
write.table(y, "y.txt", sep = ", ", row.names = FALSE, col.names = FALSE)

# Set hyperparameters and write to files:
sigsqBeta <- 1e+08
Au <- 10^5
Aeps <- 10^5
Ageog <- 10^5
write(sigsqBeta, "sigsqBeta.txt")
write(Au, "Au.txt")
write(Ageog, "Ageog.txt")
write(Aeps, "Aeps.txt")

# Set Gibbs and variational Bayes iteration sizes.  If `nIterGibbs' is zero then
# variational Bayes used instead of Gibbs sampling:
nIterGibbs <- 0
nBurnin <- 1000
nThin <- 5
nIterVB <- 100
write(nIterVB, "nIterVB.txt")
write(nIterGibbs, "nIterGibbs.txt")
write(nBurnin, "nBurnin.txt")
write(nThin, "nThin.txt")

TRIALS <- 1

if (fitModel) {
  # Run Infer.NET:
  
  vtimes <- list()
  for (trial in 1:TRIALS) {
     timeTaken <- system.time(RunInferNET(dir.InferNET.support, "sydneyHousingData.cs"))
     print(timeTaken)
     vtimes[[trial]] <- timeTaken
     
     print(vtimes)
  }
  
  # Read in posterior density parameters:
  mu.q.betaUgeog <- scan("mu.q.betaUgeog.txt")
  Sigma.q.betaUgeog <- as.matrix(read.table("Sigma.q.betaUgeog.txt"))
}

if (plotFits) {
  # Figure 1
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 1.6
  cex.mainVal <- 2
  
  longitLatitg <- expand.grid(seq(min(chernihBdry[, 1] - 0.05 * (max(chernihBdry[, 
    1]) - min(chernihBdry[, 1]))), max(chernihBdry[, 1] + 0.05 * (max(chernihBdry[, 
    1]) - min(chernihBdry[, 1]))), length = ng), seq(min(chernihBdry[, 2] - 0.05 * 
    (max(chernihBdry[, 2]) - min(chernihBdry[, 2]))), max(chernihBdry[, 2] + 
    0.05 * (max(chernihBdry[, 2]) - min(chernihBdry[, 2]))), length = ng))
  longitg <- longitLatitg[, 1]
  latitg <- longitLatitg[, 2]
  gridSizeg <- nrow(longitLatitg)
  Z1g <- ZOSull(rep(0, gridSizeg), range.x1, intKnots.x1)
  Z2g <- ZOSull(rep(0, gridSizeg), range.x2, intKnots.x2)
  Z3g <- ZOSull(rep(0, gridSizeg), range.x3, intKnots.x3)
  Z4g <- ZOSull(rep(0, gridSizeg), range.x4, intKnots.x4)
  Zgeogg <- Ztps(longitLatitg, chernihKnots)
  Cmatg <- cbind(rep(1, gridSizeg), rep(0, gridSizeg), rep(0, gridSizeg), rep(0, 
    gridSizeg), rep(0, gridSizeg), longitg, latitg, Z1g, Z2g, Z3g, Z4g, Zgeogg)
  
  longitgOrig <- seq(min(chernihBdry[, 1] - 0.05 * (max(chernihBdry[, 1]) - min(chernihBdry[, 
    1]))), max(chernihBdry[, 1] + 0.05 * (max(chernihBdry[, 1]) - min(chernihBdry[, 
    1]))), length = ng) * longit.std + longit.mean
  latitgOrig <- seq(min(chernihBdry[, 2] - 0.05 * (max(chernihBdry[, 2]) - min(chernihBdry[, 
    2]))), max(chernihBdry[, 2] + 0.05 * (max(chernihBdry[, 2]) - min(chernihBdry[, 
    2]))), length = ng) * latit.std + latit.mean
  
  temp <- NULL
  for (i in 1:nrow(Cmatg)) {
    muFig <- sum(Cmatg[i, ] * t(mu.q.betaUgeog)) * y.std + y.mean
    varFig <- as.numeric(diag((Cmatg[i, ] * y.std) %*% Sigma.q.betaUgeog %*% 
      as.matrix(Cmatg[i, ] * y.std)))
    temp <- rbind(temp, exp(muFig + varFig/2))
  }
  fhatgOrig <- temp
  
  fhatgOrigBoundary <- fhatgOrig
  for (i in 1:nrow(longitLatitg)) {
    if (point.in.polygon(longitLatitg[i, 1], longitLatitg[i, 2], chernihBdry[, 
      1], chernihBdry[, 2])) {
    } else {
      fhatgOrigBoundary[i] <- NA
    }
  }
  
  if (createPNG) {
    png(paste(dir.Figures, "sydneyHousingData1.png", sep = ""), res = 144, width = 1400, 
      height = 1200)
  }
  
  # only estimate
  par(bty = "l")
  filled.contour(longitgOrig, latitgOrig, matrix(fhatgOrigBoundary, nrow = ng, 
    ncol = ng), las = 0, color = terrain.colors, asp = 1, plot.title = title(main = "", 
    xlab = "degrees longitude", ylab = "degrees latitude", cex.lab = cex.labVal, 
    cex.main = cex.mainVal), plot.axes = {
    lines(chernihBdry[, 1] * longit.std + longit.mean, chernihBdry[, 2] * latit.std + 
      latit.mean, col = "black", lwd = 5)
    axis(1, rbind(150.72, 150.93, 151.13, 151.34))
    axis(2, rbind(-33.69, -33.87, -34.04))
  }, key.title = title(main = "    Price\n    (Australian\n     dollars)"))
  
  dev.off()
  
  # Figure 2
  ng <- 401
  lwdVal <- 2
  cex.labVal <- 1.5
  
  # Fit line for x1 (income):
  
  # par(mfrow = c(1, 1), mai = c(0, 0, 0, 0))
  x1g <- seq(min(x1), max(x1), length = ng)
  Z1g <- ZOSull(x1g, range.x1, intKnots.x1)
  Z2g <- ZOSull(rep(0, ng), range.x2, intKnots.x2)
  Z3g <- ZOSull(rep(0, ng), range.x3, intKnots.x3)
  Z4g <- ZOSull(rep(0, ng), range.x4, intKnots.x4)
  Zgeogg <- Ztps(cbind(rep(0.5, ng), rep(-0.5, ng)), chernihKnots)
  Cmatg1 <- cbind(rep(1, ng), x1g, rep(0, ng), rep(0, ng), rep(0, ng), rep(0.5, 
    ng), rep(-0.5, ng), Z1g, Z2g, Z3g, Z4g, Zgeogg)
  
  x1gOrig <- x1g * x1.std + x1.mean
  
  muFig <- (Cmatg1 %*% mu.q.betaUgeog) * y.std + y.mean
  varFig <- as.numeric(diag((Cmatg1 * y.std) %*% Sigma.q.betaUgeog %*% t(as.matrix(Cmatg1 * 
    y.std))))
  
  fhatg1Orig <- exp(muFig + varFig/2)
  fhatg1OrigLower <- qlnorm(0.025, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  fhatg1OrigUpper <- qlnorm(0.975, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  
  xlimVal <- c(min(x1gOrig), max(x1gOrig))
  ylimVal <- c(min(fhatg1OrigLower), max(fhatg1OrigUpper))
  
  randomsample <- sample(n)
  
  if (createPNG) 
    png(paste(dir.Figures, "sydneyHousingData2.png", sep = ""), res = 144, width = 900, 
      height = 900)
  
  plot(x1gOrig, fhatg1Orig, ylim = ylimVal, xlim = xlimVal, xlab = "income in Australian dollars", 
    ylab = "real estate price in Australian dollars", bty = "l", cex.lab = cex.labVal, 
    col = "blue", type = "l", lwd = lwdVal)
  lines(x1gOrig, fhatg1OrigLower, lwd = lwdVal, col = "blue", lty = 2)
  lines(x1gOrig, fhatg1OrigUpper, lwd = lwdVal, col = "blue", lty = 2)
  rug(x1[randomsample[1:(0.2 * n)]] * x1.std + x1.mean, ticksize = 0.01, col = "blue")
  
  dev.off()
  dev.off()
  
  # Fit line for x2 (coastline distance):
  
  par(mfrow = c(1, 1), mai = c(0, 0, 0, 0))
  x2g <- seq(min(x2), max(x2), length = ng)
  Z1g <- ZOSull(rep(0, ng), range.x1, intKnots.x1)
  Z2g <- ZOSull(x2g, range.x2, intKnots.x2)
  Z3g <- ZOSull(rep(0, ng), range.x3, intKnots.x3)
  Z4g <- ZOSull(rep(0, ng), range.x4, intKnots.x4)
  Zgeogg <- Ztps(cbind(rep(0.5, ng), rep(-0.5, ng)), chernihKnots)
  Cmatg2 <- cbind(rep(1, ng), rep(0, ng), x2g, rep(0, ng), rep(0, ng), rep(0.5, 
    ng), rep(-0.5, ng), Z1g, Z2g, Z3g, Z4g, Zgeogg)
  
  x2gOrig <- x2g * x2.std + x2.mean
  
  muFig <- (Cmatg2 %*% mu.q.betaUgeog) * y.std + y.mean
  varFig <- as.numeric(diag((Cmatg2 * y.std) %*% Sigma.q.betaUgeog %*% t(as.matrix(Cmatg2 * 
    y.std))))
  
  fhatg2Orig <- exp(muFig + varFig/2)
  fhatg2OrigLower <- qlnorm(0.025, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  fhatg2OrigUpper <- qlnorm(0.975, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  
  xlimVal <- c(min(x2gOrig), max(x2gOrig))
  ylimVal <- c(min(fhatg2OrigLower), max(fhatg2OrigUpper))
  
  if (createPNG) 
    png(paste(dir.Figures, "sydneyHousingData3.png", sep = ""), res = 144, width = 900, 
      height = 900)
  
  plot(x2gOrig, fhatg2Orig, ylim = ylimVal, xlim = xlimVal, xlab = "distance from coastline in km", 
    ylab = "real estate price in Australian dollars", bty = "l", cex.lab = cex.labVal, 
    col = "blue", type = "l", lwd = lwdVal)
  lines(x2gOrig, fhatg2OrigLower, lwd = lwdVal, col = "blue", lty = 2)
  lines(x2gOrig, fhatg2OrigUpper, lwd = lwdVal, col = "blue", lty = 2)
  rug(x2[randomsample[1:(0.2 * n)]] * x2.std + x2.mean, ticksize = 0.01, col = "blue")
  
  dev.off()
  dev.off()
  
  # Fit line for x3 (particulate matter 10 level):
  
  par(mfrow = c(1, 1), mai = c(0, 0, 0, 0))
  x3g <- seq(min(x3), max(x3), length = ng)
  Z1g <- ZOSull(rep(0, ng), range.x1, intKnots.x1)
  Z2g <- ZOSull(rep(0, ng), range.x2, intKnots.x2)
  Z3g <- ZOSull(x3g, range.x3, intKnots.x3)
  Z4g <- ZOSull(rep(0, ng), range.x4, intKnots.x4)
  Zgeogg <- Ztps(cbind(rep(0.5, ng), rep(-0.5, ng)), chernihKnots)
  Cmatg3 <- cbind(rep(1, ng), rep(0, ng), rep(0, ng), x3g, rep(0, ng), rep(0.5, 
    ng), rep(-0.5, ng), Z1g, Z2g, Z3g, Z4g, Zgeogg)
  
  x3gOrig <- x3g * x3.std + x3.mean
  
  muFig <- (Cmatg3 %*% mu.q.betaUgeog) * y.std + y.mean
  varFig <- as.numeric(diag((Cmatg3 * y.std) %*% Sigma.q.betaUgeog %*% t(as.matrix(Cmatg3 * 
    y.std))))
  
  fhatg3Orig <- exp(muFig + varFig/2)
  fhatg3OrigLower <- qlnorm(0.025, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  fhatg3OrigUpper <- qlnorm(0.975, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  
  xlimVal <- c(min(x3gOrig), max(x3gOrig))
  ylimVal <- c(min(fhatg3OrigLower), max(fhatg3OrigUpper))
  
  if (createPNG) 
    png(paste(dir.Figures, "sydneyHousingData4.png", sep = ""), res = 144, width = 900, 
      height = 900)
  
  plot(x3gOrig, fhatg3Orig, ylim = ylimVal, xlim = xlimVal, xlab = "particulate matter 10 level", 
    ylab = "real estate price in Australian dollars", bty = "l", cex.lab = cex.labVal, 
    col = "blue", type = "l", lwd = lwdVal)
  lines(x3gOrig, fhatg3OrigLower, lwd = lwdVal, col = "blue", lty = 2)
  lines(x3gOrig, fhatg3OrigUpper, lwd = lwdVal, col = "blue", lty = 2)
  rug(x3[randomsample[1:(0.2 * n)]] * x3.std + x3.mean, ticksize = 0.01, col = "blue")
  
  dev.off()
  dev.off()
  
  # Fit line for x4 (nitrogen dioxide level):
  
  par(mfrow = c(1, 1), mai = c(0, 0, 0, 0))
  x4g <- seq(min(x4), max(x4), length = ng)
  Z1g <- ZOSull(rep(0, ng), range.x1, intKnots.x1)
  Z2g <- ZOSull(rep(0, ng), range.x2, intKnots.x2)
  Z3g <- ZOSull(rep(0, ng), range.x3, intKnots.x3)
  Z4g <- ZOSull(x4g, range.x4, intKnots.x4)
  Zgeogg <- Ztps(cbind(rep(0.5, ng), rep(-0.5, ng)), chernihKnots)
  Cmatg4 <- cbind(rep(1, ng), rep(0, ng), rep(0, ng), rep(0, ng), x4g, rep(0.5, 
    ng), rep(-0.5, ng), Z1g, Z2g, Z3g, Z4g, Zgeogg)
  
  x4gOrig <- x4g * x4.std + x4.mean
  
  muFig <- (Cmatg4 %*% mu.q.betaUgeog) * y.std + y.mean
  varFig <- as.numeric(diag((Cmatg4 * y.std) %*% Sigma.q.betaUgeog %*% t(as.matrix(Cmatg4 * 
    y.std))))
  
  fhatg4Orig <- exp(muFig + varFig/2)
  fhatg4OrigLower <- qlnorm(0.025, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  fhatg4OrigUpper <- qlnorm(0.975, meanlog = muFig, sdlog = sqrt(varFig), lower.tail = TRUE, 
    log.p = FALSE)
  
  xlimVal <- c(min(x4gOrig), max(x4gOrig))
  ylimVal <- c(min(fhatg4OrigLower), max(fhatg4OrigUpper))
  
  if (createPNG) {
    png(paste(dir.Figures, "sydneyHousingData5.png", sep = ""), res = 144, width = 900, 
      height = 900)
  }
  
  plot(x4gOrig, fhatg4Orig, ylim = ylimVal, xlim = xlimVal, xlab = "nitrogen dioxide level", 
    ylab = "real estate price in Australian dollars", bty = "l", cex.lab = cex.labVal, 
    col = "blue", type = "l", lwd = lwdVal)
  lines(x4gOrig, fhatg4OrigLower, lwd = lwdVal, col = "blue", lty = 2)
  lines(x4gOrig, fhatg4OrigUpper, lwd = lwdVal, col = "blue", lty = 2)
  rug(x4[randomsample[1:(0.2 * n)]] * x4.std + x4.mean, ticksize = 0.01, col = "blue")
  
  dev.off()
  dev.off()
}

################################################################################ 

if (REMOVE.INPUTS) {
  file.remove("ncX.txt")
  file.remove("numKnots.txt")
  file.remove("numAddComp.txt")
  file.remove("numGeoKnots.txt")
  file.remove("Cmat.txt")
  file.remove("y.txt")
  file.remove("Au.txt")
  file.remove("sigsqBeta.txt")
  file.remove("Au.txt")
  file.remove("Ageog.txt")
  file.remove("Aeps.txt")
  file.remove("nIterVB.txt")
  file.remove("nBurnin.txt")
  file.remove("nIterGibbs.txt")
  file.remove("nThin.txt")
}

if (REMOVE.OUTPUTS) {
  file.remove("mu.q.betaUgeog.txt")
  file.remove("Sigma.q.betaUgeog.txt")
  file.remove("parms.q.sigsqEps.txt")
  file.remove("parms.q.sigsqU1.txt")
  file.remove("parms.q.sigsqU2.txt")
  file.remove("parms.q.sigsqU3.txt")
  file.remove("parms.q.sigsqU4.txt")
  file.remove("parms.q.sigsqUgeog.txt")
  file.remove("texput.log")
}

############ End of SydneyHousingData ###############
 
