//////////////////////////  C# programme: sydneyHousingData /////////////////////////

// C# file containing support functions for the R/InferNET code for the paper:
//
// Luts J, Shen SSJ, Ormerod JT and Wand MP (2014)
//    Semiparametric Regression Analysis via Infer.NET

// Example for Section 7 of the paper.

// Last changed: 25 JULY 2014 by J.T Ormerod

// Load required C# and Infer.NET libraries: 
using System; using System.Collections.Generic;
using System.Text; using System.IO;
using System.Reflection; using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
   class addModel
   {
      public static void Main()
      {
         // Set full path-name of current working directory:
         string dir = ".\\";
          
         // Read in the number of columns of the X matrix:
         int ncX = new DataTable(dir+"ncX.txt").IntNumeric;

         // Read in the number of additive model components:
         int numAddComp = new DataTable(dir+"numAddComp.txt").IntNumeric;

         // Read in the number of knots for each additive model component:
         int numKnots = new DataTable(dir+"numKnots.txt").IntNumeric;

         // Read in the number of geo knots:
         int numGeoKnots = new DataTable(dir + "numGeoKnots.txt").IntNumeric;
         
         // Read input files into data tables: 
         DataTable CTable = new DataTable(dir+"Cmat.txt");
         DataTable yTable = new DataTable(dir+"y.txt");

         // Extract total number of observations and and number of columns
         // in the C matrix, and declare as integers:
         int n = CTable.numRow;
         int ncC = CTable.numCol;

         // Read in hyperparameter values:
         double sigsqBeta = new DataTable(dir+"sigsqBeta.txt").DoubleNumeric;
         double Au = new DataTable(dir+"Au.txt").DoubleNumeric;
         double Ageog = new DataTable(dir + "Ageog.txt").DoubleNumeric;
         double Aeps = new DataTable(dir+"Aeps.txt").DoubleNumeric;

         // Read in number of Gibbs samples. If this number
         // is zero then varitional approximation is used.
         int nIterGibbs = new DataTable(dir+"nIterGibbs.txt").IntNumeric;
         int nBurnin = new DataTable(dir+"nBurnin.txt").IntNumeric;
         int nThin = new DataTable(dir+"nThin.txt").IntNumeric;

         // Read in number of Iterations for variational Bayes:
         int nIterVB = new DataTable(dir+"nIterVB.txt").IntNumeric;

         // Specify prior for the precision of spline coefficients for 
         // each additive model component:
         Variable<double>[] tauU = new Variable<double>[numAddComp];
         for (int i = 0; i < numAddComp; i++) {
             tauU[i] = Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Au, -2)));
         }

         Variable<double> tauUgeog = (Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Ageog, -2))));
            
         // Create a vector named "zeroVec" of length ncC:
         // (Note that this is automatically set to 
         // a vector of zeroes by convention.)
         Vector zeroVec = Vector.Zero(ncC);

         // Create positive definite matrix of dimension ncC by ncC:
         // (Note that this is automatically set to a matrix of zeroes
         // by convention.)
         PositiveDefiniteMatrix betauPrecDummy = new PositiveDefiniteMatrix(ncC,ncC);

         // Fill in diagonal entries of betauPrecDummy to make it diffuse:
         for (int i = 0; i < ncC; i++)
            betauPrecDummy[i,i] = 1e-10;

         // Creating a "VectorGaussian" (Infer.NET class) variable 
         // named "betauWork". Note that the <Vector> specifies that it
         // inherits methods of class "Vector".
         Variable<Vector> 
            betauWork = Variable.VectorGaussianFromMeanAndPrecision(
                        zeroVec,betauPrecDummy).Named("betauWork");

         // Creating an array of variables of type "double" of length ncC: 
         Variable<double>[] betauDummy = new Variable<double>[ncC];

         // Populate betauDummy with the entries of "betauWork": 
         for (int j = 0; j < ncC; j++)
            betauDummy[j] = Variable.GetItem(betauWork,j);

         // Set up an auxiliary variable with mean betau and
         // covariance of betau, and then make observed value
         // zero (this is a trick that has mathematical justification
         // in the Wang & Wand paper):
         Variable<double>[] auxVar = new Variable<double>[ncC];
         for (int j = 0; j < ncX; j++) {
            auxVar[j] = Variable.GaussianFromMeanAndVariance(betauDummy[j],sigsqBeta);
            auxVar[j].ObservedValue = 0.0;
         }
         for (int i = 0; i < numAddComp; i++) {
             for (int j = 0; j < numKnots; j++) {
                 auxVar[ncX + i * numKnots + j] = Variable.GaussianFromMeanAndPrecision(betauDummy[ncX + i * numKnots + j], tauU[i]);
                 auxVar[ncX + i * numKnots + j].ObservedValue = 0.0;
             }
         }
         for (int j = 0; j < numGeoKnots; j++) {
             auxVar[ncX + numAddComp * numKnots + j] = Variable.GaussianFromMeanAndPrecision(betauDummy[ncX + numAddComp * numKnots + j], tauUgeog);
             auxVar[ncX + numAddComp * numKnots + j].ObservedValue = 0.0;
         }     


         // Specify prior for the error precision:
         Variable<double> tauEps = (Variable.GammaFromShapeAndRate(0.5, Variable.GammaFromShapeAndRate(0.5, Math.Pow(Aeps, -2)))).Named("tauEps");

         // Specify the likelihood: 
         Range index = new Range(n).Named("index");
         VariableArray<double> y = Variable.Array<double>(index).Named("y");
         VariableArray<Vector> cvec = Variable.Array<Vector>(index).Named("cvec");
         y[index] = Variable.GaussianFromMeanAndPrecision(Variable.InnerProduct(betauWork,cvec[index]),tauEps);

         // Create "cvecTable", and then assign the regression input data, 
         // stored in "cvecTable" and "yTable", to the "cvec" and "y" variables:
         Vector[] cvecTable = new Vector[n];
         for (int i = 0; i < n; i++) {
            cvecTable[i] = Vector.Zero(ncC);
            for (int j = 0; j < ncC; j++)
               cvecTable[i][j] = CTable.DataMatrix[i,j];
         }
         cvec.ObservedValue = cvecTable;
         y.ObservedValue = yTable.arrayForIN;

         // Specify inference engine and algorithm:
         InferenceEngine engine = new InferenceEngine();

         if (nIterGibbs>0) {
            GibbsSampling gs = new GibbsSampling();
            gs.BurnIn = nBurnin ; gs.Thin = nThin;
            engine.Algorithm = gs;
            engine.NumberOfIterations = nBurnin + nIterGibbs;
         } else {
            engine.Algorithm = new VariationalMessagePassing();
            engine.NumberOfIterations = nIterVB;
         }

         // Output parameters of the approximate posterior: 
         SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetMean(),
                   dir+"mu.q.betaUgeog.txt");
         SaveData.SaveTable(engine.Infer<VectorGaussian>(betauWork).GetVariance(),
                   dir + "Sigma.q.betaUgeog.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauEps),dir+"parms.q.sigsqEps.txt");
         SaveData.SaveTable(engine.Infer<Gamma>(tauUgeog), dir + "parms.q.sigsqUgeog.txt");
         for (int i = 0; i < numAddComp; i++) {
            SaveData.SaveTable(engine.Infer<Gamma>(tauU[i]), 
                      dir+"parms.q.sigsqU"+(i + 1).ToString()+".txt");
         }
      }
   }
}

/////////////////////////////  End of sydneyHousingData /////////////////////////////
