########### R function: accVarApp ##########

# For assessment of the accuracy of a variational
# approximate posterior density function.

# If type = "Normal"            then parVec = (mean, variance)
# If type = "InvGamma"          then parVec = (shape, rate)
# If type = "InvGammaSqrt"      then parVec = (shape, rate)
# If type = "Beta"              then parVec = (shape1, shape2)
# If type = "Bernoulli"         then parVec = (mean)
# If type = "Discrete"          then parVec = list(atoms, probabs)
# If type = "NormalMix"         then parVec = list(wVec, meanVec, sdVec)
# If type = "scrF"              then parVec = (qq, rr, ss, tt)
# If type = "scrG"              then parVec = (pp, qq, rr, ss)
# If type = "scrJLN"            then parVec = (qq, rr, ss)
# If type = "scrJplus"          then parVec = (pp, qq, rr)
# If type = "scrJplusRecip"     then parVec = (pp, qq, rr)
# If type = "scrJplusRecipMix"  then parVec = list(wVec, ppVec, qqVec, rrVec)
# If type = "scrHrecip"         then parVec = (pp, qq, rr)

accVarApp <- function(parVec, parName, MCsample, type, plotDensities = TRUE, parTrue = NULL, 
  colVA = "blue", colMC = "red", colTruth = "seagreen", colAcc = "DarkMagenta", 
  gridSize = 1001, lwdVal = 3, ltyVA = 1, ltyMC = 1, cex.labVal = 2, cex.mainVal = 2, 
  cex.dist = 3, xlabVal = "", plotAccuracy = FALSE) {
  require("KernSmooth")
  
  MClower <- quantile(MCsample, 5e-04)
  MCupper <- quantile(MCsample, 0.9995)
  
  if (type == "Normal") {
    VAlower <- parVec[1] - qnorm(0.9995) * sqrt(parVec[2])
    VAupper <- parVec[1] + qnorm(0.9995) * sqrt(parVec[2])
  }
  
  if (type == "InvGamma") {
    VAlower <- 1/qgamma(0.9995, parVec[1], rate = parVec[2])
    VAupper <- 1/qgamma(5e-04, parVec[1], rate = parVec[2])
  }
  
  if (type == "InvGammaSqrt") {
    VAlower <- 1/sqrt(qgamma(0.9995, parVec[1], rate = parVec[2]))
    VAupper <- 1/sqrt(qgamma(5e-04, parVec[1], rate = parVec[2]))
  }
  
  if (type == "Beta") {
    VAlower <- 0
    VAupper <- 1
  }
  
  if (type == "Bernoulli") {
    VAlower <- -0.1
    VAupper <- 1.1
  }
  
  if (type == "Discrete") {
    VAlower <- 1.05 * min(parVec[[1]]) - 0.05 * max(parVec[[1]])
    VAupper <- 1.05 * max(parVec[[1]]) - 0.05 * min(parVec[[1]])
  }
  
  if (type == "NormalMix") {
    VAlower <- qnorMix(5e-04, parVec[[1]], parVec[[2]], parVec[[3]])
    VAupper <- qnorMix(0.9995, parVec[[1]], parVec[[2]], parVec[[3]])
  }
  
  if (type == "scrF") {
    xScrF <- rScrF(1e+05, parVec[1], parVec[2], parVec[3], parVec[4])
    VAlower <- as.numeric(quantile(xScrF, 5e-04))
    VAupper <- as.numeric(quantile(xScrF, 0.9995))
  }
  
  if (type == "scrG") {
    VAlower <- qScrG(5e-04, "scrG", parVec)
    VAupper <- qScrG(0.9995, "scrG", parVec)
  }
  
  if (type == "scrJLN") {
    xScrJLN <- exp(rScrJ(1e+05, 0.5 * (parVec[1] + 1), 4 * parVec[2], parVec[3])/2)
    VAlower <- as.numeric(quantile(xScrJLN, 5e-04))
    VAupper <- as.numeric(quantile(xScrJLN, 0.9995))
  }
  
  if (type == "scrJplus") {
    xScrJplus <- rScrJplus(1e+05, parVec[1], parVec[2], parVec[3])
    VAlower <- as.numeric(quantile(xScrJplus, 5e-04))
    VAupper <- as.numeric(quantile(xScrJplus, 0.9995))
  }
  
  if (type == "scrJplusRecip") {
    xScrJplus <- rScrJplus(1e+05, parVec[1], parVec[2], parVec[3])
    VAlower <- 1/as.numeric(quantile(xScrJplus, 0.9995))
    VAupper <- 1/as.numeric(quantile(xScrJplus, 5e-04))
  }
  
  if (type == "scrHrecip") {
    xScrH <- rScrH(1e+05, parVec[1], parVec[2], parVec[3])
    VAlower <- 1/as.numeric(quantile(xScrH, 0.9995))
    VAupper <- 1/as.numeric(quantile(xScrH, 5e-04))
  }
  
  if (type == "scrJplusRecipMix") {
    nMix <- length(parVec[[1]])
    xSamps <- matrix(NA, 10000, nMix)
    
    for (k in 1:nMix) xSamps[, k] <- rScrJplus(10000, parVec[[2]][k], parVec[[3]][k], 
      parVec[[4]][k])
    
    inds <- sample(1:nMix, 10000, replace = TRUE, prob = parVec[[1]])
    
    xScrJplusMix <- rep(NA, 10000)
    for (i in 1:10000) xScrJplusMix[i] <- xSamps[i, inds[i]]
    
    VAlower <- 1/as.numeric(quantile(xScrJplusMix, 0.9995))
    VAupper <- 1/as.numeric(quantile(xScrJplusMix, 5e-04))
    
  }
  
  xgLow <- min(MClower, VAlower)
  xgUpp <- max(MCupper, VAupper)
  xg <- seq(xgLow, xgUpp, length = gridSize)
  xlimVal <- c(xgLow, xgUpp)
  
  if (type == "Normal") 
    VApostg <- dnorm(xg, parVec[1], sqrt(parVec[2]))
  if (type == "InvGamma") 
    VApostg <- exp(parVec[1] * log(parVec[2]) - lgamma(parVec[1]) - (parVec[1] + 
      1) * log(xg) - parVec[2]/xg)
  if (type == "InvGammaSqrt") 
    VApostg <- 2 * exp(parVec[1] * log(parVec[2]) - lgamma(parVec[1]) - (2 * 
      parVec[1] + 1) * log(xg) - parVec[2]/(xg^2))
  if (type == "Beta") 
    VApostg <- dbeta(xg, parVec[1], parVec[2])
  if (type == "NormalMix") 
    VApostg <- dnorMix(xg, parVec[[1]], parVec[[2]], parVec[[3]])
  if (type == "scrF") 
    VApostg <- exp(parVec[1] * log(1 + xg^2) - parVec[2] * xg^2 + parVec[3] * 
      xg * sqrt(1 + xg^2) + parVec[4] * xg - logScrF(0, parVec[1], parVec[2], 
      parVec[3], parVec[4])$modulus)
  if (type == "scrG") 
    VApostg <- dScrG(xg, "scrG", parVec)
  if (type == "scrJLN") 
    VApostg <- 2 * exp(parVec[1] * log(xg) - parVec[2] * (log(xg)^2) - parVec[3]/(xg^2) - 
      logScrJ(0, 0.5 * (parVec[1] + 1), 0.25 * parVec[2], 0, parVec[3])$modulus)
  if (type == "scrJplus") 
    VApostg <- exp(parVec[1] * log(xg) + parVec[2] * xg - parVec[3] * xg^2 - 
      logScrJplus(parVec[1], parVec[2], parVec[3]))
  if (type == "scrJplusRecip") 
    VApostg <- exp(-(parVec[1] + 2) * log(xg) + parVec[2]/xg - parVec[3]/(xg^2) - 
      logScrJplus(parVec[1], parVec[2], parVec[3]))
  if (type == "scrHrecip") 
    VApostg <- exp(-(parVec[1] + 2) * log(xg) - parVec[2]/(xg^2) - log(parVec[3] + 
      xg^2) - logScrH(parVec[1], parVec[2], parVec[3]))
  
  if (type == "scrJplusRecipMix") {
    VApostg <- rep(0, length(xg))
    for (k in 1:length(parVec[[1]])) VApostg <- VApostg + parVec[[1]][k] * exp(-(parVec[[2]][k] + 
      2) * log(xg) + parVec[[3]][k]/xg - parVec[[4]][k]/(xg^2) - logScrJplus(parVec[[2]][k], 
      parVec[[3]][k], parVec[[4]][k]))
  }
  
  
  if (any(type == c("Normal", "InvGamma", "InvGammaSqrt", "Beta", "NormalMix", 
    "scrF", "scrG", "scrJLN", "scrJplus", "scrJplusRecip", "scrHrecip", "scrJplusRecipMix"))) {
    
    # Compute accuracy value:
    
    # MCpostg <- density(MCsample,n=gridSize,from=min(xg),to=max(xg))$y
    MCpostg <- bkde(MCsample, bandwidth = dpik(MCsample), range.x = range(xg), 
      gridsize = gridSize)$y
    accurVal <- round(100 - 50 * trapint(xg, abs(MCpostg - VApostg)))
    
    if (plotDensities) {
      ylimVal <- range(c(VApostg, MCpostg))
      plot(xg, VApostg, type = "n", bty = "l", main = parName, cex.main = cex.mainVal, 
        xlab = "", ylab = "approx. posterior density", cex.lab = cex.labVal, 
        ylim = ylimVal)
      lines(xg, VApostg, lwd = lwdVal, col = colVA, lty = ltyVA)
      lines(xg, MCpostg, lwd = lwdVal, col = colMC, lty = ltyMC)
      
      
      if (!is.null(parTrue)) 
        lines(rep(parTrue, 2), c(0, ylimVal[2]), col = colTruth)
      abline(h = 0)
      
      # Plot accuracy value:
      
      if (plotAccuracy) {
        text(0.15 * min(xlimVal) + 0.85 * max(xlimVal), 0.1 * min(ylimVal) + 
          0.9 * max(ylimVal), as.character(paste(accurVal, "%", sep = "")), 
          cex = cex.dist, col = colAcc)
        text(0.15 * min(xlimVal) + 0.85 * max(xlimVal), 0.3 * min(ylimVal) + 
          0.7 * max(ylimVal), "accuracy", cex = 0.5 * cex.dist, col = colAcc)
      } else {
        legend("topright", c("Infer.NET", "MCMC"), lty = c(1, 1), lwd = c(3, 
          3), col = c(colVA, colMC), cex = 1)
      }
    }
  }
  
  if (type == "Bernoulli") {
    # Plot accuracy value:
    
    muMC <- mean(MCsample)
    muVA <- parVec[1]
    accurVal <- round(100 * (1 - abs(muMC - muVA)))
    
    if (plotDensities) {
      
      ylimVal <- c(0, 1)
      
      xlims0 <- c(-0.05, 0.05)
      xlims1 <- c(0.95, 1.05)
      
      plot(0, 0, type = "n", xlim = c(-0.1, 1.1), ylim = c(0, 1), bty = "l", 
        main = parName, cex.main = cex.mainVal)
      
      x0VApoly <- list(x = c(xlims0, rev(xlims0)), y = c(rep(0, 2), rep((1 - 
        muVA), 2)))
      x0MCpoly <- list(x = c(xlims0, rev(xlims0)), y = c(rep(0, 2), rep((1 - 
        muMC), 2)))
      
      polygon(x0VApoly, col = colVA, density = 5)
      polygon(x0MCpoly, col = colMC, density = 5, angle = 135)
      
      x1VApoly <- list(x = c(xlims1, rev(xlims1)), y = c(rep(0, 2), rep(muVA, 
        2)))
      x1MCpoly <- list(x = c(xlims1, rev(xlims1)), y = c(rep(0, 2), rep(muMC, 
        2)))
      
      polygon(x1VApoly, col = colMC, density = 15)
      polygon(x1MCpoly, col = , density = 15, angle = 135)
      
      lines(c(xlims0[1], xlims1[2]), rep(0, 2))
      
      if (plotAccuracy) {
        text(0.1 * min(xlimVal) + 0.9 * max(xlimVal), 0.1 * min(ylimVal) + 
          0.9 * max(ylimVal), as.character(paste(accurVal, "%", sep = "")), 
          cex = cex.dist, col = colAcc)
        text(0.1 * min(xlimVal) + 0.9 * max(xlimVal), 0.3 * min(ylimVal) + 
          0.7 * max(ylimVal), "accuracy", cex = 0.5 * cex.dist, col = colAcc)
      } else {
        legend("topright", c("Infer.NET", "MCMC"), lty = c(1, 1), lwd = c(3, 
          3), col = c(colVA, colMC), cex = 1)
      }
    }
    
  }
  
  if (type == "Discrete") {
    VAatoms <- parVec[[1]]
    VAprobs <- parVec[[2]]
    VAsample <- sample(VAatoms, 1e+05, replace = TRUE, prob = VAprobs)
    
    atomHist <- intersect(signif(VAatoms, 3), signif(unique(MCsample), 3))
    gapVal <- min(diff(atomHist))
    
    breaksLow <- min(atomHist) + 0.5 * gapVal
    breaksUpp <- max(atomHist) - 0.5 * gapVal
    
    breaksLowEnough <- FALSE
    while (!breaksLowEnough) {
      if (breaksLow < min(min(MCsample), min(VAsample))) 
        breaksLowEnough <- TRUE
      breaksLow <- breaksLow - gapVal
    }
    
    breaksUppEnough <- FALSE
    while (!breaksUppEnough) {
      if (breaksUpp > max(max(MCsample), max(VAsample))) 
        breaksUppEnough <- TRUE
      breaksUpp <- breaksUpp + gapVal
    }
    
    breaksHist <- seq(breaksLow, breaksUpp, by = gapVal)
    
    histObjMC <- hist(MCsample, breaks = breaksHist, plot = FALSE)
    histObjVA <- hist(VAsample, breaks = breaksHist, plot = FALSE)
    
    densMC <- histObjMC$counts/sum(histObjMC$counts)
    densVA <- histObjVA$counts/sum(histObjVA$counts)
    histObjMC$counts <- densMC
    histObjVA$counts <- densVA
    
    ylimVal <- range(c(densMC, densVA))
    
    if (plotDensities) {
      plot(histObjMC, density = 20, col = colMC, border = colMC, xlim = c(xgLow, 
        xgUpp), ylim = ylimVal, ylab = "probability", xlab = "", main = parName, 
        cex.main = cex.mainVal, cex.lab = cex.labVal, )
      
      plot(histObjVA, density = 20, angle = 135, col = colVA, border = colVA, 
        ylab = "probability", xlim = c(xgLow, xgUpp), ylim = ylimVal, add = plotDensities)
    }
    
    accurVal <- round(100 - 50 * sum(abs(densMC - densVA)))
    
    if (plotAccuracy) {
      text(0.15 * min(xlimVal) + 0.85 * max(xlimVal), 0.1 * min(ylimVal) + 
        0.9 * max(ylimVal), as.character(paste(accurVal, "%", sep = "")), 
        cex = cex.dist, col = colAcc)
      text(0.15 * min(xlimVal) + 0.85 * max(xlimVal), 0.3 * min(ylimVal) + 
        0.7 * max(ylimVal), "accuracy", cex = 0.5 * cex.dist, col = colAcc)
      if (!is.null(parTrue)) 
        lines(rep(parTrue, 2), c(0, ylimVal[2]), col = colTruth)
      
    } else {
      legend("topright", c("Infer.NET", "MCMC"), lty = c(1, 1), lwd = c(3, 
        3), col = c(colVA, colMC), cex = 1)
    }
    
  }
  
  yPos <- -0.2 * diff(ylimVal)
  text(mean(xg), yPos, xlabVal, cex = cex.labVal, xpd = T)
  
  return(accurVal)
}



trapint <- function(xgrid, fgrid) {
  ng <- length(xgrid)
  xvec <- xgrid[2:ng] - xgrid[1:(ng - 1)]
  fvec <- fgrid[1:(ng - 1)] + fgrid[2:ng]
  integ <- sum(xvec * fvec)/2
  return(integ)
}

########### End of accVarApp ############


########## R-function: summMCMC ##########

# For compact graphical summary of MCMC output.  This version handles multiple
# chain input and does a Brooks-Gelman-Rubin plot if there is more than one
# chain.

# Last changed: 13 AUG 2010 by M.P.Wand.

summMCMC <- function(xList, EPSfileName, PDFfileName, PNGfileName, plotInd = 1, parNames, 
  columnHeadings, colourVersion = TRUE, columnCols, credLevel = 0.95, numerSummCex = 1.3, 
  BGRsttPos = 10, BGRyRange = c(0.95, 1.25), BGRtickPos = 1.2, BGRlogTransf, BGRlogitTransf, 
  KDExlim, KDEvertLine = TRUE, KDEvertLineCol = "black", addTruthToKDE = NULL, 
  addBaseLine = FALSE) {
  library(KernSmooth)
  options(warn = -1)
  x <- lapply(xList, as.matrix)
  num.par <- ncol(x[[plotInd]])
  samp.size <- nrow(x[[plotInd]])
  num.chains <- length(x)
  
  # Divert figure to EPS, PDF or PNG file if filename specified.
  
  if (!missing(EPSfileName)) 
    postscript(EPSfileName, horizontal = FALSE, width = 11, height = (num.par + 
      1))
  
  if (!missing(PDFfileName)) 
    pdf(PDFfileName, width = 9, height = (num.par + 1))
  
  if (!missing(PNGfileName)) 
    png(PNGfileName, width = 630, height = 70 * (num.par + 1))
  
  if (missing(PNGfileName)) 
    op <- par()
  if (missing(columnHeadings)) 
    columnHeadings <- c("parameter", "trace", "lag 1", "acf", "BGR", "density", 
      "summary")
  
  if (colourVersion) {
    
    if (missing(columnCols)) {
      columnCols <- c("darkmagenta", "green4", "darkorange", "dodgerblue", 
        "darkgoldenrod1", "red", "navy")
      if (KDEvertLine & missing(KDEvertLineCol)) 
        KDEvertLineCol <- "DarkGreen"
    }
  }
  
  panels.per.par <- length(columnHeadings)
  if (num.chains == 1) 
    panels.per.par <- panels.per.par - 1
  
  if (!colourVersion) {
    columnCols <- rep(NA, num.par)
    for (i in 1:length(columnHeadings)) columnCols[i] <- "black"
  }
  
  
  empty.panel <- function(addBaseLineFlag = addBaseLine) {
    plot(0, 0, type = "n", xlim = c(0, 1), ylim = c(0, 1), xaxt = "n", yaxt = "n", 
      xlab = "", ylab = "", bty = "o")
    if (addBaseLineFlag) 
      lines(c(-0.05, 1.05), rep(-0.01, 2), lwd = 1)
    invisible()
  }
  
  par(mfrow = c((num.par + 1), panels.per.par))
  
  headInds <- 1:7
  if (num.chains == 1) 
    headInds <- c(1:4, 6, 7)
  for (i in 1:panels.per.par) {
    par(ann = F, mar = rep(0, 4), xaxt = "n", yaxt = "n", xpd = TRUE)
    empty.panel()
    text(0.5, 0.5, columnHeadings[headInds[i]], cex = 2.2, col = columnCols[headInds[i]])
  }
  
  for (j in 1:num.par) {
    # Write the variable name
    
    par(ann = F, mar = c(0, 0, 0, 0), xaxt = "n", yaxt = "n")
    
    
    empty.panel(addBaseLineFlag = ((j == num.par) & addBaseLine))
    if (length(parNames[[j]]) == 1) 
      text(0.5, 0.5, parNames[[j]][1], cex = 3, col = columnCols[1])
    if (length(parNames[[j]]) == 2) {
      text(0.5, 0.7, parNames[[j]][1], cex = 1.9, col = columnCols[1])
      text(0.5, 0.3, parNames[[j]][2], cex = 1.9, col = columnCols[1])
    }
    if (length(parNames[[j]]) == 3) {
      text(0.5, 0.8, parNames[[j]][1], cex = 1, col = columnCols[1])
      text(0.5, 0.5, parNames[[j]][2], cex = 1, col = columnCols[1])
      text(0.5, 0.2, parNames[[j]][3], cex = 1, col = columnCols[1])
    }
    
    # Do the trace plot
    
    plot(x[[plotInd]][, j], xlab = "", ylab = "", type = "l", col = columnCols[2])
    
    if ((!missing(PNGfileName)) & (j == num.par) & addBaseLine) {
      xVec <- 1:length(x[[plotInd]][, j])
      yVec <- x[[plotInd]][, j]
      xLow <- 1.05 * min(xVec) - 0.05 * max(xVec)
      xUpp <- 1.05 * max(xVec) - 0.05 * min(xVec)
      yLow <- 1.01 * min(yVec) - 0.01 * max(yVec)
      lines(c(xLow, xUpp), rep(yLow, 2), lwd = 2, col = "black")
    }
    
    # Do the lag 1 plot
    
    plot(x[[plotInd]][1:(samp.size - 1), j], x[[plotInd]][2:samp.size, j], xlab = "", 
      ylab = "", type = "n")
    
    points(x[[plotInd]][1:(samp.size - 1), j], x[[plotInd]][2:samp.size, j], 
      pch = 1, cex = 0.5, col = columnCols[3])
    
    
    if ((!missing(PNGfileName)) & (j == num.par) & addBaseLine) {
      xVec <- x[[plotInd]][1:(samp.size - 1), j]
      yVec <- x[[plotInd]][2:samp.size, j]
      xLow <- 1.05 * min(xVec) - 0.05 * max(xVec)
      xUpp <- 1.05 * max(xVec) - 0.05 * min(xVec)
      yLow <- 1.01 * min(yVec) - 0.01 * max(yVec)
      lines(c(xLow, xUpp), rep(yLow, 2), lwd = 2, col = "black")
    }
    
    # Do the ACF plot
    
    ci.col.val <- "black"
    if (colourVersion) 
      ci.col.val <- "blue"
    acf(x[[plotInd]][, j], lag.max = 20, col = columnCols[4], lwd = 2, ci.col = ci.col.val)
    
    if ((!missing(PNGfileName)) & (j == num.par) & addBaseLine) {
      acfVec <- as.numeric(acf(x[[plotInd]][, j], lag.max = 20, plot = FALSE)$acf)
      nACF <- length(x[[plotInd]][, j])
      CIlow <- -2/sqrt(nACF)
      yVec <- c(acfVec, CIlow)
      xVec <- 0:20
      xLow <- 1.05 * min(xVec) - 0.05 * max(xVec)
      xUpp <- 1.05 * max(xVec) - 0.05 * min(xVec)
      yLow <- 1.01 * min(yVec) - 0.01 * max(yVec)
      lines(c(xLow, xUpp), rep(yLow, 2), lwd = 2, col = "black")
    }
    
    if (num.chains > 1) {
      # Multiple chains.  Do the Gelman-Rubin plot
      
      x.list <- list(x[[1]][, j])
      for (k in 2:num.chains) x.list <- c(x.list, list(x[[k]][, j]))
      
      if ((!missing(BGRlogTransf))) {
        if (any(j == BGRlogTransf)) {
          x.list <- list(log(x[[1]][, j]))
          for (k in 2:num.chains) x.list <- c(x.list, list(log(x[[k]][, j])))
        }
      }
      if ((!missing(BGRlogitTransf))) {
        if (any(j == BGRlogitTransf)) {
          x.list <- list(log(x[[1]][, j]))
          for (k in 2:num.chains) x.list <- c(x.list, list(logit(x[[k]][, 
          j])))
        }
      }
      
      BGR.outp <- BGRinterval(x.list, BGRsttPos)
      
      # Extract x-axis and y-axis components of `BGR.outp'.
      
      BGR.x <- BGR.outp$x
      BGR.y <- BGR.outp$numer/BGR.outp$denom
      
      lb.wt <- 0.2 * length(BGR.x)
      plot(0, 0, type = "n", xlim = c(-lb.wt, length(BGR.x)), ylim = BGRyRange, 
        xlab = "", ylab = "", bty = "l")
      lines(BGR.x, BGR.y, lwd = 1, col = columnCols[5], err = -1)
      lines(rep(0, 2), BGRyRange, err = -1)
      
      text(-0.75 * lb.wt, BGRtickPos, as.character(BGRtickPos), srt = 90, cex = 0.8)
      text(-0.75 * lb.wt, 1, "1.00", srt = 90, cex = 0.8)
      abline(h = 1, lwd = 1)
      lines(c(-lb.wt/2, 0), rep(BGRtickPos, 2))
    }
    
    # Do the density plot
    
    h <- dpik(x[[plotInd]][, j])
    est <- bkde(x[[plotInd]][, j], bandwidth = h)
    
    if (!missing(KDExlim)) 
      xlim.val <- KDExlim[[j]]
    
    if (missing(KDExlim)) 
      xlim.val <- range(est$x)
    
    lb.ht <- 0.2 * (max(est$y) - min(est$y))
    ylim.val <- c(-lb.ht, 1.1 * max(est$y))
    plot(est, type = "l", xlab = "", ylab = "", ylim = ylim.val, col = columnCols[6], 
      xlim = xlim.val, lwd = 2)
    lines(c(min(est$x), max(est$x)), rep(0, 2))
    
    if (KDEvertLine) 
      lines(rep(0, 2), c(0, max(est$y)), lwd = 3, err = -1, col = KDEvertLineCol)
    
    if (!is.null(addTruthToKDE)) 
      lines(rep(addTruthToKDE[j], 2), c(0, max(est$y)), lwd = 1, lty = 2, , 
        err = -1, col = KDEvertLineCol)
    
    
    x.labels <- pretty(x[[plotInd]][, j], n = 3)
    for (i in 1:length(x.labels)) {
      text(x.labels[i], -0.6 * lb.ht, as.character(x.labels[i]), cex = 0.8)
      lines(rep(x.labels[i], 2), c(0, -0.1 * lb.ht))
    }
    
    if ((!missing(PNGfileName)) & (j == num.par) & addBaseLine) {
      xVec <- xlim.val
      yVec <- ylim.val
      xLow <- 1.05 * min(xVec) - 0.05 * max(xVec)
      xUpp <- 1.05 * max(xVec) - 0.05 * min(xVec)
      yLow <- 1.01 * min(yVec) - 0.01 * max(yVec)
      lines(c(xLow, xUpp), rep(yLow, 2), lwd = 2, col = "black")
    }
    
    empty.panel(addBaseLine = (j == num.par))
    text(0.5, 0.75, paste("posterior mean: ", as.character(signif(mean(x[[plotInd]][, 
      j]), 3)), sep = ""), col = columnCols[7], cex = numerSummCex)
    text(0.5, 0.5, paste(100 * credLevel, "% credible interval: ", sep = ""), 
      col = columnCols[7], cex = numerSummCex)
    
    text(0.5, 0.25, paste("(", as.character(signif(quantile(x[[plotInd]][, j], 
      (1 - credLevel)/2), 3)), ",", as.character(signif(quantile(x[[plotInd]][, 
      j], (1 + credLevel)/2), 3)), ")", sep = ""), col = columnCols[7], cex = numerSummCex)
  }
  
  if (missing(PNGfileName)) 
    par(op)
  invisible()
}

########## End of summMCMC ##########
 
