########## R function: ZOSull ##########

# For creation of O'Sullivan-type Z matrices.

ZOSull <- function(x, range.x, intKnots, drv = 0) {
  # Check legality of range.x.
  
  if (!missing(range.x)) {
    if (length(range.x) != 2) 
      stop("range.x must be of length 2.")
    if (range.x[1] > range.x[2]) 
      stop("range.x[1] exceeds range.x[1].")
    if (range.x[1] > min(x)) 
      stop("range.x[1] must be <= than min(x).")
    if (range.x[2] < max(x)) 
      stop("range.x[2] must be >= than max(x).")
    
  }
  
  if (drv > 2) 
    stop("splines not smooth enough for more than 2 derivatives")
  
  library("splines")
  
  # Set defaults for `range.x' and `intKnots'
  
  if (missing(range.x)) 
    range.x <- c(1.05 * min(x) - 0.05 * max(x), 1.05 * max(x) - 0.05 * min(x))
  
  if (missing(intKnots)) {
    numIntKnots <- min(length(unique(x)), 35)
    intKnots <- quantile(unique(x), seq(0, 1, length = (numIntKnots + 2))[-c(1, 
      (numIntKnots + 2))])
  }
  numIntKnots <- length(intKnots)
  
  # Obtain the penalty matrix.
  
  allKnots <- c(rep(range.x[1], 4), intKnots, rep(range.x[2], 4))
  K <- length(intKnots)
  L <- 3 * (K + 8)
  xtilde <- (rep(allKnots, each = 3)[-c(1, (L - 1), L)] + rep(allKnots, each = 3)[-c(1, 
    2, L)])/2
  wts <- rep(diff(allKnots), each = 3) * rep(c(1, 4, 1)/6, K + 7)
  Bdd <- spline.des(allKnots, xtilde, derivs = rep(2, length(xtilde)), outer.ok = TRUE)$design
  Omega <- t(Bdd * wts) %*% Bdd
  
  # Use the spectral decomposition of Omega to obtain Z.
  
  svdOmega <- svd(Omega)
  indsZ <- 1:(numIntKnots + 2)
  UZ <- svdOmega$u[, indsZ]
  LZ <- t(t(UZ)/sqrt(svdOmega$d[indsZ]))
  
  # Perform stability check.
  
  indsX <- (numIntKnots + 3):(numIntKnots + 4)
  UX <- svdOmega$u[, indsX]
  L <- cbind(UX, LZ)
  stabCheck <- t(crossprod(L, t(crossprod(L, Omega))))
  if (sum(stabCheck^2) > 1.0001 * (numIntKnots + 2)) 
    print("WARNING: NUMERICAL INSTABILITY ARISING\\
        FROM SPECTRAL DECOMPOSITION")
  
  # Obtain B and post-multiply by LZ matrix to get Z.
  
  B <- spline.des(allKnots, x, derivs = rep(drv, length(x)), outer.ok = TRUE)$design
  
  Z <- B %*% LZ
  
  # Add the `range.x' and 'intKnots' as attributes of the return object.
  
  attr(Z, "range.x") <- range.x
  attr(Z, "intKnots") <- intKnots
  
  # Return Z matrix with 2 attributes.
  
  return(Z)
}

########## End of ZOSull ##########

########## R function: Ztps ##########

# For creation of thin plate spline-type Z matrices.

Ztps <- function(x, knots) {
  # Set up thin plate spline generalised covariance function:
  
  tps.cov <- function(r, m = 2, d = 1) {
    r <- as.matrix(r)
    num.row <- nrow(r)
    num.col <- ncol(r)
    r <- as.vector(r)
    nzi <- (1:length(r))[r != 0]
    ans <- rep(0, length(r))
    if ((d + 1)%%2 != 0) 
      ans[nzi] <- (abs(r[nzi]))^(2 * m - d) * log(abs(r[nzi]))  # d is even
 else ans[nzi] <- (abs(r[nzi]))^(2 * m - d)
    
    if (num.col > 1) 
      ans <- matrix(ans, num.row, num.col)  # d is odd
    return(ans)
  }
  
  # Set up function for matrix square-roots:
  
  matrix.sqrt <- function(A) {
    sva <- svd(A)
    if (min(sva$d) >= 0) 
      Asqrt <- t(sva$v %*% (t(sva$u) * sqrt(sva$d))) else stop("Matrix square root is not defined")
    return(Asqrt)
  }
  
  # Obtain matrix of inter-knot distances:
  
  numKnots <- nrow(knots)
  
  dist.mat <- matrix(0, numKnots, numKnots)
  dist.mat[lower.tri(dist.mat)] <- dist(as.matrix(knots))
  dist.mat <- dist.mat + t(dist.mat)
  
  Omega <- tps.cov(dist.mat, d = 2)
  
  # Obtain preliminary Z matrix of knot to data covariances:
  
  x.knot.diffs.1 <- outer(x[, 1], knots[, 1], "-")
  x.knot.diffs.2 <- outer(x[, 2], knots[, 2], "-")
  x.knot.dists <- sqrt(x.knot.diffs.1^2 + x.knot.diffs.2^2)
  
  prelim.Z <- tps.cov(x.knot.dists, m = 2, d = 2)
  
  # Transform to canonical form:
  
  sqrt.Omega <- matrix.sqrt(Omega)
  Z <- t(solve(sqrt.Omega, t(prelim.Z)))
  
  return(Z)
}


########## End of Ztps ##########

########## R function: save.default.knots.2D ##########

# Overwrites the save.default.knots.2D function from library("SemiPar")
# bypassing the user's need to enter a filename for the knots.  

save.default.knots.2D <- function(x1, x2, num.knots, knots.filename) {
  if (missing(num.knots)) 
    num.knots <- max(10, min(50, round(length(x1)/4)))
  X <- cbind(x1, x2)
  dup.inds <- (1:nrow(X))[dup.matrix(X) == TRUE]
  if (length(dup.inds) > 0) 
    X <- X[-dup.inds, ]
  knots <- cluster::clara(X, num.knots)$medoids
  plot(x1, x2, xlab = "", ylab = "", bty = "n", pch = 1, xaxt = "n", yaxt = "n")
  points(knots[, 1], knots[, 2], col = "red", lwd = 3, cex = 2)
  dev.off()
  write(t(knots), file = knots.filename, ncolumns = 2)
  return(knots)
}

########## End of save.default.knots.2D ##########
 
